# Autogenerated file. Changes will be overwritten.

try:
    import coriolis
except ImportError:
    import CRL, Hurricane, Viewer, Cfg
    from Hurricane import (
        Technology, DataBase, DbU, Library,
        Layer, BasicLayer,
        Cell, Net, Horizontal, Vertical, Rectilinear, Box, Point,
        Instance, Transformation,
        NetExternalComponents,
    )
    from helpers import u, l
    from helpers.technology import setEnclosures
    from helpers.overlay import CfgCache, UpdateSession
else:
    from coriolis import CRL, Hurricane, Viewer, Cfg
    from coriolis.Hurricane import (
        Technology, DataBase, DbU, Library,
        Layer, BasicLayer,
        Cell, Net, Horizontal, Vertical, Rectilinear, Box, Point,
        Instance, Transformation,
        NetExternalComponents,
    )
    from coriolis.helpers import u, l
    from coriolis.helpers.technology import setEnclosures
    from coriolis.helpers.overlay import CfgCache, UpdateSession

__all__ = ["setup"]

def createRL(tech, net, layer, coords):
    coords = [Point(u(x), u(y)) for x,y in coords]
    Rectilinear.create(net, tech.getLayer(layer), coords)

def _routing():
    af = CRL.AllianceFramework.get()
    db = DataBase.getDB()
    tech = db.getTechnology()

    rg = CRL.RoutingGauge.create('StdCellLib')
    rg.setSymbolic(False)
    metal = tech.getLayer('li')
    via = tech.getLayer('li_mcon_m1')
    setEnclosures(via, metal, (u(0.0), u(0.075)))
    rg.addLayerGauge(CRL.RoutingLayerGauge.create(
        metal, CRL.RoutingLayerGauge.Vertical, CRL.RoutingLayerGauge.PinOnly, 0, 0.0,
        u(0.0), u(0.51), u(0.17), u(0.17), u(0.17), u(0.17),
    ))
    metal = tech.getLayer('m1')
    via = tech.getLayer('li_mcon_m1')
    setEnclosures(via, metal, (u(0.03), u(0.075)))
    via = tech.getLayer('m1_via_m2')
    setEnclosures(via, metal, (u(0.11), u(0.085)))
    rg.addLayerGauge(CRL.RoutingLayerGauge.create(
        metal, CRL.RoutingLayerGauge.Horizontal, CRL.RoutingLayerGauge.Default, 1, 0.0,
        u(0.0), u(0.46), u(0.32), u(0.23), u(0.15), u(0.14),
    ))
    metal = tech.getLayer('m2')
    via = tech.getLayer('m1_via_m2')
    setEnclosures(via, metal, (u(0.11), u(0.085)))
    via = tech.getLayer('m2_via2_m3')
    setEnclosures(via, metal, (u(0.085), u(0.09)))
    rg.addLayerGauge(CRL.RoutingLayerGauge.create(
        metal, CRL.RoutingLayerGauge.Vertical, CRL.RoutingLayerGauge.Default, 2, 0.0,
        u(0.0), u(0.51), u(0.37), u(0.26), u(0.2), u(0.14),
    ))
    metal = tech.getLayer('m3')
    via = tech.getLayer('m2_via2_m3')
    setEnclosures(via, metal, (u(0.085), u(0.09)))
    via = tech.getLayer('m3_via3_m4')
    setEnclosures(via, metal, (u(0.49), u(0.09)))
    rg.addLayerGauge(CRL.RoutingLayerGauge.create(
        metal, CRL.RoutingLayerGauge.Horizontal, CRL.RoutingLayerGauge.Default, 3, 0.0,
        u(0.0), u(0.68), u(0.38), u(0.33), u(0.2), u(0.3),
    ))
    metal = tech.getLayer('m4')
    via = tech.getLayer('m3_via3_m4')
    setEnclosures(via, metal, (u(0.49), u(0.09)))
    via = tech.getLayer('m4_via4_m5')
    setEnclosures(via, metal, (u(0.19), u(0.4)))
    rg.addLayerGauge(CRL.RoutingLayerGauge.create(
        metal, CRL.RoutingLayerGauge.Vertical, CRL.RoutingLayerGauge.Default, 4, 0.0,
        u(0.0), u(1.48), u(1.18), u(0.33), u(0.8), u(0.3),
    ))
    metal = tech.getLayer('m5')
    via = tech.getLayer('m4_via4_m5')
    setEnclosures(via, metal, (u(0.31), u(0.4)))
    rg.addLayerGauge(CRL.RoutingLayerGauge.create(
        metal, CRL.RoutingLayerGauge.Horizontal, CRL.RoutingLayerGauge.Default, 5, 0.0,
        u(0.0), u(3.2), u(1.6), u(1.6), u(0.8), u(1.6),
    ))
    af.addRoutingGauge(rg)
    af.setRoutingGauge('StdCellLib')

    cg = CRL.CellGauge.create(
        'StdCellLib', 'm1',
        u(1.0), u(10.0), u(1.0),
    )
    af.addCellGauge(cg)
    af.setCellGauge('StdCellLib')

    # Place & Route setup
    with CfgCache(priority=Cfg.Parameter.Priority.ConfigurationFile) as cfg:
        cfg.lefImport.minTerminalWidth = 0.0
        cfg.crlcore.groundName = 'vss'
        cfg.crlcore.powerName = 'vdd'
        cfg.etesian.aspectRatio = 1.00
        cfg.etesian.aspectRatio = [10, 1000]
        cfg.etesian.spaceMargin = 0.10
        cfg.etesian.uniformDensity = True
        cfg.etesian.routingDriven = False
        cfg.etesian.latchUpDistance = u(30.0 - 1.0)
        cfg.etesian.diodeName = 'diode_w1'
        cfg.etesian.antennaInsertThreshold = 0.50
        cfg.etesian.antennaMaxWL = u(250.0)
        cfg.etesian.feedNames = 'tie,decap_w0'
        cfg.etesian.cell.zero = 'zero_x1'
        cfg.etesian.cell.one = 'one_x1'
        cfg.etesian.bloat = 'disabled'
        cfg.etesian.effort = 2
        cfg.etesian.effort = (
            ('Fast', 1),
            ('Standard', 2),
            ('High', 3 ),
            ('Extreme', 4 ),
        )
        cfg.etesian.graphics = 2
        cfg.etesian.graphics = (
            ('Show every step', 1),
            ('Show lower bound', 2),
            ('Show result only', 3),
        )
        cfg.anabatic.routingGauge = 'StdCellLib'
        cfg.anabatic.globalLengthThreshold = 1450
        cfg.anabatic.saturateRatio = 0.90
        cfg.anabatic.saturateRp = 10
        cfg.anabatic.topRoutingLayer = 'm5'
        cfg.anabatic.edgeLength = 48
        cfg.anabatic.edgeWidth = 8
        cfg.anabatic.edgeCostH = 9.0
        cfg.anabatic.edgeCostK = -10.0
        cfg.anabatic.edgeHInc = 1.0
        cfg.anabatic.edgeHScaling = 1.0
        cfg.anabatic.globalIterations = 10
        cfg.anabatic.globalIterations = [ 1, 100 ]
        cfg.anabatic.gcell.displayMode = 1
        cfg.anabatic.gcell.displayMode = (("Boundary", 1), ("Density", 2))
        cfg.katana.hTracksReservedLocal = 4
        cfg.katana.hTracksReservedLocal = [0, 20]
        cfg.katana.vTracksReservedLocal = 3
        cfg.katana.vTracksReservedLocal = [0, 20]
        cfg.katana.termSatReservedLocal = 8
        cfg.katana.termSatThreshold = 9
        cfg.katana.eventsLimit = 4000002
        cfg.katana.ripupCost = 3
        cfg.katana.ripupCost = [0, None]
        cfg.katana.strapRipupLimit = 16
        cfg.katana.strapRipupLimit = [1, None]
        cfg.katana.localRipupLimit = 9
        cfg.katana.localRipupLimit = [1, None]
        cfg.katana.globalRipupLimit = 5
        cfg.katana.globalRipupLimit = [1, None]
        cfg.katana.longGlobalRipupLimit = 5
        cfg.chip.padCoreSide = 'South'

    # Plugins setup
    with CfgCache(priority=Cfg.Parameter.Priority.ConfigurationFile) as cfg:
        cfg.viewer.minimumSize = 500
        cfg.viewer.pixelThreshold = 10
        cfg.chip.block.rails.count = 5
        cfg.chip.block.rails.hWidth = u(2.68)
        cfg.chip.block.rails.vWidth = u(2.68)
        cfg.chip.block.rails.hSpacing = u(0.7)
        cfg.chip.block.rails.vSpacing = u(0.7)
        cfg.clockTree.minimumSide = l(600)
        cfg.clockTree.buffer = 'buf_x1'
        cfg.clockTree.placerEngine = 'Etesian'
        cfg.block.spareSide = 10
        cfg.spares.buffer = 'buf_x4'
        cfg.spares.maxSinks = 31


def _load():
    af = CRL.AllianceFramework.get()
    db = DataBase.getDB()
    tech = db.getTechnology()
    rootlib = db.getRootLibrary()

    lib = Library.create(rootlib, 'StdCellLib')
    new_cells = {
        'a2_x2': Cell.create(lib, 'a2_x2'),
        'a3_x2': Cell.create(lib, 'a3_x2'),
        'a4_x2': Cell.create(lib, 'a4_x2'),
        'ao22_x2': Cell.create(lib, 'ao22_x2'),
        'mx2_x2': Cell.create(lib, 'mx2_x2'),
        'mx3_x2': Cell.create(lib, 'mx3_x2'),
        'nsnrlatch_x1': Cell.create(lib, 'nsnrlatch_x1'),
        'nxr2_x1': Cell.create(lib, 'nxr2_x1'),
        'o2_x2': Cell.create(lib, 'o2_x2'),
        'o3_x2': Cell.create(lib, 'o3_x2'),
        'o4_x2': Cell.create(lib, 'o4_x2'),
        'oa22_x2': Cell.create(lib, 'oa22_x2'),
        'powmid_x0': Cell.create(lib, 'powmid_x0'),
        'sff1_x4': Cell.create(lib, 'sff1_x4'),
        'sff1r_x4': Cell.create(lib, 'sff1r_x4'),
        'xr2_x1': Cell.create(lib, 'xr2_x1'),
        'fill': Cell.create(lib, 'fill'),
        'tie': Cell.create(lib, 'tie'),
        'tie_diff': Cell.create(lib, 'tie_diff'),
        'tie_poly': Cell.create(lib, 'tie_poly'),
        'fill_w2': Cell.create(lib, 'fill_w2'),
        'tie_w2': Cell.create(lib, 'tie_w2'),
        'tie_diff_w2': Cell.create(lib, 'tie_diff_w2'),
        'tie_poly_w2': Cell.create(lib, 'tie_poly_w2'),
        'fill_w4': Cell.create(lib, 'fill_w4'),
        'tie_w4': Cell.create(lib, 'tie_w4'),
        'tie_diff_w4': Cell.create(lib, 'tie_diff_w4'),
        'tie_poly_w4': Cell.create(lib, 'tie_poly_w4'),
        'diode_w1': Cell.create(lib, 'diode_w1'),
        'zero_x1': Cell.create(lib, 'zero_x1'),
        'one_x1': Cell.create(lib, 'one_x1'),
        'zeroone_x1': Cell.create(lib, 'zeroone_x1'),
        'decap_w0': Cell.create(lib, 'decap_w0'),
        'inv_x0': Cell.create(lib, 'inv_x0'),
        'inv_x1': Cell.create(lib, 'inv_x1'),
        'inv_x2': Cell.create(lib, 'inv_x2'),
        'inv_x4': Cell.create(lib, 'inv_x4'),
        'buf_x1': Cell.create(lib, 'buf_x1'),
        'buf_x2': Cell.create(lib, 'buf_x2'),
        'buf_x4': Cell.create(lib, 'buf_x4'),
        'nand2_x0': Cell.create(lib, 'nand2_x0'),
        'nand3_x0': Cell.create(lib, 'nand3_x0'),
        'nand4_x0': Cell.create(lib, 'nand4_x0'),
        'nor2_x0': Cell.create(lib, 'nor2_x0'),
        'nor3_x0': Cell.create(lib, 'nor3_x0'),
        'nor4_x0': Cell.create(lib, 'nor4_x0'),
    }

    cell = new_cells['a2_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((2.375,5.5),(2.375,6.5),(1.925,6.5),(1.925,8.5),(2.375,8.5),(2.375,9.5),(3.075,9.5),(3.075,5.5),(2.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,9.305),(0.625,9.305),(0.625,8.5),(1.075,8.5),(1.075,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,8.995),(1.375,9.0),(1.3,9.0),(1.3,9.4),(1.375,9.4),(1.375,9.405),(1.625,9.405),(1.625,9.4),(1.7,9.4),(1.7,9.0),(1.625,9.0),(1.625,8.995),(1.375,8.995)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(4.3,10.8),(4.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.415,9.045),(2.415,9.215),(2.585,9.215),(2.585,9.045),(2.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.115),(1.415,9.285),(1.585,9.285),(1.585,9.115),(1.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.015),(0.415,9.185),(0.585,9.185),(0.585,9.015),(0.415,9.015)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((3.075,0.6),(3.075,2.5),(3.625,2.5),(3.625,0.6),(3.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,5.5),(3.075,9.5),(3.625,9.5),(3.625,5.5),(3.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,1.37),(3.415,8.63),(3.585,8.63),(3.585,1.37),(3.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,5.64),(3.415,5.81),(3.585,5.81),(3.585,5.64),(3.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,5.98),(3.415,6.15),(3.585,6.15),(3.585,5.98),(3.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.32),(3.415,6.49),(3.585,6.49),(3.585,6.32),(3.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.66),(3.415,6.83),(3.585,6.83),(3.585,6.66),(3.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.0),(3.415,7.17),(3.585,7.17),(3.585,7.0),(3.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.34),(3.415,7.51),(3.585,7.51),(3.585,7.34),(3.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.68),(3.415,7.85),(3.585,7.85),(3.585,7.68),(3.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.02),(3.415,8.19),(3.585,8.19),(3.585,8.02),(3.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.36),(3.415,8.53),(3.585,8.53),(3.585,8.36),(3.415,8.36)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.49),(3.415,1.66),(3.585,1.66),(3.585,1.49),(3.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.83),(3.415,2.0),(3.585,2.0),(3.585,1.83),(3.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.17),(3.415,2.34),(3.585,2.34),(3.585,2.17),(3.415,2.17)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((2.25,5.375),(2.25,6.375),(0.25,6.375),(0.25,9.43),(0.75,9.43),(0.75,8.625),(2.25,8.625),(2.25,9.625),(3.75,9.625),(3.75,5.375),(2.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.225,0.275),(0.225,1.025),(0.25,1.025),(0.25,1.03),(0.75,1.03),(0.75,1.025),(0.775,1.025),(0.775,0.275),(0.225,0.275)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,8.87),(1.25,8.875),(1.175,8.875),(1.175,9.525),(1.25,9.525),(1.25,9.53),(1.75,9.53),(1.75,9.525),(1.825,9.525),(1.825,8.875),(1.75,8.875),(1.75,8.87),(1.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.3,0.475),(1.3,1.425),(0.25,1.425),(0.25,3.575),(1.7,3.575),(1.7,2.625),(3.75,2.625),(3.75,0.475),(1.3,0.475)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'poly',
            ((2.925,0.47),(2.925,3.835),(1.465,3.835),(1.465,4.165),(2.925,4.165),(2.925,9.63),(3.075,9.63),(3.075,0.47),(2.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,6.5),(1.075,8.5),(1.925,8.5),(1.925,6.5),(1.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.55),(0.375,3.45),(0.925,3.45),(0.925,1.55),(0.375,1.55)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,3.345),(0.585,3.345),(0.585,2.085),(1.415,2.085),(1.415,8.63),(1.585,8.63),(1.585,4.165),(1.685,4.165),(1.685,3.835),(1.585,3.835),(1.585,1.915),(0.585,1.915),(0.585,1.37),(0.415,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((1.515,3.915),(1.515,4.085),(1.685,4.085),(1.685,3.915),(1.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.735),(0.415,1.905),(0.585,1.905),(0.585,1.735),(0.415,1.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.075),(0.415,2.245),(0.585,2.245),(0.585,2.075),(0.415,2.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.415),(0.415,2.585),(0.585,2.585),(0.585,2.415),(0.415,2.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.755),(0.415,2.925),(0.585,2.925),(0.585,2.755),(0.415,2.755)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,3.095),(0.415,3.265),(0.585,3.265),(0.585,3.095),(0.415,3.095)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,6.735),(1.415,6.905),(1.585,6.905),(1.585,6.735),(1.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.075),(1.415,7.245),(1.585,7.245),(1.585,7.075),(1.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.415),(1.415,7.585),(1.585,7.585),(1.585,7.415),(1.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.755),(1.415,7.925),(1.585,7.925),(1.585,7.755),(1.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,8.095),(1.415,8.265),(1.585,8.265),(1.585,8.095),(1.415,8.095)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.35,0.4),(0.35,0.9),(0.375,0.9),(0.375,0.905),(0.625,0.905),(0.625,0.9),(0.65,0.9),(0.65,0.4),(0.35,0.4)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,1.55),(0.925,3.45),(1.075,3.45),(1.075,1.55),(0.925,1.55)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,0.6),(1.925,2.5),(3.075,2.5),(3.075,0.6),(1.925,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.415,0.615),(0.415,0.785),(0.585,0.785),(0.585,0.615),(0.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.835),(2.415,1.005),(2.585,1.005),(2.585,0.835),(2.415,0.835)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((1.425,0.6),(1.425,1.55),(1.075,1.55),(1.075,3.45),(1.575,3.45),(1.575,2.5),(1.925,2.5),(1.925,0.6),(1.425,0.6)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.925,1.42),(0.925,4.835),(0.465,4.835),(0.465,5.165),(0.925,5.165),(0.925,8.63),(1.075,8.63),(1.075,1.42),(0.925,1.42)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,3.515),(0.415,8.63),(0.585,8.63),(0.585,5.165),(0.685,5.165),(0.685,4.835),(0.585,4.835),(0.585,3.515),(0.415,3.515)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(3.515), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(3.515), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,4.915),(0.515,5.085),(0.685,5.085),(0.685,4.915),(0.515,4.915)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.925,4.835),(1.925,8.63),(2.075,8.63),(2.075,5.165),(2.535,5.165),(2.535,4.835),(1.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((1.925,0.47),(1.925,3.165),(2.535,3.165),(2.535,2.835),(2.075,2.835),(2.075,0.47),(1.925,0.47)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,1.37),(2.415,2.835),(2.315,2.835),(2.315,3.165),(2.415,3.165),(2.415,4.835),(2.315,4.835),(2.315,5.165),(2.415,5.165),(2.415,8.63),(2.585,8.63),(2.585,1.37),(2.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.315,2.915),(2.315,3.085),(2.485,3.085),(2.485,2.915),(2.315,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.315,4.915),(2.315,5.085),(2.485,5.085),(2.485,4.915),(2.315,4.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)

    cell = new_cells['a3_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(5.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((3.375,5.5),(3.375,6.5),(2.925,6.5),(2.925,8.5),(3.375,8.5),(3.375,9.5),(4.075,9.5),(4.075,5.5),(3.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,8.995),(2.375,9.0),(2.3,9.0),(2.3,9.4),(2.375,9.4),(2.375,9.405),(2.625,9.405),(2.625,9.4),(2.7,9.4),(2.7,9.0),(2.625,9.0),(2.625,8.995),(2.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,6.5),(0.925,8.5),(1.375,8.5),(1.375,9.305),(1.625,9.305),(1.625,8.5),(2.075,8.5),(2.075,6.5),(0.925,6.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(5.3,10.8),(5.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(5.0,10.0),(5.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,9.045),(3.415,9.215),(3.585,9.215),(3.585,9.045),(3.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,9.115),(2.415,9.285),(2.585,9.285),(2.585,9.115),(2.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.015),(1.415,9.185),(1.585,9.185),(1.585,9.015),(1.415,9.015)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((4.075,5.5),(4.075,9.5),(4.625,9.5),(4.625,5.5),(4.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.075,0.6),(4.075,2.5),(4.625,2.5),(4.625,0.6),(4.075,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,1.37),(4.415,8.63),(4.585,8.63),(4.585,1.37),(4.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((4.415,1.49),(4.415,1.66),(4.585,1.66),(4.585,1.49),(4.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.83),(4.415,2.0),(4.585,2.0),(4.585,1.83),(4.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,2.17),(4.415,2.34),(4.585,2.34),(4.585,2.17),(4.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.64),(4.415,5.81),(4.585,5.81),(4.585,5.64),(4.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.98),(4.415,6.15),(4.585,6.15),(4.585,5.98),(4.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.32),(4.415,6.49),(4.585,6.49),(4.585,6.32),(4.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.66),(4.415,6.83),(4.585,6.83),(4.585,6.66),(4.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.0),(4.415,7.17),(4.585,7.17),(4.585,7.0),(4.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.34),(4.415,7.51),(4.585,7.51),(4.585,7.34),(4.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.68),(4.415,7.85),(4.585,7.85),(4.585,7.68),(4.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.02),(4.415,8.19),(4.585,8.19),(4.585,8.02),(4.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.36),(4.415,8.53),(4.585,8.53),(4.585,8.36),(4.415,8.36)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((3.25,5.375),(3.25,6.375),(0.25,6.375),(0.25,8.625),(1.25,8.625),(1.25,9.43),(1.75,9.43),(1.75,8.625),(3.25,8.625),(3.25,9.625),(4.75,9.625),(4.75,5.375),(3.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((2.25,0.37),(2.25,0.375),(2.175,0.375),(2.175,1.025),(2.25,1.025),(2.25,1.03),(2.75,1.03),(2.75,1.025),(2.825,1.025),(2.825,0.375),(2.75,0.375),(2.75,0.37),(2.25,0.37)),
        )
        createRL(
            tech, net, 'nsdm',
            ((3.25,0.475),(3.25,1.375),(0.25,1.375),(0.25,3.625),(3.75,3.625),(3.75,2.625),(4.75,2.625),(4.75,0.475),(3.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((2.25,8.87),(2.25,8.875),(2.175,8.875),(2.175,9.525),(2.25,9.525),(2.25,9.53),(2.75,9.53),(2.75,9.525),(2.825,9.525),(2.825,8.875),(2.75,8.875),(2.75,8.87),(2.25,8.87)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'poly',
            ((3.925,0.47),(3.925,3.835),(3.465,3.835),(3.465,4.165),(3.925,4.165),(3.925,9.63),(4.075,9.63),(4.075,0.47),(3.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,6.5),(2.075,8.5),(2.925,8.5),(2.925,6.5),(2.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,3.5),(0.925,3.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,8.5),(0.925,8.5),(0.925,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.6),(0.415,2.33),(0.585,2.33),(0.585,2.085),(3.415,2.085),(3.415,7.915),(2.585,7.915),(2.585,7.67),(2.415,7.67),(2.415,7.915),(0.585,7.915),(0.585,7.67),(0.415,7.67),(0.415,8.4),(0.585,8.4),(0.585,8.085),(2.415,8.085),(2.415,8.4),(2.585,8.4),(2.585,8.085),(3.585,8.085),(3.585,4.165),(3.685,4.165),(3.685,3.835),(3.585,3.835),(3.585,1.915),(0.585,1.915),(0.585,1.6),(0.415,1.6)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.81),(2.415,7.98),(2.585,7.98),(2.585,7.81),(2.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,8.15),(2.415,8.32),(2.585,8.32),(2.585,8.15),(2.415,8.15)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.81),(0.415,7.98),(0.585,7.98),(0.585,7.81),(0.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.15),(0.415,8.32),(0.585,8.32),(0.585,8.15),(0.415,8.15)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.68),(0.415,1.85),(0.585,1.85),(0.585,1.68),(0.415,1.68)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.02),(0.415,2.19),(0.585,2.19),(0.585,2.02),(0.415,2.02)),
        )
        createRL(
            tech, net, 'licon',
            ((3.515,3.915),(3.515,4.085),(3.685,4.085),(3.685,3.915),(3.515,3.915)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((2.375,0.495),(2.375,0.5),(2.3,0.5),(2.3,0.9),(2.375,0.9),(2.375,0.905),(2.625,0.905),(2.625,0.9),(2.7,0.9),(2.7,0.5),(2.625,0.5),(2.625,0.495),(2.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,1.5),(1.925,3.5),(2.075,3.5),(2.075,1.5),(1.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,1.5),(0.925,3.5),(1.075,3.5),(1.075,1.5),(0.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,0.6),(3.375,1.5),(2.925,1.5),(2.925,3.5),(3.625,3.5),(3.625,2.5),(4.075,2.5),(4.075,0.6),(3.375,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(5.0,1.2),(5.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,0.835),(3.415,1.005),(3.585,1.005),(3.585,0.835),(3.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.615),(2.415,0.785),(2.585,0.785),(2.585,0.615),(2.415,0.615)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,3.835),(0.465,3.835),(0.465,4.165),(0.925,4.165),(0.925,8.63),(1.075,8.63),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,2.5),(0.415,7.5),(0.585,7.5),(0.585,4.165),(0.685,4.165),(0.685,3.835),(0.585,3.835),(0.585,2.5),(0.415,2.5)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(2.5), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(2.5), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,3.915),(0.515,4.085),(0.685,4.085),(0.685,3.915),(0.515,3.915)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((1.075,1.5),(1.075,3.5),(1.925,3.5),(1.925,1.5),(1.075,1.5)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((2.075,1.5),(2.075,3.5),(2.925,3.5),(2.925,1.5),(2.075,1.5)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,4.835),(1.465,4.835),(1.465,5.165),(1.925,5.165),(1.925,8.63),(2.075,8.63),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,2.255),(1.415,7.745),(1.585,7.745),(1.585,5.165),(1.685,5.165),(1.685,4.835),(1.585,4.835),(1.585,2.255),(1.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(2.255), u(7.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(2.255), u(7.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.515,4.915),(1.515,5.085),(1.685,5.085),(1.685,4.915),(1.515,4.915)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((2.925,1.37),(2.925,4.835),(2.465,4.835),(2.465,5.165),(2.925,5.165),(2.925,8.63),(3.075,8.63),(3.075,1.37),(2.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,2.255),(2.415,7.5),(2.585,7.5),(2.585,5.165),(2.685,5.165),(2.685,4.835),(2.585,4.835),(2.585,2.255),(2.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(2.255), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(2.255), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.515,4.915),(2.515,5.085),(2.685,5.085),(2.685,4.915),(2.515,4.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)

    cell = new_cells['a4_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(6.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            '_net3': Net.create(cell, '_net3'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'i3': Net.create(cell, 'i3'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((3.375,8.995),(3.375,9.0),(3.3,9.0),(3.3,9.4),(3.375,9.4),(3.375,9.405),(3.625,9.405),(3.625,9.4),(3.7,9.4),(3.7,9.0),(3.625,9.0),(3.625,8.995),(3.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,5.5),(4.375,6.5),(3.925,6.5),(3.925,8.5),(4.375,8.5),(4.375,9.5),(5.075,9.5),(5.075,5.5),(4.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,6.5),(1.925,8.5),(2.375,8.5),(2.375,9.305),(2.625,9.305),(2.625,8.5),(3.075,8.5),(3.075,6.5),(1.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,8.995),(1.375,9.0),(1.3,9.0),(1.3,9.4),(1.375,9.4),(1.375,9.405),(1.625,9.405),(1.625,9.4),(1.7,9.4),(1.7,9.0),(1.625,9.0),(1.625,8.995),(1.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,9.305),(0.625,9.305),(0.625,8.5),(1.075,8.5),(1.075,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(6.3,10.8),(6.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(6.0,10.0),(6.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.0), u(6.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.0), u(6.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.415,9.015),(2.415,9.185),(2.585,9.185),(2.585,9.015),(2.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,9.115),(3.415,9.285),(3.585,9.285),(3.585,9.115),(3.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.015),(0.415,9.185),(0.585,9.185),(0.585,9.015),(0.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.115),(1.415,9.285),(1.585,9.285),(1.585,9.115),(1.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.045),(4.415,9.215),(4.585,9.215),(4.585,9.045),(4.415,9.045)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((5.075,0.6),(5.075,2.5),(5.625,2.5),(5.625,0.6),(5.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.075,5.5),(5.075,9.5),(5.625,9.5),(5.625,5.5),(5.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((5.415,1.37),(5.415,8.63),(5.585,8.63),(5.585,1.37),(5.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((5.415,5.64),(5.415,5.81),(5.585,5.81),(5.585,5.64),(5.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,5.98),(5.415,6.15),(5.585,6.15),(5.585,5.98),(5.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,6.32),(5.415,6.49),(5.585,6.49),(5.585,6.32),(5.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,6.66),(5.415,6.83),(5.585,6.83),(5.585,6.66),(5.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.0),(5.415,7.17),(5.585,7.17),(5.585,7.0),(5.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.34),(5.415,7.51),(5.585,7.51),(5.585,7.34),(5.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.68),(5.415,7.85),(5.585,7.85),(5.585,7.68),(5.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,8.02),(5.415,8.19),(5.585,8.19),(5.585,8.02),(5.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,8.36),(5.415,8.53),(5.585,8.53),(5.585,8.36),(5.415,8.36)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,1.49),(5.415,1.66),(5.585,1.66),(5.585,1.49),(5.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,1.83),(5.415,2.0),(5.585,2.0),(5.585,1.83),(5.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,2.17),(5.415,2.34),(5.585,2.34),(5.585,2.17),(5.415,2.17)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((4.25,5.375),(4.25,6.375),(0.25,6.375),(0.25,9.43),(0.75,9.43),(0.75,8.625),(2.25,8.625),(2.25,9.43),(2.75,9.43),(2.75,8.625),(4.25,8.625),(4.25,9.625),(5.75,9.625),(5.75,5.375),(4.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((3.25,0.37),(3.25,0.375),(3.175,0.375),(3.175,1.025),(3.25,1.025),(3.25,1.03),(3.75,1.03),(3.75,1.025),(3.825,1.025),(3.825,0.375),(3.75,0.375),(3.75,0.37),(3.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((1.25,0.37),(1.25,0.375),(1.175,0.375),(1.175,1.025),(1.25,1.025),(1.25,1.03),(1.75,1.03),(1.75,1.025),(1.825,1.025),(1.825,0.375),(1.75,0.375),(1.75,0.37),(1.25,0.37)),
        )
        createRL(
            tech, net, 'nsdm',
            ((3.25,8.87),(3.25,8.875),(3.175,8.875),(3.175,9.525),(3.25,9.525),(3.25,9.53),(3.75,9.53),(3.75,9.525),(3.825,9.525),(3.825,8.875),(3.75,8.875),(3.75,8.87),(3.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((4.25,0.475),(4.25,1.375),(0.25,1.375),(0.25,3.625),(4.75,3.625),(4.75,2.625),(5.75,2.625),(5.75,0.475),(4.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,8.87),(1.25,8.875),(1.175,8.875),(1.175,9.525),(1.25,9.525),(1.25,9.53),(1.75,9.53),(1.75,9.525),(1.825,9.525),(1.825,8.875),(1.75,8.875),(1.75,8.87),(1.25,8.87)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'poly',
            ((4.925,0.47),(4.925,4.335),(4.465,4.335),(4.465,4.665),(4.925,4.665),(4.925,9.63),(5.075,9.63),(5.075,0.47),(4.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,6.5),(1.075,8.5),(1.925,8.5),(1.925,6.5),(1.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,3.5),(0.925,3.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,6.5),(3.075,8.5),(3.925,8.5),(3.925,6.5),(3.075,6.5)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.6),(0.415,2.33),(0.585,2.33),(0.585,2.085),(4.415,2.085),(4.415,7.915),(3.585,7.915),(3.585,7.67),(3.415,7.67),(3.415,7.915),(1.585,7.915),(1.585,7.67),(1.415,7.67),(1.415,8.4),(1.585,8.4),(1.585,8.085),(3.415,8.085),(3.415,8.4),(3.585,8.4),(3.585,8.085),(4.585,8.085),(4.585,4.665),(4.685,4.665),(4.685,4.335),(4.585,4.335),(4.585,1.915),(0.585,1.915),(0.585,1.6),(0.415,1.6)),
        )
        createRL(
            tech, net, 'licon',
            ((4.515,4.415),(4.515,4.585),(4.685,4.585),(4.685,4.415),(4.515,4.415)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.81),(3.415,7.98),(3.585,7.98),(3.585,7.81),(3.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.15),(3.415,8.32),(3.585,8.32),(3.585,8.15),(3.415,8.15)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.68),(0.415,1.85),(0.585,1.85),(0.585,1.68),(0.415,1.68)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.02),(0.415,2.19),(0.585,2.19),(0.585,2.02),(0.415,2.02)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.81),(1.415,7.98),(1.585,7.98),(1.585,7.81),(1.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,8.15),(1.415,8.32),(1.585,8.32),(1.585,8.15),(1.415,8.15)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.925,1.5),(0.925,3.5),(1.075,3.5),(1.075,1.5),(0.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,1.5),(1.925,3.5),(2.075,3.5),(2.075,1.5),(1.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,0.6),(4.375,1.5),(3.925,1.5),(3.925,3.5),(4.625,3.5),(4.625,2.5),(5.075,2.5),(5.075,0.6),(4.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.495),(1.375,0.5),(1.3,0.5),(1.3,0.9),(1.375,0.9),(1.375,0.905),(1.625,0.905),(1.625,0.9),(1.7,0.9),(1.7,0.5),(1.625,0.5),(1.625,0.495),(1.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,0.495),(3.375,0.5),(3.3,0.5),(3.3,0.9),(3.375,0.9),(3.375,0.905),(3.625,0.905),(3.625,0.9),(3.7,0.9),(3.7,0.5),(3.625,0.5),(3.625,0.495),(3.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,1.5),(2.925,3.5),(3.075,3.5),(3.075,1.5),(2.925,1.5)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(6.0,1.2),(6.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.0), u(6.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.0), u(6.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,0.615),(1.415,0.785),(1.585,0.785),(1.585,0.615),(1.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,0.835),(4.415,1.005),(4.585,1.005),(4.585,0.835),(4.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,0.615),(3.415,0.785),(3.585,0.785),(3.585,0.615),(3.415,0.615)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,4.835),(0.465,4.835),(0.465,5.165),(0.925,5.165),(0.925,8.63),(1.075,8.63),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,2.5),(0.415,8.63),(0.585,8.63),(0.585,5.165),(0.685,5.165),(0.685,4.835),(0.585,4.835),(0.585,2.5),(0.415,2.5)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(2.5), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(2.5), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,4.915),(0.515,5.085),(0.685,5.085),(0.685,4.915),(0.515,4.915)),
        )
        net = nets['_net3']
        createRL(
            tech, net, 'difftap',
            ((1.075,1.5),(1.075,3.5),(1.925,3.5),(1.925,1.5),(1.075,1.5)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,5.835),(1.465,5.835),(1.465,6.165),(1.925,6.165),(1.925,8.63),(2.075,8.63),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,2.255),(1.415,7.5),(1.585,7.5),(1.585,6.165),(1.685,6.165),(1.685,5.835),(1.585,5.835),(1.585,2.255),(1.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(2.255), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(2.255), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.515,5.915),(1.515,6.085),(1.685,6.085),(1.685,5.915),(1.515,5.915)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((2.075,1.5),(2.075,3.5),(2.925,3.5),(2.925,1.5),(2.075,1.5)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((2.925,1.37),(2.925,4.835),(2.465,4.835),(2.465,5.165),(2.925,5.165),(2.925,8.63),(3.075,8.63),(3.075,1.37),(2.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,2.255),(2.415,7.745),(2.585,7.745),(2.585,5.165),(2.685,5.165),(2.685,4.835),(2.585,4.835),(2.585,2.255),(2.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(2.255), u(7.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(2.255), u(7.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.515,4.915),(2.515,5.085),(2.685,5.085),(2.685,4.915),(2.515,4.915)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((3.075,1.5),(3.075,3.5),(3.925,3.5),(3.925,1.5),(3.075,1.5)),
        )
        net = nets['i3']
        createRL(
            tech, net, 'poly',
            ((3.925,1.37),(3.925,3.835),(3.465,3.835),(3.465,4.165),(3.925,4.165),(3.925,8.63),(4.075,8.63),(4.075,1.37),(3.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,2.255),(3.415,7.5),(3.585,7.5),(3.585,4.165),(3.685,4.165),(3.685,3.835),(3.585,3.835),(3.585,2.255),(3.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(0.17), u(2.255), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(0.17), u(2.255), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.515,3.915),(3.515,4.085),(3.685,4.085),(3.685,3.915),(3.515,3.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)
        nets['i3'].setDirection(Net.Direction.IN)

    cell = new_cells['ao22_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(5.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((2.375,0.495),(2.375,0.5),(2.25,0.5),(2.25,0.9),(2.375,0.9),(2.375,0.905),(2.625,0.905),(2.625,0.9),(2.75,0.9),(2.75,0.5),(2.625,0.5),(2.625,0.495),(2.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,1.5),(1.925,2.5),(2.075,2.5),(2.075,1.5),(1.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,0.6),(3.375,1.5),(2.925,1.5),(2.925,2.5),(4.075,2.5),(4.075,0.6),(3.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,1.5),(0.925,2.5),(1.075,2.5),(1.075,1.5),(0.925,1.5)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(5.0,1.2),(5.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,0.835),(3.415,1.005),(3.585,1.005),(3.585,0.835),(3.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.615),(2.415,0.785),(2.585,0.785),(2.585,0.615),(2.415,0.615)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((4.075,0.6),(4.075,2.5),(4.625,2.5),(4.625,0.6),(4.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.075,5.5),(4.075,9.5),(4.625,9.5),(4.625,5.5),(4.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,1.37),(4.415,8.63),(4.585,8.63),(4.585,1.37),(4.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((4.415,1.49),(4.415,1.66),(4.585,1.66),(4.585,1.49),(4.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.83),(4.415,2.0),(4.585,2.0),(4.585,1.83),(4.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,2.17),(4.415,2.34),(4.585,2.34),(4.585,2.17),(4.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.64),(4.415,5.81),(4.585,5.81),(4.585,5.64),(4.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.98),(4.415,6.15),(4.585,6.15),(4.585,5.98),(4.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.32),(4.415,6.49),(4.585,6.49),(4.585,6.32),(4.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.66),(4.415,6.83),(4.585,6.83),(4.585,6.66),(4.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.0),(4.415,7.17),(4.585,7.17),(4.585,7.0),(4.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.34),(4.415,7.51),(4.585,7.51),(4.585,7.34),(4.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.68),(4.415,7.85),(4.585,7.85),(4.585,7.68),(4.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.02),(4.415,8.19),(4.585,8.19),(4.585,8.02),(4.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.36),(4.415,8.53),(4.585,8.53),(4.585,8.36),(4.415,8.36)),
        )
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((3.25,0.475),(3.25,1.375),(0.25,1.375),(0.25,2.625),(1.25,2.625),(1.25,3.33),(1.75,3.33),(1.75,2.625),(4.75,2.625),(4.75,0.475),(3.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((2.25,8.87),(2.25,8.875),(2.125,8.875),(2.125,9.525),(2.25,9.525),(2.25,9.53),(2.75,9.53),(2.75,9.525),(2.875,9.525),(2.875,8.875),(2.75,8.875),(2.75,8.87),(2.25,8.87)),
        )
        createRL(
            tech, net, 'psdm',
            ((3.25,5.375),(3.25,6.175),(0.25,6.175),(0.25,9.43),(0.75,9.43),(0.75,8.425),(3.25,8.425),(3.25,9.625),(4.75,9.625),(4.75,5.375),(3.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((2.25,0.37),(2.25,0.375),(2.125,0.375),(2.125,1.025),(2.25,1.025),(2.25,1.03),(2.75,1.03),(2.75,1.025),(2.875,1.025),(2.875,0.375),(2.75,0.375),(2.75,0.37),(2.25,0.37)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'poly',
            ((3.925,0.47),(3.925,3.835),(2.465,3.835),(2.465,4.165),(3.925,4.165),(3.925,9.63),(4.075,9.63),(4.075,0.47),(3.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,1.5),(1.075,2.5),(1.375,2.5),(1.375,3.205),(1.625,3.205),(1.625,2.5),(1.925,2.5),(1.925,1.5),(1.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,6.3),(2.075,8.3),(2.925,8.3),(2.925,6.3),(2.075,6.3)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,2.915),(1.415,3.085),(1.585,3.085),(1.585,2.915),(1.415,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,6.535),(2.415,6.705),(2.585,6.705),(2.585,6.535),(2.415,6.535)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,6.875),(2.415,7.045),(2.585,7.045),(2.585,6.875),(2.415,6.875)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.215),(2.415,7.385),(2.585,7.385),(2.585,7.215),(2.415,7.215)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.555),(2.415,7.725),(2.585,7.725),(2.585,7.555),(2.415,7.555)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.895),(2.415,8.065),(2.585,8.065),(2.585,7.895),(2.415,7.895)),
        )
        createRL(
            tech, net, 'licon',
            ((2.515,3.915),(2.515,4.085),(2.685,4.085),(2.685,3.915),(2.515,3.915)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,2.835),(1.415,3.165),(1.585,3.165),(1.585,3.085),(2.415,3.085),(2.415,8.145),(2.585,8.145),(2.585,4.165),(2.685,4.165),(2.685,3.835),(2.585,3.835),(2.585,2.915),(1.585,2.915),(1.585,2.835),(1.415,2.835)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((0.375,6.3),(0.375,9.305),(0.625,9.305),(0.625,8.3),(1.075,8.3),(1.075,6.3),(0.375,6.3)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,8.995),(2.375,9.0),(2.25,9.0),(2.25,9.4),(2.375,9.4),(2.375,9.405),(2.625,9.405),(2.625,9.4),(2.75,9.4),(2.75,9.0),(2.625,9.0),(2.625,8.995),(2.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,5.5),(3.375,6.3),(2.925,6.3),(2.925,8.3),(3.375,8.3),(3.375,9.5),(4.075,9.5),(4.075,5.5),(3.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,6.3),(1.925,8.3),(2.075,8.3),(2.075,6.3),(1.925,6.3)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(5.3,10.8),(5.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(5.0,10.0),(5.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.415,9.115),(2.415,9.285),(2.585,9.285),(2.585,9.115),(2.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,9.045),(3.415,9.215),(3.585,9.215),(3.585,9.045),(3.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.015),(0.415,9.185),(0.585,9.185),(0.585,9.015),(0.415,9.015)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,2.5),(0.925,2.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,1.5),(2.075,2.5),(2.925,2.5),(2.925,1.5),(2.075,1.5)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.665),(0.415,2.335),(0.585,2.335),(0.585,2.15),(2.415,2.15),(2.415,2.335),(2.585,2.335),(2.585,1.665),(2.415,1.665),(2.415,1.85),(0.585,1.85),(0.585,1.665),(0.415,1.665)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.745),(0.415,1.915),(0.585,1.915),(0.585,1.745),(0.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.085),(0.415,2.255),(0.585,2.255),(0.585,2.085),(0.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.745),(2.415,1.915),(2.585,1.915),(2.585,1.745),(2.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.085),(2.415,2.255),(2.585,2.255),(2.585,2.085),(2.415,2.085)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,3.835),(0.465,3.835),(0.465,4.165),(0.925,4.165),(0.925,8.43),(1.075,8.43),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,2.505),(0.415,8.63),(0.585,8.63),(0.585,4.165),(0.685,4.165),(0.685,3.835),(0.585,3.835),(0.585,2.505),(0.415,2.505)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(2.505), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(2.505), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,3.915),(0.515,4.085),(0.685,4.085),(0.685,3.915),(0.515,3.915)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((1.075,6.3),(1.075,8.3),(1.925,8.3),(1.925,6.3),(1.075,6.3)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,4.835),(1.465,4.835),(1.465,5.165),(1.925,5.165),(1.925,8.43),(2.075,8.43),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,3.5),(1.415,8.63),(1.585,8.63),(1.585,5.165),(1.685,5.165),(1.685,4.835),(1.585,4.835),(1.585,3.5),(1.415,3.5)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(3.5), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(3.5), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.515,4.915),(1.515,5.085),(1.685,5.085),(1.685,4.915),(1.515,4.915)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((2.925,4.835),(2.925,8.43),(3.075,8.43),(3.075,5.165),(3.535,5.165),(3.535,4.835),(2.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((2.925,1.37),(2.925,3.165),(3.535,3.165),(3.535,2.835),(3.075,2.835),(3.075,1.37),(2.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,1.37),(3.415,2.835),(3.315,2.835),(3.315,3.165),(3.415,3.165),(3.415,4.835),(3.315,4.835),(3.315,5.165),(3.415,5.165),(3.415,8.63),(3.585,8.63),(3.585,1.37),(3.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.315,2.915),(3.315,3.085),(3.485,3.085),(3.485,2.915),(3.315,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.315,4.915),(3.315,5.085),(3.485,5.085),(3.485,4.915),(3.315,4.915)),
        )
        nets['vss'].setGlobal(True)
        nets['vdd'].setGlobal(True)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)

    cell = new_cells['mx2_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(7.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            '_net3': Net.create(cell, '_net3'),
            '_net4': Net.create(cell, '_net4'),
            '_net5': Net.create(cell, '_net5'),
            'cmd': Net.create(cell, 'cmd'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((4.375,0.495),(4.375,0.5),(4.3,0.5),(4.3,0.9),(4.375,0.9),(4.375,0.905),(4.625,0.905),(4.625,0.9),(4.7,0.9),(4.7,0.5),(4.625,0.5),(4.625,0.495),(4.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,0.6),(2.925,1.5),(3.075,1.5),(3.075,0.6),(2.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.375,0.6),(5.375,1.6),(4.925,1.6),(4.925,2.5),(6.075,2.5),(6.075,0.6),(5.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,0.6),(0.925,1.5),(2.075,1.5),(2.075,0.6),(0.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,1.6),(3.925,2.5),(4.075,2.5),(4.075,1.6),(3.925,1.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(7.0,1.2),(7.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,0.835),(1.415,1.005),(1.585,1.005),(1.585,0.835),(1.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,0.835),(5.415,1.005),(5.585,1.005),(5.585,0.835),(5.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,0.615),(4.415,0.785),(4.585,0.785),(4.585,0.615),(4.415,0.615)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((6.075,0.6),(6.075,2.5),(6.625,2.5),(6.625,0.6),(6.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.075,5.5),(6.075,9.5),(6.625,9.5),(6.625,5.5),(6.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((6.415,1.37),(6.415,8.63),(6.585,8.63),(6.585,1.37),(6.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(6.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(6.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((6.415,1.49),(6.415,1.66),(6.585,1.66),(6.585,1.49),(6.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,1.83),(6.415,2.0),(6.585,2.0),(6.585,1.83),(6.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,2.17),(6.415,2.34),(6.585,2.34),(6.585,2.17),(6.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,5.64),(6.415,5.81),(6.585,5.81),(6.585,5.64),(6.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,5.98),(6.415,6.15),(6.585,6.15),(6.585,5.98),(6.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,6.32),(6.415,6.49),(6.585,6.49),(6.585,6.32),(6.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,6.66),(6.415,6.83),(6.585,6.83),(6.585,6.66),(6.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.0),(6.415,7.17),(6.585,7.17),(6.585,7.0),(6.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.34),(6.415,7.51),(6.585,7.51),(6.585,7.34),(6.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.68),(6.415,7.85),(6.585,7.85),(6.585,7.68),(6.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.02),(6.415,8.19),(6.585,8.19),(6.585,8.02),(6.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.36),(6.415,8.53),(6.585,8.53),(6.585,8.36),(6.415,8.36)),
        )
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.25,0.475),(0.25,2.33),(0.75,2.33),(0.75,1.625),(3.25,1.625),(3.25,3.33),(3.75,3.33),(3.75,2.625),(6.75,2.625),(6.75,0.475),(5.25,0.475),(5.25,1.475),(3.75,1.475),(3.75,0.475),(0.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((4.25,8.87),(4.25,8.875),(4.175,8.875),(4.175,9.525),(4.25,9.525),(4.25,9.53),(4.75,9.53),(4.75,9.525),(4.825,9.525),(4.825,8.875),(4.75,8.875),(4.75,8.87),(4.25,8.87)),
        )
        createRL(
            tech, net, 'psdm',
            ((4.25,0.37),(4.25,0.375),(4.175,0.375),(4.175,1.025),(4.25,1.025),(4.25,1.03),(4.75,1.03),(4.75,1.025),(4.825,1.025),(4.825,0.375),(4.75,0.375),(4.75,0.37),(4.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((5.25,5.375),(5.25,6.375),(0.25,6.375),(0.25,8.625),(1.25,8.625),(1.25,9.43),(1.75,9.43),(1.75,8.625),(5.25,8.625),(5.25,9.625),(6.75,9.625),(6.75,5.375),(5.25,5.375)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'poly',
            ((5.925,0.47),(5.925,4.835),(3.465,4.835),(3.465,5.165),(5.925,5.165),(5.925,9.63),(6.075,9.63),(6.075,0.47),(5.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,0.6),(3.075,1.5),(3.375,1.5),(3.375,3.205),(3.625,3.205),(3.625,2.5),(3.925,2.5),(3.925,1.6),(3.625,1.6),(3.625,0.6),(3.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,6.5),(3.075,8.5),(3.925,8.5),(3.925,6.5),(3.075,6.5)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.915),(3.415,3.085),(3.585,3.085),(3.585,2.915),(3.415,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.515,4.915),(3.515,5.085),(3.685,5.085),(3.685,4.915),(3.515,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.735),(3.415,6.905),(3.585,6.905),(3.585,6.735),(3.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.075),(3.415,7.245),(3.585,7.245),(3.585,7.075),(3.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.415),(3.415,7.585),(3.585,7.585),(3.585,7.415),(3.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.755),(3.415,7.925),(3.585,7.925),(3.585,7.755),(3.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.095),(3.415,8.265),(3.585,8.265),(3.585,8.095),(3.415,8.095)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,2.835),(3.415,8.63),(3.585,8.63),(3.585,5.165),(3.685,5.165),(3.685,4.835),(3.585,4.835),(3.585,2.835),(3.415,2.835)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((5.375,5.5),(5.375,6.5),(4.925,6.5),(4.925,8.5),(5.375,8.5),(5.375,9.5),(6.075,9.5),(6.075,5.5),(5.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,8.995),(4.375,9.0),(4.3,9.0),(4.3,9.4),(4.375,9.4),(4.375,9.405),(4.625,9.405),(4.625,9.4),(4.7,9.4),(4.7,9.0),(4.625,9.0),(4.625,8.995),(4.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,6.5),(0.925,8.5),(1.375,8.5),(1.375,9.305),(1.625,9.305),(1.625,8.5),(2.075,8.5),(2.075,6.5),(0.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,6.5),(3.925,8.5),(4.075,8.5),(4.075,6.5),(3.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,6.5),(2.925,8.5),(3.075,8.5),(3.075,6.5),(2.925,6.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(7.3,10.8),(7.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(7.0,10.0),(7.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,9.015),(1.415,9.185),(1.585,9.185),(1.585,9.015),(1.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,9.045),(5.415,9.215),(5.585,9.215),(5.585,9.045),(5.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.115),(4.415,9.285),(4.585,9.285),(4.585,9.115),(4.415,9.115)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((2.075,0.6),(2.075,1.5),(2.925,1.5),(2.925,0.6),(2.075,0.6)),
        )
        net = nets['_net5']
        createRL(
            tech, net, 'poly',
            ((2.925,0.47),(2.925,1.835),(2.765,1.835),(2.765,2.165),(3.075,2.165),(3.075,0.47),(2.925,0.47)),
        )
        createRL(
            tech, net, 'poly',
            ((3.925,5.835),(3.925,8.63),(4.075,8.63),(4.075,6.165),(4.285,6.165),(4.285,5.835),(3.925,5.835)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,8.5),(0.925,8.5),(0.925,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,0.6),(0.375,2.205),(0.625,2.205),(0.625,1.5),(0.925,1.5),(0.925,0.6),(0.375,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.835),(0.415,8.345),(0.585,8.345),(0.585,2.085),(2.815,2.085),(2.815,2.165),(2.985,2.165),(2.985,2.085),(4.165,2.085),(4.165,5.835),(4.065,5.835),(4.065,6.165),(4.235,6.165),(4.235,6.005),(4.335,6.005),(4.335,1.915),(2.985,1.915),(2.985,1.835),(2.815,1.835),(2.815,1.915),(0.585,1.915),(0.585,1.835),(0.415,1.835)),
        )
        createRL(
            tech, net, 'licon',
            ((2.815,1.915),(2.815,2.085),(2.985,2.085),(2.985,1.915),(2.815,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((4.065,5.915),(4.065,6.085),(4.235,6.085),(4.235,5.915),(4.065,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.915),(0.415,2.085),(0.585,2.085),(0.585,1.915),(0.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.735),(0.415,6.905),(0.585,6.905),(0.585,6.735),(0.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.075),(0.415,7.245),(0.585,7.245),(0.585,7.075),(0.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.415),(0.415,7.585),(0.585,7.585),(0.585,7.415),(0.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.755),(0.415,7.925),(0.585,7.925),(0.585,7.755),(0.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.095),(0.415,8.265),(0.585,8.265),(0.585,8.095),(0.415,8.095)),
        )
        net = nets['_net4']
        createRL(
            tech, net, 'difftap',
            ((4.075,6.5),(4.075,8.5),(4.925,8.5),(4.925,6.5),(4.075,6.5)),
        )
        net = nets['cmd']
        createRL(
            tech, net, 'poly',
            ((0.925,0.47),(0.925,8.63),(1.075,8.63),(1.075,4.165),(2.925,4.165),(2.925,8.63),(3.075,8.63),(3.075,4.165),(4.075,4.165),(4.075,1.47),(3.925,1.47),(3.925,3.835),(1.075,3.835),(1.075,0.47),(0.925,0.47)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,3.915),(2.415,4.085),(2.585,4.085),(2.585,3.915),(2.415,3.915)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,2.255),(2.415,8.63),(2.585,8.63),(2.585,2.255),(2.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(2.255), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(2.255), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((4.075,1.6),(4.075,2.5),(4.925,2.5),(4.925,1.6),(4.075,1.6)),
        )
        net = nets['_net3']
        createRL(
            tech, net, 'difftap',
            ((2.075,6.5),(2.075,8.5),(2.925,8.5),(2.925,6.5),(2.075,6.5)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((1.465,5.835),(1.465,6.165),(1.925,6.165),(1.925,8.63),(2.075,8.63),(2.075,5.835),(1.465,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((1.925,0.47),(1.925,2.835),(1.465,2.835),(1.465,3.165),(2.075,3.165),(2.075,0.47),(1.925,0.47)),
        )
        createRL(
            tech, net, 'licon',
            ((1.515,2.915),(1.515,3.085),(1.685,3.085),(1.685,2.915),(1.515,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.515,5.915),(1.515,6.085),(1.685,6.085),(1.685,5.915),(1.515,5.915)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,2.255),(1.415,8.63),(1.585,8.63),(1.585,6.165),(1.685,6.165),(1.685,5.835),(1.585,5.835),(1.585,3.165),(1.685,3.165),(1.685,2.835),(1.585,2.835),(1.585,2.255),(1.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(2.255), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(2.255), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((4.925,1.47),(4.925,3.165),(5.535,3.165),(5.535,2.835),(5.075,2.835),(5.075,1.47),(4.925,1.47)),
        )
        createRL(
            tech, net, 'poly',
            ((4.765,5.835),(4.765,6.165),(4.925,6.165),(4.925,8.63),(5.075,8.63),(5.075,5.835),(4.765,5.835)),
        )
        createRL(
            tech, net, 'li',
            ((5.415,1.37),(5.415,2.835),(5.315,2.835),(5.315,3.165),(5.415,3.165),(5.415,5.915),(4.985,5.915),(4.985,5.835),(4.815,5.835),(4.815,6.165),(4.985,6.165),(4.985,6.085),(5.415,6.085),(5.415,8.63),(5.585,8.63),(5.585,1.37),(5.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((5.315,2.915),(5.315,3.085),(5.485,3.085),(5.485,2.915),(5.315,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((4.815,5.915),(4.815,6.085),(4.985,6.085),(4.985,5.915),(4.815,5.915)),
        )
        nets['vss'].setGlobal(True)
        nets['vdd'].setGlobal(True)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['cmd'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)

    cell = new_cells['mx3_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(11.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net10': Net.create(cell, '_net10'),
            '_net2': Net.create(cell, '_net2'),
            '_net3': Net.create(cell, '_net3'),
            '_net4': Net.create(cell, '_net4'),
            '_net5': Net.create(cell, '_net5'),
            '_net6': Net.create(cell, '_net6'),
            '_net7': Net.create(cell, '_net7'),
            '_net8': Net.create(cell, '_net8'),
            '_net9': Net.create(cell, '_net9'),
            'cmd0': Net.create(cell, 'cmd0'),
            'cmd1': Net.create(cell, 'cmd1'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['_net5']
        createRL(
            tech, net, 'difftap',
            ((3.375,6.5),(3.375,7.5),(3.075,7.5),(3.075,9.5),(3.625,9.5),(3.625,8.5),(3.925,8.5),(3.925,6.5),(3.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((8.075,7.5),(8.075,9.5),(8.625,9.5),(8.625,7.5),(8.075,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,1.1),(3.075,2.3),(3.375,2.3),(3.375,3.205),(3.625,3.205),(3.625,2.8),(3.925,2.8),(3.925,1.6),(3.625,1.6),(3.625,1.1),(3.075,1.1)),
        )
        createRL(
            tech, net, 'difftap',
            ((8.075,0.7),(8.075,1.9),(8.625,1.9),(8.625,0.7),(8.075,0.7)),
        )
        createRL(
            tech, net, 'poly',
            ((9.925,1.37),(9.925,3.835),(9.465,3.835),(9.465,4.165),(9.925,4.165),(9.925,9.63),(10.075,9.63),(10.075,1.37),(9.925,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.715),(3.415,6.885),(3.585,6.885),(3.585,6.715),(3.415,6.715)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.055),(3.415,7.225),(3.585,7.225),(3.585,7.055),(3.415,7.055)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.395),(3.415,7.565),(3.585,7.565),(3.585,7.395),(3.415,7.395)),
        )
        createRL(
            tech, net, 'licon',
            ((9.515,3.915),(9.515,4.085),(9.685,4.085),(9.685,3.915),(9.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,1.53),(8.415,1.7),(8.585,1.7),(8.585,1.53),(8.415,1.53)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,7.66),(8.415,7.83),(8.585,7.83),(8.585,7.66),(8.415,7.66)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.0),(8.415,8.17),(8.585,8.17),(8.585,8.0),(8.415,8.0)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.34),(8.415,8.51),(8.585,8.51),(8.585,8.34),(8.415,8.34)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.915),(3.415,3.085),(3.585,3.085),(3.585,2.915),(3.415,2.915)),
        )
        createRL(
            tech, net, 'li',
            ((8.415,1.37),(8.415,2.085),(9.415,2.085),(9.415,6.915),(4.585,6.915),(4.585,2.915),(3.585,2.915),(3.585,2.835),(3.415,2.835),(3.415,3.165),(3.585,3.165),(3.585,3.085),(4.415,3.085),(4.415,6.915),(3.585,6.915),(3.585,6.635),(3.415,6.635),(3.415,7.745),(3.585,7.745),(3.585,7.085),(8.415,7.085),(8.415,8.63),(8.585,8.63),(8.585,7.085),(9.585,7.085),(9.585,4.165),(9.685,4.165),(9.685,3.835),(9.585,3.835),(9.585,1.915),(8.585,1.915),(8.585,1.37),(8.415,1.37)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((4.925,6.5),(4.925,8.5),(5.075,8.5),(5.075,6.5),(4.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.925,7.5),(5.925,9.5),(7.075,9.5),(7.075,7.5),(5.925,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,8.995),(4.375,9.0),(4.3,9.0),(4.3,9.4),(4.375,9.4),(4.375,9.405),(4.625,9.405),(4.625,9.4),(4.7,9.4),(4.7,9.0),(4.625,9.0),(4.625,8.995),(4.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((8.925,5.5),(8.925,6.9),(9.375,6.9),(9.375,9.5),(10.075,9.5),(10.075,5.5),(8.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,6.5),(3.925,8.5),(4.075,8.5),(4.075,6.5),(3.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,5.5),(0.375,9.305),(0.625,9.305),(0.625,6.9),(1.075,6.9),(1.075,5.5),(0.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((7.925,7.5),(7.925,9.5),(8.075,9.5),(8.075,7.5),(7.925,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,7.5),(2.925,9.5),(3.075,9.5),(3.075,7.5),(2.925,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,7.5),(1.925,9.5),(2.075,9.5),(2.075,7.5),(1.925,7.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(11.3,10.8),(11.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((9.415,7.255),(9.415,8.8),(6.585,8.8),(6.585,7.655),(6.415,7.655),(6.415,8.8),(0.0,8.8),(0.0,10.0),(11.0,10.0),(11.0,8.8),(9.585,8.8),(9.585,7.255),(9.415,7.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(11.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(11.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((9.415,7.425),(9.415,7.595),(9.585,7.595),(9.585,7.425),(9.415,7.425)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,7.765),(9.415,7.935),(9.585,7.935),(9.585,7.765),(9.415,7.765)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,8.105),(9.415,8.275),(9.585,8.275),(9.585,8.105),(9.415,8.105)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,8.445),(9.415,8.615),(9.585,8.615),(9.585,8.445),(9.415,8.445)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,8.785),(9.415,8.955),(9.585,8.955),(9.585,8.785),(9.415,8.785)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,9.125),(9.415,9.295),(9.585,9.295),(9.585,9.125),(9.415,9.125)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.735),(6.415,7.905),(6.585,7.905),(6.585,7.735),(6.415,7.735)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.075),(6.415,8.245),(6.585,8.245),(6.585,8.075),(6.415,8.075)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.415),(6.415,8.585),(6.585,8.585),(6.585,8.415),(6.415,8.415)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.755),(6.415,8.925),(6.585,8.925),(6.585,8.755),(6.415,8.755)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,9.095),(6.415,9.265),(6.585,9.265),(6.585,9.095),(6.415,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.115),(4.415,9.285),(4.585,9.285),(4.585,9.115),(4.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.015),(0.415,9.185),(0.585,9.185),(0.585,9.015),(0.415,9.015)),
        )
        net = nets['_net6']
        createRL(
            tech, net, 'difftap',
            ((4.075,6.5),(4.075,8.5),(4.925,8.5),(4.925,6.5),(4.075,6.5)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((4.25,0.37),(4.25,0.375),(4.175,0.375),(4.175,1.025),(4.25,1.025),(4.25,1.03),(4.75,1.03),(4.75,1.025),(4.825,1.025),(4.825,0.375),(4.75,0.375),(4.75,0.37),(4.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.25,5.375),(0.25,9.43),(0.75,9.43),(0.75,8.5),(1.25,8.5),(1.25,9.625),(3.75,9.625),(3.75,8.625),(5.25,8.625),(5.25,9.625),(8.75,9.625),(8.75,8.5),(9.25,8.5),(9.25,9.625),(10.75,9.625),(10.75,5.375),(8.25,5.375),(8.25,5.5),(1.75,5.5),(1.75,5.375),(0.25,5.375)),
        )
        createRL(
            tech, net, 'nsdm',
            ((4.25,8.87),(4.25,8.875),(4.175,8.875),(4.175,9.525),(4.25,9.525),(4.25,9.53),(4.75,9.53),(4.75,9.525),(4.825,9.525),(4.825,8.875),(4.75,8.875),(4.75,8.87),(4.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,0.57),(0.25,3.825),(1.75,3.825),(1.75,3.5),(8.25,3.5),(8.25,3.625),(10.75,3.625),(10.75,1.375),(9.75,1.375),(9.75,0.57),(9.25,0.57),(9.25,1.5),(8.75,1.5),(8.75,0.575),(5.25,0.575),(5.25,1.475),(3.75,1.475),(3.75,0.975),(1.25,0.975),(1.25,1.5),(0.75,1.5),(0.75,0.57),(0.25,0.57)),
        )
        net = nets['_net3']
        createRL(
            tech, net, 'poly',
            ((3.615,5.835),(3.615,6.165),(3.925,6.165),(3.925,8.63),(4.075,8.63),(4.075,5.835),(3.615,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((2.925,0.97),(2.925,2.835),(2.765,2.835),(2.765,3.165),(3.075,3.165),(3.075,0.97),(2.925,0.97)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,5.5),(1.075,6.9),(1.625,6.9),(1.625,5.5),(1.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,2.9),(1.075,3.7),(1.625,3.7),(1.625,2.9),(1.075,2.9)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,3.045),(1.415,3.215),(1.585,3.215),(1.585,3.045),(1.415,3.045)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,3.385),(1.415,3.555),(1.585,3.555),(1.585,3.385),(1.415,3.385)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,5.715),(1.415,5.885),(1.585,5.885),(1.585,5.715),(1.415,5.715)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,6.055),(1.415,6.225),(1.585,6.225),(1.585,6.055),(1.415,6.055)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,6.395),(1.415,6.565),(1.585,6.565),(1.585,6.395),(1.415,6.395)),
        )
        createRL(
            tech, net, 'licon',
            ((2.815,2.915),(2.815,3.085),(2.985,3.085),(2.985,2.915),(2.815,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.665,5.915),(3.665,6.085),(3.835,6.085),(3.835,5.915),(3.665,5.915)),
        )
        createRL(
            tech, net, 'li',
            ((2.815,2.835),(2.815,2.915),(1.415,2.915),(1.415,6.745),(1.585,6.745),(1.585,6.085),(3.665,6.085),(3.665,6.165),(3.835,6.165),(3.835,5.835),(3.665,5.835),(3.665,5.915),(1.585,5.915),(1.585,3.085),(2.815,3.085),(2.815,3.165),(2.985,3.165),(2.985,3.085),(3.0,3.085),(3.0,2.915),(2.985,2.915),(2.985,2.835),(2.815,2.835)),
        )
        net = nets['_net9']
        createRL(
            tech, net, 'difftap',
            ((2.075,1.1),(2.075,2.3),(2.925,2.3),(2.925,1.1),(2.075,1.1)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((3.925,1.6),(3.925,2.8),(4.075,2.8),(4.075,1.6),(3.925,1.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((9.375,0.695),(9.375,2.9),(8.925,2.9),(8.925,3.5),(10.075,3.5),(10.075,1.5),(9.625,1.5),(9.625,0.695),(9.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.925,0.7),(5.925,1.9),(7.075,1.9),(7.075,0.7),(5.925,0.7)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,0.495),(4.375,0.5),(4.3,0.5),(4.3,0.9),(4.375,0.9),(4.375,0.905),(4.625,0.905),(4.625,0.9),(4.7,0.9),(4.7,0.5),(4.625,0.5),(4.625,0.495),(4.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,0.695),(0.375,3.7),(1.075,3.7),(1.075,2.9),(0.625,2.9),(0.625,0.695),(0.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,1.1),(1.925,2.3),(2.075,2.3),(2.075,1.1),(1.925,1.1)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,1.1),(2.925,2.3),(3.075,2.3),(3.075,1.1),(2.925,1.1)),
        )
        createRL(
            tech, net, 'difftap',
            ((7.925,0.7),(7.925,1.9),(8.075,1.9),(8.075,0.7),(7.925,0.7)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.925,1.6),(4.925,2.8),(5.075,2.8),(5.075,1.6),(4.925,1.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(11.0,1.2),(11.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(11.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(11.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((9.415,0.815),(9.415,0.985),(9.585,0.985),(9.585,0.815),(9.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,0.615),(4.415,0.785),(4.585,0.785),(4.585,0.615),(4.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,0.885),(6.415,1.055),(6.585,1.055),(6.585,0.885),(6.415,0.885)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.815),(0.415,0.985),(0.585,0.985),(0.585,0.815),(0.415,0.815)),
        )
        net = nets['_net7']
        createRL(
            tech, net, 'difftap',
            ((7.075,0.7),(7.075,1.9),(7.925,1.9),(7.925,0.7),(7.075,0.7)),
        )
        net = nets['_net4']
        createRL(
            tech, net, 'poly',
            ((6.925,0.57),(6.925,2.835),(5.925,2.835),(5.925,9.63),(6.075,9.63),(6.075,3.165),(7.235,3.165),(7.235,2.835),(7.075,2.835),(7.075,0.57),(6.925,0.57)),
        )
        createRL(
            tech, net, 'difftap',
            ((8.375,2.9),(8.375,3.5),(8.925,3.5),(8.925,2.9),(8.375,2.9)),
        )
        createRL(
            tech, net, 'difftap',
            ((8.375,5.5),(8.375,6.9),(8.925,6.9),(8.925,5.5),(8.375,5.5)),
        )
        createRL(
            tech, net, 'licon',
            ((7.015,2.915),(7.015,3.085),(7.185,3.085),(7.185,2.915),(7.015,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,5.715),(8.415,5.885),(8.585,5.885),(8.585,5.715),(8.415,5.715)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,6.055),(8.415,6.225),(8.585,6.225),(8.585,6.055),(8.415,6.055)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,6.395),(8.415,6.565),(8.585,6.565),(8.585,6.395),(8.415,6.395)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,3.115),(8.415,3.285),(8.585,3.285),(8.585,3.115),(8.415,3.115)),
        )
        createRL(
            tech, net, 'li',
            ((7.015,2.835),(7.015,2.915),(7.0,2.915),(7.0,3.085),(7.015,3.085),(7.015,3.165),(7.185,3.165),(7.185,3.085),(8.415,3.085),(8.415,6.745),(8.585,6.745),(8.585,2.915),(7.185,2.915),(7.185,2.835),(7.015,2.835)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((1.375,7.5),(1.375,9.5),(1.925,9.5),(1.925,7.5),(1.375,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.075,6.5),(5.075,8.5),(5.375,8.5),(5.375,9.5),(5.925,9.5),(5.925,7.5),(5.625,7.5),(5.625,6.5),(5.075,6.5)),
        )
        createRL(
            tech, net, 'li',
            ((5.415,7.255),(5.415,7.915),(1.585,7.915),(1.585,7.58),(1.415,7.58),(1.415,8.63),(1.585,8.63),(1.585,8.085),(5.415,8.085),(5.415,8.63),(5.585,8.63),(5.585,7.255),(5.415,7.255)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.66),(1.415,7.83),(1.585,7.83),(1.585,7.66),(1.415,7.66)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,8.0),(1.415,8.17),(1.585,8.17),(1.585,8.0),(1.415,8.0)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,8.34),(1.415,8.51),(1.585,8.51),(1.585,8.34),(1.415,8.34)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.35),(5.415,7.52),(5.585,7.52),(5.585,7.35),(5.415,7.35)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.69),(5.415,7.86),(5.585,7.86),(5.585,7.69),(5.415,7.69)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,8.03),(5.415,8.2),(5.585,8.2),(5.585,8.03),(5.415,8.03)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,8.37),(5.415,8.54),(5.585,8.54),(5.585,8.37),(5.415,8.37)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((10.075,1.5),(10.075,3.5),(10.625,3.5),(10.625,1.5),(10.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((10.075,5.5),(10.075,9.5),(10.625,9.5),(10.625,5.5),(10.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((10.415,1.37),(10.415,8.63),(10.585,8.63),(10.585,1.37),(10.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(10.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(10.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((10.415,5.64),(10.415,5.81),(10.585,5.81),(10.585,5.64),(10.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,5.98),(10.415,6.15),(10.585,6.15),(10.585,5.98),(10.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,6.32),(10.415,6.49),(10.585,6.49),(10.585,6.32),(10.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,6.66),(10.415,6.83),(10.585,6.83),(10.585,6.66),(10.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,7.0),(10.415,7.17),(10.585,7.17),(10.585,7.0),(10.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,7.34),(10.415,7.51),(10.585,7.51),(10.585,7.34),(10.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,7.68),(10.415,7.85),(10.585,7.85),(10.585,7.68),(10.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,8.02),(10.415,8.19),(10.585,8.19),(10.585,8.02),(10.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,8.36),(10.415,8.53),(10.585,8.53),(10.585,8.36),(10.415,8.36)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,1.735),(10.415,1.905),(10.585,1.905),(10.585,1.735),(10.415,1.735)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,2.075),(10.415,2.245),(10.585,2.245),(10.585,2.075),(10.415,2.075)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,2.415),(10.415,2.585),(10.585,2.585),(10.585,2.415),(10.415,2.415)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,2.755),(10.415,2.925),(10.585,2.925),(10.585,2.755),(10.415,2.755)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,3.095),(10.415,3.265),(10.585,3.265),(10.585,3.095),(10.415,3.095)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((5.375,0.7),(5.375,1.6),(5.075,1.6),(5.075,2.8),(5.625,2.8),(5.625,1.9),(5.925,1.9),(5.925,0.7),(5.375,0.7)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,1.1),(1.375,2.3),(1.925,2.3),(1.925,1.1),(1.375,1.1)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,1.56),(1.415,1.73),(1.585,1.73),(1.585,1.56),(1.415,1.56)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,1.9),(1.415,2.07),(1.585,2.07),(1.585,1.9),(1.415,1.9)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,1.47),(5.415,1.64),(5.585,1.64),(5.585,1.47),(5.415,1.47)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,1.81),(5.415,1.98),(5.585,1.98),(5.585,1.81),(5.415,1.81)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,2.15),(5.415,2.32),(5.585,2.32),(5.585,2.15),(5.415,2.15)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,2.49),(5.415,2.66),(5.585,2.66),(5.585,2.49),(5.415,2.49)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.37),(1.415,2.15),(1.585,2.15),(1.585,2.085),(5.415,2.085),(5.415,2.74),(5.585,2.74),(5.585,1.37),(5.415,1.37),(5.415,1.915),(1.585,1.915),(1.585,1.37),(1.415,1.37)),
        )
        net = nets['cmd0']
        createRL(
            tech, net, 'poly',
            ((8.925,2.77),(8.925,4.835),(6.465,4.835),(6.465,5.165),(6.925,5.165),(6.925,9.63),(7.075,9.63),(7.075,5.165),(8.925,5.165),(8.925,7.03),(9.075,7.03),(9.075,2.77),(8.925,2.77)),
        )
        createRL(
            tech, net, 'poly',
            ((5.925,0.57),(5.925,2.415),(6.535,2.415),(6.535,2.085),(6.075,2.085),(6.075,0.57),(5.925,0.57)),
        )
        createRL(
            tech, net, 'licon',
            ((6.515,4.915),(6.515,5.085),(6.685,5.085),(6.685,4.915),(6.515,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.315,2.165),(6.315,2.335),(6.485,2.335),(6.485,2.165),(6.315,2.165)),
        )
        createRL(
            tech, net, 'li',
            ((6.415,1.37),(6.415,2.085),(6.315,2.085),(6.315,2.415),(6.415,2.415),(6.415,6.745),(6.585,6.745),(6.585,5.165),(6.685,5.165),(6.685,4.835),(6.585,4.835),(6.585,1.37),(6.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(6.5), u(0.17), u(1.37), u(6.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(6.5), u(0.17), u(1.37), u(6.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((7.075,7.5),(7.075,9.5),(7.925,9.5),(7.925,7.5),(7.075,7.5)),
        )
        net = nets['_net10']
        createRL(
            tech, net, 'difftap',
            ((4.075,1.6),(4.075,2.8),(4.925,2.8),(4.925,1.6),(4.075,1.6)),
        )
        net = nets['cmd1']
        createRL(
            tech, net, 'poly',
            ((0.925,2.77),(0.925,4.835),(0.465,4.835),(0.465,5.165),(0.925,5.165),(0.925,7.03),(1.075,7.03),(1.075,2.77),(0.925,2.77)),
        )
        createRL(
            tech, net, 'poly',
            ((3.925,1.47),(3.925,3.925),(2.925,3.925),(2.925,6.835),(2.765,6.835),(2.765,7.165),(2.925,7.165),(2.925,9.63),(3.075,9.63),(3.075,4.075),(4.075,4.075),(4.075,1.47),(3.925,1.47)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,8.63),(0.585,8.63),(0.585,7.085),(2.815,7.085),(2.815,7.165),(2.985,7.165),(2.985,7.085),(3.0,7.085),(3.0,6.915),(2.985,6.915),(2.985,6.835),(2.815,6.835),(2.815,6.915),(0.585,6.915),(0.585,5.165),(0.685,5.165),(0.685,4.835),(0.585,4.835),(0.585,1.37),(0.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,4.915),(0.515,5.085),(0.685,5.085),(0.685,4.915),(0.515,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.815,6.915),(2.815,7.085),(2.985,7.085),(2.985,6.915),(2.815,6.915)),
        )
        net = nets['_net8']
        createRL(
            tech, net, 'difftap',
            ((2.075,7.5),(2.075,9.5),(2.925,9.5),(2.925,7.5),(2.075,7.5)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((7.465,5.835),(7.465,6.165),(7.925,6.165),(7.925,9.63),(8.075,9.63),(8.075,5.835),(7.465,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((7.925,0.57),(7.925,3.935),(7.465,3.935),(7.465,4.265),(8.075,4.265),(8.075,0.57),(7.925,0.57)),
        )
        createRL(
            tech, net, 'licon',
            ((7.515,4.015),(7.515,4.185),(7.685,4.185),(7.685,4.015),(7.515,4.015)),
        )
        createRL(
            tech, net, 'licon',
            ((7.515,5.915),(7.515,6.085),(7.685,6.085),(7.685,5.915),(7.515,5.915)),
        )
        createRL(
            tech, net, 'li',
            ((7.415,3.255),(7.415,6.745),(7.585,6.745),(7.585,6.165),(7.685,6.165),(7.685,5.835),(7.585,5.835),(7.585,4.265),(7.685,4.265),(7.685,3.935),(7.585,3.935),(7.585,3.255),(7.415,3.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(7.5), u(0.17), u(3.255), u(6.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(7.5), u(0.17), u(3.255), u(6.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((4.925,1.47),(4.925,8.63),(5.075,8.63),(5.075,5.165),(5.535,5.165),(5.535,4.835),(5.075,4.835),(5.075,1.47),(4.925,1.47)),
        )
        createRL(
            tech, net, 'li',
            ((5.415,2.91),(5.415,4.835),(5.315,4.835),(5.315,5.165),(5.415,5.165),(5.415,6.745),(5.585,6.745),(5.585,2.91),(5.415,2.91)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(0.17), u(2.91), u(6.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(0.17), u(2.91), u(6.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((5.315,4.915),(5.315,5.085),(5.485,5.085),(5.485,4.915),(5.315,4.915)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((1.925,0.97),(1.925,9.63),(2.075,9.63),(2.075,5.165),(2.535,5.165),(2.535,4.835),(2.075,4.835),(2.075,0.97),(1.925,0.97)),
        )
        createRL(
            tech, net, 'licon',
            ((2.315,4.915),(2.315,5.085),(2.485,5.085),(2.485,4.915),(2.315,4.915)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,3.255),(2.415,4.835),(2.315,4.835),(2.315,5.165),(2.415,5.165),(2.415,5.745),(2.585,5.745),(2.585,3.255),(2.415,3.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(3.255), u(5.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(3.255), u(5.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['cmd0'].setDirection(Net.Direction.IN)
        nets['cmd1'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)

    cell = new_cells['nsnrlatch_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(5.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            'nq': Net.create(cell, 'nq'),
            'nrst': Net.create(cell, 'nrst'),
            'nset': Net.create(cell, 'nset'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,3.5),(0.925,3.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,6.5),(1.075,8.5),(1.925,8.5),(1.925,6.5),(1.075,6.5)),
        )
        createRL(
            tech, net, 'poly',
            ((2.925,1.37),(2.925,5.335),(2.765,5.335),(2.765,5.665),(2.925,5.665),(2.925,8.63),(3.075,8.63),(3.075,1.37),(2.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,3.345),(0.585,3.345),(0.585,2.15),(1.415,2.15),(1.415,8.63),(1.585,8.63),(1.585,5.585),(2.815,5.585),(2.815,5.665),(2.985,5.665),(2.985,5.335),(2.815,5.335),(2.815,5.415),(1.585,5.415),(1.585,1.37),(1.415,1.37),(1.415,1.85),(0.585,1.85),(0.585,1.37),(0.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.815,5.415),(2.815,5.585),(2.985,5.585),(2.985,5.415),(2.815,5.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.735),(0.415,1.905),(0.585,1.905),(0.585,1.735),(0.415,1.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.075),(0.415,2.245),(0.585,2.245),(0.585,2.075),(0.415,2.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.415),(0.415,2.585),(0.585,2.585),(0.585,2.415),(0.415,2.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.755),(0.415,2.925),(0.585,2.925),(0.585,2.755),(0.415,2.755)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,3.095),(0.415,3.265),(0.585,3.265),(0.585,3.095),(0.415,3.095)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,6.735),(1.415,6.905),(1.585,6.905),(1.585,6.735),(1.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.075),(1.415,7.245),(1.585,7.245),(1.585,7.075),(1.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.415),(1.415,7.585),(1.585,7.585),(1.585,7.415),(1.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.755),(1.415,7.925),(1.585,7.925),(1.585,7.755),(1.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,8.095),(1.415,8.265),(1.585,8.265),(1.585,8.095),(1.415,8.095)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((3.375,8.995),(3.375,9.0),(3.25,9.0),(3.25,9.4),(3.375,9.4),(3.375,9.405),(3.625,9.405),(3.625,9.4),(3.75,9.4),(3.75,9.0),(3.625,9.0),(3.625,8.995),(3.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,6.5),(1.925,8.5),(3.075,8.5),(3.075,6.5),(1.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,9.305),(0.625,9.305),(0.625,8.5),(1.075,8.5),(1.075,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,6.5),(3.925,8.5),(4.375,8.5),(4.375,9.305),(4.625,9.305),(4.625,6.5),(3.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,8.995),(1.375,9.0),(1.25,9.0),(1.25,9.4),(1.375,9.4),(1.375,9.405),(1.625,9.405),(1.625,9.4),(1.75,9.4),(1.75,9.0),(1.625,9.0),(1.625,8.995),(1.375,8.995)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(5.3,10.8),(5.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,6.655),(2.415,8.8),(0.0,8.8),(0.0,10.0),(5.0,10.0),(5.0,8.8),(2.585,8.8),(2.585,6.655),(2.415,6.655)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,9.115),(3.415,9.285),(3.585,9.285),(3.585,9.115),(3.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.015),(0.415,9.185),(0.585,9.185),(0.585,9.015),(0.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.115),(1.415,9.285),(1.585,9.285),(1.585,9.115),(1.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,6.735),(2.415,6.905),(2.585,6.905),(2.585,6.735),(2.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.075),(2.415,7.245),(2.585,7.245),(2.585,7.075),(2.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.415),(2.415,7.585),(2.585,7.585),(2.585,7.415),(2.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.755),(2.415,7.925),(2.585,7.925),(2.585,7.755),(2.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,8.095),(2.415,8.265),(2.585,8.265),(2.585,8.095),(2.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.015),(4.415,9.185),(4.585,9.185),(4.585,9.015),(4.415,9.015)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((3.25,0.37),(3.25,0.375),(3.125,0.375),(3.125,1.025),(3.25,1.025),(3.25,1.03),(3.75,1.03),(3.75,1.025),(3.875,1.025),(3.875,0.375),(3.75,0.375),(3.75,0.37),(3.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.25,6.375),(0.25,9.43),(0.75,9.43),(0.75,8.625),(4.25,8.625),(4.25,9.43),(4.75,9.43),(4.75,6.375),(0.25,6.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((1.25,0.37),(1.25,0.375),(1.125,0.375),(1.125,1.025),(1.25,1.025),(1.25,1.03),(1.75,1.03),(1.75,1.025),(1.875,1.025),(1.875,0.375),(1.75,0.375),(1.75,0.37),(1.25,0.37)),
        )
        createRL(
            tech, net, 'nsdm',
            ((3.25,8.87),(3.25,8.875),(3.125,8.875),(3.125,9.525),(3.25,9.525),(3.25,9.53),(3.75,9.53),(3.75,9.525),(3.875,9.525),(3.875,8.875),(3.75,8.875),(3.75,8.87),(3.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,1.375),(0.25,3.625),(4.75,3.625),(4.75,1.375),(0.25,1.375)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,8.87),(1.25,8.875),(1.125,8.875),(1.125,9.525),(1.25,9.525),(1.25,9.53),(1.75,9.53),(1.75,9.525),(1.875,9.525),(1.875,8.875),(1.75,8.875),(1.75,8.87),(1.25,8.87)),
        )
        net = nets['nq']
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,8.63),(2.075,8.63),(2.075,4.665),(2.235,4.665),(2.235,4.335),(2.075,4.335),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.075,1.5),(4.075,3.5),(4.625,3.5),(4.625,1.5),(4.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,6.5),(3.075,8.5),(3.925,8.5),(3.925,6.5),(3.075,6.5)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,1.37),(3.415,4.415),(2.185,4.415),(2.185,4.335),(2.015,4.335),(2.015,4.665),(2.185,4.665),(2.185,4.585),(3.415,4.585),(3.415,8.63),(3.585,8.63),(3.585,2.15),(4.415,2.15),(4.415,3.345),(4.585,3.345),(4.585,1.37),(4.415,1.37),(4.415,1.85),(3.585,1.85),(3.585,1.37),(3.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.015,4.415),(2.015,4.585),(2.185,4.585),(2.185,4.415),(2.015,4.415)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.735),(4.415,1.905),(4.585,1.905),(4.585,1.735),(4.415,1.735)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,2.075),(4.415,2.245),(4.585,2.245),(4.585,2.075),(4.415,2.075)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,2.415),(4.415,2.585),(4.585,2.585),(4.585,2.415),(4.415,2.415)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,2.755),(4.415,2.925),(4.585,2.925),(4.585,2.755),(4.415,2.755)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,3.095),(4.415,3.265),(4.585,3.265),(4.585,3.095),(4.415,3.095)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.735),(3.415,6.905),(3.585,6.905),(3.585,6.735),(3.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.075),(3.415,7.245),(3.585,7.245),(3.585,7.075),(3.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.415),(3.415,7.585),(3.585,7.585),(3.585,7.415),(3.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.755),(3.415,7.925),(3.585,7.925),(3.585,7.755),(3.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.095),(3.415,8.265),(3.585,8.265),(3.585,8.095),(3.415,8.095)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((1.075,1.5),(1.075,3.5),(1.925,3.5),(1.925,1.5),(1.075,1.5)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((3.925,1.5),(3.925,3.5),(4.075,3.5),(4.075,1.5),(3.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,1.5),(0.925,3.5),(1.075,3.5),(1.075,1.5),(0.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,1.5),(1.925,3.5),(3.075,3.5),(3.075,1.5),(1.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.495),(1.375,0.5),(1.25,0.5),(1.25,0.9),(1.375,0.9),(1.375,0.905),(1.625,0.905),(1.625,0.9),(1.75,0.9),(1.75,0.5),(1.625,0.5),(1.625,0.495),(1.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,0.495),(3.375,0.5),(3.25,0.5),(3.25,0.9),(3.375,0.9),(3.375,0.905),(3.625,0.905),(3.625,0.9),(3.75,0.9),(3.75,0.5),(3.625,0.5),(3.625,0.495),(3.375,0.495)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.415,1.2),(2.415,3.345),(2.585,3.345),(2.585,1.2),(5.0,1.2),(5.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,0.615),(1.415,0.785),(1.585,0.785),(1.585,0.615),(1.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.735),(2.415,1.905),(2.585,1.905),(2.585,1.735),(2.415,1.735)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.075),(2.415,2.245),(2.585,2.245),(2.585,2.075),(2.415,2.075)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.415),(2.415,2.585),(2.585,2.585),(2.585,2.415),(2.415,2.415)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.755),(2.415,2.925),(2.585,2.925),(2.585,2.755),(2.415,2.755)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,3.095),(2.415,3.265),(2.585,3.265),(2.585,3.095),(2.415,3.095)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,0.615),(3.415,0.785),(3.585,0.785),(3.585,0.615),(3.415,0.615)),
        )
        net = nets['nrst']
        createRL(
            tech, net, 'poly',
            ((3.925,1.37),(3.925,8.63),(4.075,8.63),(4.075,5.165),(4.535,5.165),(4.535,4.835),(4.075,4.835),(4.075,1.37),(3.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,4.0),(4.415,4.835),(4.315,4.835),(4.315,5.165),(4.415,5.165),(4.415,8.63),(4.585,8.63),(4.585,4.0),(4.415,4.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(4.5), u(0.17), u(4.0), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(4.5), u(0.17), u(4.0), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((4.315,4.915),(4.315,5.085),(4.485,5.085),(4.485,4.915),(4.315,4.915)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((3.075,1.5),(3.075,3.5),(3.925,3.5),(3.925,1.5),(3.075,1.5)),
        )
        net = nets['nset']
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,4.835),(0.465,4.835),(0.465,5.165),(0.925,5.165),(0.925,8.63),(1.075,8.63),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,4.0),(0.415,8.63),(0.585,8.63),(0.585,5.165),(0.685,5.165),(0.685,4.835),(0.585,4.835),(0.585,4.0),(0.415,4.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(4.0), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(4.0), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,4.915),(0.515,5.085),(0.685,5.085),(0.685,4.915),(0.515,4.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['nrst'].setDirection(Net.Direction.IN)
        nets['nset'].setDirection(Net.Direction.IN)

    cell = new_cells['nxr2_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(7.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            '_net3': Net.create(cell, '_net3'),
            '_net4': Net.create(cell, '_net4'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((3.075,5.5),(3.075,9.5),(3.925,9.5),(3.925,5.5),(3.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,0.6),(3.075,2.5),(3.925,2.5),(3.925,0.6),(3.075,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,1.37),(2.415,7.5),(2.585,7.5),(2.585,7.085),(3.415,7.085),(3.415,7.745),(3.585,7.745),(3.585,7.085),(3.6,7.085),(3.6,6.915),(3.585,6.915),(3.585,5.625),(3.415,5.625),(3.415,6.915),(2.585,6.915),(2.585,2.085),(3.415,2.085),(3.415,2.42),(3.585,2.42),(3.585,2.085),(3.6,2.085),(3.6,1.915),(3.585,1.915),(3.585,1.37),(3.415,1.37),(3.415,1.915),(2.585,1.915),(2.585,1.37),(2.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(1.37), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(1.37), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,1.49),(3.415,1.66),(3.585,1.66),(3.585,1.49),(3.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.83),(3.415,2.0),(3.585,2.0),(3.585,1.83),(3.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.17),(3.415,2.34),(3.585,2.34),(3.585,2.17),(3.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,5.705),(3.415,5.875),(3.585,5.875),(3.585,5.705),(3.415,5.705)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.045),(3.415,6.215),(3.585,6.215),(3.585,6.045),(3.415,6.045)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.385),(3.415,6.555),(3.585,6.555),(3.585,6.385),(3.415,6.385)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.725),(3.415,6.895),(3.585,6.895),(3.585,6.725),(3.415,6.725)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.065),(3.415,7.235),(3.585,7.235),(3.585,7.065),(3.415,7.065)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.405),(3.415,7.575),(3.585,7.575),(3.585,7.405),(3.415,7.405)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((2.925,0.6),(2.925,2.5),(3.075,2.5),(3.075,0.6),(2.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.35,0.4),(6.35,0.9),(6.375,0.9),(6.375,0.905),(6.625,0.905),(6.625,0.9),(6.65,0.9),(6.65,0.4),(6.35,0.4)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.925,0.6),(4.925,2.5),(6.075,2.5),(6.075,1.5),(5.625,1.5),(5.625,0.6),(4.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.35,0.4),(0.35,0.9),(0.375,0.9),(0.375,0.905),(0.625,0.905),(0.625,0.9),(0.65,0.9),(0.65,0.4),(0.35,0.4)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,0.6),(3.925,2.5),(4.075,2.5),(4.075,0.6),(3.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.6),(1.375,1.5),(0.925,1.5),(0.925,2.5),(2.075,2.5),(2.075,0.6),(1.375,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(7.0,1.2),(7.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,0.835),(1.415,1.005),(1.585,1.005),(1.585,0.835),(1.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,0.835),(5.415,1.005),(5.585,1.005),(5.585,0.835),(5.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.615),(0.415,0.785),(0.585,0.785),(0.585,0.615),(0.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,0.615),(6.415,0.785),(6.585,0.785),(6.585,0.615),(6.415,0.615)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((4.075,0.6),(4.075,2.5),(4.925,2.5),(4.925,0.6),(4.075,0.6)),
        )
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((6.25,8.87),(6.25,8.875),(6.225,8.875),(6.225,9.625),(6.775,9.625),(6.775,8.875),(6.75,8.875),(6.75,8.87),(6.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,0.475),(1.25,1.375),(0.25,1.375),(0.25,2.625),(6.75,2.625),(6.75,1.375),(5.75,1.375),(5.75,0.475),(1.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,8.87),(0.25,8.875),(0.225,8.875),(0.225,9.625),(0.775,9.625),(0.775,8.875),(0.75,8.875),(0.75,8.87),(0.25,8.87)),
        )
        createRL(
            tech, net, 'psdm',
            ((1.25,5.375),(1.25,6.375),(0.25,6.375),(0.25,8.625),(1.25,8.625),(1.25,9.625),(5.75,9.625),(5.75,8.625),(6.75,8.625),(6.75,6.375),(5.75,6.375),(5.75,5.375),(1.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((6.225,0.275),(6.225,1.025),(6.25,1.025),(6.25,1.03),(6.75,1.03),(6.75,1.025),(6.775,1.025),(6.775,0.275),(6.225,0.275)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.225,0.275),(0.225,1.025),(0.25,1.025),(0.25,1.03),(0.75,1.03),(0.75,1.025),(0.775,1.025),(0.775,0.275),(0.225,0.275)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'poly',
            ((3.925,0.47),(3.925,3.835),(0.465,3.835),(0.465,4.165),(3.925,4.165),(3.925,9.63),(4.075,9.63),(4.075,0.47),(3.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,8.5),(0.925,8.5),(0.925,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,2.5),(0.925,2.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'licon',
            ((0.515,3.915),(0.515,4.085),(0.685,4.085),(0.685,3.915),(0.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.745),(0.415,1.915),(0.585,1.915),(0.585,1.745),(0.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.085),(0.415,2.255),(0.585,2.255),(0.585,2.085),(0.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.735),(0.415,6.905),(0.585,6.905),(0.585,6.735),(0.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.075),(0.415,7.245),(0.585,7.245),(0.585,7.075),(0.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.415),(0.415,7.585),(0.585,7.585),(0.585,7.415),(0.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.755),(0.415,7.925),(0.585,7.925),(0.585,7.755),(0.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.095),(0.415,8.265),(0.585,8.265),(0.585,8.095),(0.415,8.095)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.665),(0.415,8.345),(0.585,8.345),(0.585,4.165),(0.685,4.165),(0.685,3.835),(0.585,3.835),(0.585,1.665),(0.415,1.665)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((6.375,8.995),(6.375,9.0),(6.35,9.0),(6.35,9.5),(6.65,9.5),(6.65,9.0),(6.625,9.0),(6.625,8.995),(6.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.925,5.5),(4.925,9.5),(5.625,9.5),(5.625,8.5),(6.075,8.5),(6.075,6.5),(5.625,6.5),(5.625,5.5),(4.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,5.5),(1.375,6.5),(0.925,6.5),(0.925,8.5),(1.375,8.5),(1.375,9.5),(2.075,9.5),(2.075,5.5),(1.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,8.995),(0.375,9.0),(0.35,9.0),(0.35,9.5),(0.65,9.5),(0.65,9.0),(0.625,9.0),(0.625,8.995),(0.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,5.5),(2.925,9.5),(3.075,9.5),(3.075,5.5),(2.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,5.5),(3.925,9.5),(4.075,9.5),(4.075,5.5),(3.925,5.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(7.3,10.8),(7.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(7.0,10.0),(7.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((6.415,9.115),(6.415,9.285),(6.585,9.285),(6.585,9.115),(6.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.045),(1.415,9.215),(1.585,9.215),(1.585,9.045),(1.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,9.045),(5.415,9.215),(5.585,9.215),(5.585,9.045),(5.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.115),(0.415,9.285),(0.585,9.285),(0.585,9.115),(0.415,9.115)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((4.075,5.5),(4.075,9.5),(4.925,9.5),(4.925,5.5),(4.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,5.5),(2.075,9.5),(2.925,9.5),(2.925,5.5),(2.075,5.5)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.895),(2.415,8.065),(2.585,8.065),(2.585,7.895),(2.415,7.895)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,8.235),(2.415,8.405),(2.585,8.405),(2.585,8.235),(2.415,8.235)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.64),(4.415,5.81),(4.585,5.81),(4.585,5.64),(4.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.98),(4.415,6.15),(4.585,6.15),(4.585,5.98),(4.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.32),(4.415,6.49),(4.585,6.49),(4.585,6.32),(4.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.66),(4.415,6.83),(4.585,6.83),(4.585,6.66),(4.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.0),(4.415,7.17),(4.585,7.17),(4.585,7.0),(4.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.34),(4.415,7.51),(4.585,7.51),(4.585,7.34),(4.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.68),(4.415,7.85),(4.585,7.85),(4.585,7.68),(4.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.02),(4.415,8.19),(4.585,8.19),(4.585,8.02),(4.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.36),(4.415,8.53),(4.585,8.53),(4.585,8.36),(4.415,8.36)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,5.56),(4.415,7.915),(2.585,7.915),(2.585,7.67),(2.415,7.67),(2.415,8.63),(2.585,8.63),(2.585,8.085),(4.415,8.085),(4.415,8.63),(4.585,8.63),(4.585,5.56),(4.415,5.56)),
        )
        net = nets['_net4']
        createRL(
            tech, net, 'difftap',
            ((2.075,0.6),(2.075,2.5),(2.925,2.5),(2.925,0.6),(2.075,0.6)),
        )
        net = nets['_net3']
        createRL(
            tech, net, 'poly',
            ((4.465,3.835),(4.465,4.165),(4.925,4.165),(4.925,9.63),(5.075,9.63),(5.075,4.165),(6.535,4.165),(6.535,3.835),(4.465,3.835)),
        )
        createRL(
            tech, net, 'poly',
            ((2.925,0.47),(2.925,3.165),(3.235,3.165),(3.235,2.835),(3.075,2.835),(3.075,0.47),(2.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.075,6.5),(6.075,8.5),(6.625,8.5),(6.625,6.5),(6.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.075,1.5),(6.075,2.5),(6.625,2.5),(6.625,1.5),(6.075,1.5)),
        )
        createRL(
            tech, net, 'licon',
            ((4.515,3.915),(4.515,4.085),(4.685,4.085),(4.685,3.915),(4.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,6.735),(6.415,6.905),(6.585,6.905),(6.585,6.735),(6.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.075),(6.415,7.245),(6.585,7.245),(6.585,7.075),(6.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.415),(6.415,7.585),(6.585,7.585),(6.585,7.415),(6.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.755),(6.415,7.925),(6.585,7.925),(6.585,7.755),(6.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.095),(6.415,8.265),(6.585,8.265),(6.585,8.095),(6.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((3.015,2.915),(3.015,3.085),(3.185,3.085),(3.185,2.915),(3.015,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,1.745),(6.415,1.915),(6.585,1.915),(6.585,1.745),(6.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,2.085),(6.415,2.255),(6.585,2.255),(6.585,2.085),(6.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((6.315,3.915),(6.315,4.085),(6.485,4.085),(6.485,3.915),(6.315,3.915)),
        )
        createRL(
            tech, net, 'li',
            ((3.015,2.835),(3.015,3.165),(3.185,3.165),(3.185,3.085),(4.415,3.085),(4.415,4.005),(4.515,4.005),(4.515,4.165),(4.685,4.165),(4.685,3.835),(4.585,3.835),(4.585,2.915),(3.185,2.915),(3.185,2.835),(3.015,2.835)),
        )
        createRL(
            tech, net, 'li',
            ((6.415,1.665),(6.415,3.835),(6.315,3.835),(6.315,4.165),(6.415,4.165),(6.415,8.345),(6.585,8.345),(6.585,1.665),(6.415,1.665)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.925,4.835),(0.925,8.63),(1.075,8.63),(1.075,5.165),(1.925,5.165),(1.925,9.63),(2.075,9.63),(2.075,4.835),(0.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((1.925,0.47),(1.925,2.835),(1.075,2.835),(1.075,1.37),(0.925,1.37),(0.925,3.165),(2.075,3.165),(2.075,0.47),(1.925,0.47)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.37),(1.415,8.63),(1.585,8.63),(1.585,1.37),(1.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,2.915),(1.415,3.085),(1.585,3.085),(1.585,2.915),(1.415,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,4.915),(1.415,5.085),(1.585,5.085),(1.585,4.915),(1.415,4.915)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((4.925,0.47),(4.925,3.165),(6.075,3.165),(6.075,1.37),(5.925,1.37),(5.925,2.835),(5.075,2.835),(5.075,0.47),(4.925,0.47)),
        )
        createRL(
            tech, net, 'poly',
            ((5.465,4.835),(5.465,5.165),(5.925,5.165),(5.925,8.63),(6.075,8.63),(6.075,4.835),(5.465,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((2.925,4.835),(2.925,9.63),(3.075,9.63),(3.075,5.165),(3.235,5.165),(3.235,4.835),(2.925,4.835)),
        )
        createRL(
            tech, net, 'li',
            ((5.415,1.37),(5.415,4.915),(3.185,4.915),(3.185,4.835),(3.015,4.835),(3.015,5.165),(3.185,5.165),(3.185,5.085),(5.415,5.085),(5.415,8.63),(5.585,8.63),(5.585,5.165),(5.685,5.165),(5.685,4.835),(5.585,4.835),(5.585,1.37),(5.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((5.415,2.915),(5.415,3.085),(5.585,3.085),(5.585,2.915),(5.415,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((5.515,4.915),(5.515,5.085),(5.685,5.085),(5.685,4.915),(5.515,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.015,4.915),(3.015,5.085),(3.185,5.085),(3.185,4.915),(3.015,4.915)),
        )
        nets['vss'].setGlobal(True)
        nets['vdd'].setGlobal(True)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)

    cell = new_cells['o2_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.925,5.5),(1.925,8.5),(2.375,8.5),(2.375,9.5),(3.075,9.5),(3.075,5.5),(1.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,5.5),(0.925,8.5),(1.075,8.5),(1.075,5.5),(0.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,8.995),(1.375,9.0),(1.3,9.0),(1.3,9.4),(1.375,9.4),(1.375,9.405),(1.625,9.405),(1.625,9.4),(1.7,9.4),(1.7,9.0),(1.625,9.0),(1.625,8.995),(1.375,8.995)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(4.3,10.8),(4.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.415,9.045),(2.415,9.215),(2.585,9.215),(2.585,9.045),(2.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.115),(1.415,9.285),(1.585,9.285),(1.585,9.115),(1.415,9.115)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((3.075,0.6),(3.075,2.5),(3.625,2.5),(3.625,0.6),(3.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,5.5),(3.075,9.5),(3.625,9.5),(3.625,5.5),(3.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,1.37),(3.415,8.63),(3.585,8.63),(3.585,1.37),(3.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,5.64),(3.415,5.81),(3.585,5.81),(3.585,5.64),(3.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,5.98),(3.415,6.15),(3.585,6.15),(3.585,5.98),(3.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.32),(3.415,6.49),(3.585,6.49),(3.585,6.32),(3.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.66),(3.415,6.83),(3.585,6.83),(3.585,6.66),(3.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.0),(3.415,7.17),(3.585,7.17),(3.585,7.0),(3.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.34),(3.415,7.51),(3.585,7.51),(3.585,7.34),(3.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.68),(3.415,7.85),(3.585,7.85),(3.585,7.68),(3.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.02),(3.415,8.19),(3.585,8.19),(3.585,8.02),(3.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.36),(3.415,8.53),(3.585,8.53),(3.585,8.36),(3.415,8.36)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.49),(3.415,1.66),(3.585,1.66),(3.585,1.49),(3.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.83),(3.415,2.0),(3.585,2.0),(3.585,1.83),(3.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.17),(3.415,2.34),(3.585,2.34),(3.585,2.17),(3.415,2.17)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.25,5.375),(0.25,8.625),(2.25,8.625),(2.25,9.625),(3.75,9.625),(3.75,5.375),(0.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((1.25,0.37),(1.25,0.375),(1.175,0.375),(1.175,1.025),(1.25,1.025),(1.25,1.03),(1.75,1.03),(1.75,1.025),(1.825,1.025),(1.825,0.375),(1.75,0.375),(1.75,0.37),(1.25,0.37)),
        )
        createRL(
            tech, net, 'nsdm',
            ((2.25,0.475),(2.25,1.375),(0.75,1.375),(0.75,0.57),(0.25,0.57),(0.25,2.625),(3.75,2.625),(3.75,0.475),(2.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,8.87),(1.25,8.875),(1.175,8.875),(1.175,9.525),(1.25,9.525),(1.25,9.53),(1.75,9.53),(1.75,9.525),(1.825,9.525),(1.825,8.875),(1.75,8.875),(1.75,8.87),(1.25,8.87)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'poly',
            ((2.925,0.47),(2.925,3.835),(1.465,3.835),(1.465,4.165),(2.925,4.165),(2.925,9.63),(3.075,9.63),(3.075,0.47),(2.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,5.5),(0.375,8.5),(0.925,8.5),(0.925,5.5),(0.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,1.5),(1.075,2.5),(1.925,2.5),(1.925,1.5),(1.075,1.5)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.665),(1.415,7.915),(0.585,7.915),(0.585,7.67),(0.415,7.67),(0.415,8.4),(0.585,8.4),(0.585,8.085),(1.585,8.085),(1.585,4.165),(1.685,4.165),(1.685,3.835),(1.585,3.835),(1.585,1.665),(1.415,1.665)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.81),(0.415,7.98),(0.585,7.98),(0.585,7.81),(0.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.15),(0.415,8.32),(0.585,8.32),(0.585,8.15),(0.415,8.15)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,1.745),(1.415,1.915),(1.585,1.915),(1.585,1.745),(1.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,2.085),(1.415,2.255),(1.585,2.255),(1.585,2.085),(1.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((1.515,3.915),(1.515,4.085),(1.685,4.085),(1.685,3.915),(1.515,3.915)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.375,0.695),(0.375,2.5),(1.075,2.5),(1.075,1.5),(0.625,1.5),(0.625,0.695),(0.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.495),(1.375,0.5),(1.3,0.5),(1.3,0.9),(1.375,0.9),(1.375,0.905),(1.625,0.905),(1.625,0.9),(1.7,0.9),(1.7,0.5),(1.625,0.5),(1.625,0.495),(1.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,0.6),(2.375,1.5),(1.925,1.5),(1.925,2.5),(3.075,2.5),(3.075,0.6),(2.375,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,0.615),(1.415,0.785),(1.585,0.785),(1.585,0.615),(1.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.815),(0.415,0.985),(0.585,0.985),(0.585,0.815),(0.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.835),(2.415,1.005),(2.585,1.005),(2.585,0.835),(2.415,0.835)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((1.075,5.5),(1.075,8.5),(1.925,8.5),(1.925,5.5),(1.075,5.5)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((1.925,4.835),(1.925,8.63),(2.075,8.63),(2.075,5.165),(2.535,5.165),(2.535,4.835),(1.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,3.165),(2.535,3.165),(2.535,2.835),(2.075,2.835),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,1.37),(2.415,2.835),(2.315,2.835),(2.315,3.165),(2.415,3.165),(2.415,4.835),(2.315,4.835),(2.315,5.165),(2.415,5.165),(2.415,8.63),(2.585,8.63),(2.585,1.37),(2.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.315,2.915),(2.315,3.085),(2.485,3.085),(2.485,2.915),(2.315,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.315,4.915),(2.315,5.085),(2.485,5.085),(2.485,4.915),(2.315,4.915)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,3.835),(0.465,3.835),(0.465,4.165),(0.925,4.165),(0.925,8.63),(1.075,8.63),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,7.5),(0.585,7.5),(0.585,4.165),(0.685,4.165),(0.685,3.835),(0.585,3.835),(0.585,1.37),(0.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(1.37), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(1.37), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,3.915),(0.515,4.085),(0.685,4.085),(0.685,3.915),(0.515,3.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)

    cell = new_cells['o3_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(5.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.925,5.5),(1.925,8.5),(2.075,8.5),(2.075,5.5),(1.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,5.5),(0.925,8.5),(1.075,8.5),(1.075,5.5),(0.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,5.5),(2.925,8.5),(3.375,8.5),(3.375,9.5),(4.075,9.5),(4.075,5.5),(2.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,8.995),(2.375,9.0),(2.3,9.0),(2.3,9.4),(2.375,9.4),(2.375,9.405),(2.625,9.405),(2.625,9.4),(2.7,9.4),(2.7,9.0),(2.625,9.0),(2.625,8.995),(2.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,8.995),(0.375,9.0),(0.35,9.0),(0.35,9.5),(0.65,9.5),(0.65,9.0),(0.625,9.0),(0.625,8.995),(0.375,8.995)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(5.3,10.8),(5.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,8.255),(3.415,8.8),(0.0,8.8),(0.0,10.0),(5.0,10.0),(5.0,8.8),(3.585,8.8),(3.585,8.255),(3.415,8.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.415,9.115),(2.415,9.285),(2.585,9.285),(2.585,9.115),(2.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.435),(3.415,8.605),(3.585,8.605),(3.585,8.435),(3.415,8.435)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.775),(3.415,8.945),(3.585,8.945),(3.585,8.775),(3.415,8.775)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,9.115),(3.415,9.285),(3.585,9.285),(3.585,9.115),(3.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.115),(0.415,9.285),(0.585,9.285),(0.585,9.115),(0.415,9.115)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((4.075,0.6),(4.075,2.5),(4.625,2.5),(4.625,0.6),(4.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.075,5.5),(4.075,9.5),(4.625,9.5),(4.625,5.5),(4.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,1.37),(4.415,8.63),(4.585,8.63),(4.585,1.37),(4.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((4.415,1.49),(4.415,1.66),(4.585,1.66),(4.585,1.49),(4.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.83),(4.415,2.0),(4.585,2.0),(4.585,1.83),(4.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,2.17),(4.415,2.34),(4.585,2.34),(4.585,2.17),(4.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.64),(4.415,5.81),(4.585,5.81),(4.585,5.64),(4.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.98),(4.415,6.15),(4.585,6.15),(4.585,5.98),(4.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.32),(4.415,6.49),(4.585,6.49),(4.585,6.32),(4.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.66),(4.415,6.83),(4.585,6.83),(4.585,6.66),(4.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.0),(4.415,7.17),(4.585,7.17),(4.585,7.0),(4.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.34),(4.415,7.51),(4.585,7.51),(4.585,7.34),(4.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.68),(4.415,7.85),(4.585,7.85),(4.585,7.68),(4.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.02),(4.415,8.19),(4.585,8.19),(4.585,8.02),(4.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.36),(4.415,8.53),(4.585,8.53),(4.585,8.36),(4.415,8.36)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.225,0.275),(0.225,1.025),(0.25,1.025),(0.25,1.03),(0.75,1.03),(0.75,1.025),(0.775,1.025),(0.775,0.275),(0.225,0.275)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.25,5.375),(0.25,8.625),(3.25,8.625),(3.25,9.625),(4.75,9.625),(4.75,5.375),(0.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((2.25,0.37),(2.25,0.375),(2.175,0.375),(2.175,1.025),(2.25,1.025),(2.25,1.03),(2.75,1.03),(2.75,1.025),(2.825,1.025),(2.825,0.375),(2.75,0.375),(2.75,0.37),(2.25,0.37)),
        )
        createRL(
            tech, net, 'nsdm',
            ((3.25,0.475),(3.25,1.375),(1.75,1.375),(1.75,0.57),(1.25,0.57),(1.25,1.375),(0.25,1.375),(0.25,2.625),(4.75,2.625),(4.75,0.475),(3.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,8.87),(0.25,8.875),(0.225,8.875),(0.225,9.625),(0.775,9.625),(0.775,8.875),(0.75,8.875),(0.75,8.87),(0.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((2.25,8.87),(2.25,8.875),(2.175,8.875),(2.175,9.525),(2.25,9.525),(2.25,9.53),(2.75,9.53),(2.75,9.525),(2.825,9.525),(2.825,8.875),(2.75,8.875),(2.75,8.87),(2.25,8.87)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'poly',
            ((3.925,0.47),(3.925,3.835),(3.465,3.835),(3.465,4.165),(3.925,4.165),(3.925,9.63),(4.075,9.63),(4.075,0.47),(3.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,1.5),(2.075,2.5),(2.925,2.5),(2.925,1.5),(2.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,5.5),(0.375,8.5),(0.925,8.5),(0.925,5.5),(0.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,2.5),(0.925,2.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.6),(0.415,2.33),(0.585,2.33),(0.585,2.085),(2.415,2.085),(2.415,2.33),(2.585,2.33),(2.585,2.085),(3.415,2.085),(3.415,7.915),(0.585,7.915),(0.585,7.67),(0.415,7.67),(0.415,8.4),(0.585,8.4),(0.585,8.085),(3.585,8.085),(3.585,4.165),(3.685,4.165),(3.685,3.835),(3.585,3.835),(3.585,1.915),(2.585,1.915),(2.585,1.6),(2.415,1.6),(2.415,1.915),(0.585,1.915),(0.585,1.6),(0.415,1.6)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.81),(0.415,7.98),(0.585,7.98),(0.585,7.81),(0.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.15),(0.415,8.32),(0.585,8.32),(0.585,8.15),(0.415,8.15)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.68),(0.415,1.85),(0.585,1.85),(0.585,1.68),(0.415,1.68)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.02),(0.415,2.19),(0.585,2.19),(0.585,2.02),(0.415,2.02)),
        )
        createRL(
            tech, net, 'licon',
            ((3.515,3.915),(3.515,4.085),(3.685,4.085),(3.685,3.915),(3.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.68),(2.415,1.85),(2.585,1.85),(2.585,1.68),(2.415,1.68)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.02),(2.415,2.19),(2.585,2.19),(2.585,2.02),(2.415,2.02)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.35,0.4),(0.35,0.9),(0.375,0.9),(0.375,0.905),(0.625,0.905),(0.625,0.9),(0.65,0.9),(0.65,0.4),(0.35,0.4)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.695),(1.375,1.5),(0.925,1.5),(0.925,2.5),(2.075,2.5),(2.075,1.5),(1.625,1.5),(1.625,0.695),(1.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,0.6),(3.375,1.5),(2.925,1.5),(2.925,2.5),(4.075,2.5),(4.075,0.6),(3.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,0.495),(2.375,0.5),(2.3,0.5),(2.3,0.9),(2.375,0.9),(2.375,0.905),(2.625,0.905),(2.625,0.9),(2.7,0.9),(2.7,0.5),(2.625,0.5),(2.625,0.495),(2.375,0.495)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(3.415,1.2),(3.415,1.745),(3.585,1.745),(3.585,1.2),(5.0,1.2),(5.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.415,0.615),(2.415,0.785),(2.585,0.785),(2.585,0.615),(2.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,0.815),(1.415,0.985),(1.585,0.985),(1.585,0.815),(1.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.615),(0.415,0.785),(0.585,0.785),(0.585,0.615),(0.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,0.765),(3.415,0.935),(3.585,0.935),(3.585,0.765),(3.415,0.765)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.105),(3.415,1.275),(3.585,1.275),(3.585,1.105),(3.415,1.105)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.445),(3.415,1.615),(3.585,1.615),(3.585,1.445),(3.415,1.445)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((2.925,1.37),(2.925,4.835),(2.465,4.835),(2.465,5.165),(2.925,5.165),(2.925,8.63),(3.075,8.63),(3.075,1.37),(2.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,2.5),(2.415,7.745),(2.585,7.745),(2.585,5.165),(2.685,5.165),(2.685,4.835),(2.585,4.835),(2.585,2.5),(2.415,2.5)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(2.5), u(7.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(2.5), u(7.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.515,4.915),(2.515,5.085),(2.685,5.085),(2.685,4.915),(2.515,4.915)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((2.075,5.5),(2.075,8.5),(2.925,8.5),(2.925,5.5),(2.075,5.5)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((1.075,5.5),(1.075,8.5),(1.925,8.5),(1.925,5.5),(1.075,5.5)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,2.835),(1.365,2.835),(1.365,3.165),(1.925,3.165),(1.925,8.63),(2.075,8.63),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,2.255),(1.415,7.745),(1.585,7.745),(1.585,2.255),(1.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(2.255), u(7.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(2.255), u(7.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,2.915),(1.415,3.085),(1.585,3.085),(1.585,2.915),(1.415,2.915)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,3.835),(0.365,3.835),(0.365,4.165),(0.925,4.165),(0.925,8.63),(1.075,8.63),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,2.5),(0.415,7.5),(0.585,7.5),(0.585,2.5),(0.415,2.5)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(2.5), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(2.5), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.415,3.915),(0.415,4.085),(0.585,4.085),(0.585,3.915),(0.415,3.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)

    cell = new_cells['o4_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(6.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            '_net3': Net.create(cell, '_net3'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'i3': Net.create(cell, 'i3'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.925,5.5),(1.925,8.5),(2.075,8.5),(2.075,5.5),(1.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,8.995),(3.375,9.0),(3.3,9.0),(3.3,9.4),(3.375,9.4),(3.375,9.405),(3.625,9.405),(3.625,9.4),(3.7,9.4),(3.7,9.0),(3.625,9.0),(3.625,8.995),(3.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,5.5),(0.925,8.5),(1.075,8.5),(1.075,5.5),(0.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,5.5),(2.925,8.5),(3.075,8.5),(3.075,5.5),(2.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,8.995),(1.375,9.0),(1.3,9.0),(1.3,9.4),(1.375,9.4),(1.375,9.405),(1.625,9.405),(1.625,9.4),(1.7,9.4),(1.7,9.0),(1.625,9.0),(1.625,8.995),(1.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,5.5),(3.925,8.5),(4.375,8.5),(4.375,9.5),(5.075,9.5),(5.075,5.5),(3.925,5.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(6.3,10.8),(6.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,8.255),(4.415,8.8),(0.0,8.8),(0.0,10.0),(6.0,10.0),(6.0,8.8),(4.585,8.8),(4.585,8.255),(4.415,8.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.0), u(6.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.0), u(6.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,9.115),(3.415,9.285),(3.585,9.285),(3.585,9.115),(3.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.435),(4.415,8.605),(4.585,8.605),(4.585,8.435),(4.415,8.435)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.775),(4.415,8.945),(4.585,8.945),(4.585,8.775),(4.415,8.775)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.115),(4.415,9.285),(4.585,9.285),(4.585,9.115),(4.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.115),(1.415,9.285),(1.585,9.285),(1.585,9.115),(1.415,9.115)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((5.075,0.6),(5.075,2.5),(5.625,2.5),(5.625,0.6),(5.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.075,5.5),(5.075,9.5),(5.625,9.5),(5.625,5.5),(5.075,5.5)),
        )
        createRL(
            tech, net, 'li',
            ((5.415,1.37),(5.415,8.63),(5.585,8.63),(5.585,1.37),(5.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((5.415,1.49),(5.415,1.66),(5.585,1.66),(5.585,1.49),(5.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,1.83),(5.415,2.0),(5.585,2.0),(5.585,1.83),(5.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,2.17),(5.415,2.34),(5.585,2.34),(5.585,2.17),(5.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,5.64),(5.415,5.81),(5.585,5.81),(5.585,5.64),(5.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,5.98),(5.415,6.15),(5.585,6.15),(5.585,5.98),(5.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,6.32),(5.415,6.49),(5.585,6.49),(5.585,6.32),(5.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,6.66),(5.415,6.83),(5.585,6.83),(5.585,6.66),(5.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.0),(5.415,7.17),(5.585,7.17),(5.585,7.0),(5.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.34),(5.415,7.51),(5.585,7.51),(5.585,7.34),(5.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,7.68),(5.415,7.85),(5.585,7.85),(5.585,7.68),(5.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,8.02),(5.415,8.19),(5.585,8.19),(5.585,8.02),(5.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,8.36),(5.415,8.53),(5.585,8.53),(5.585,8.36),(5.415,8.36)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((3.25,0.37),(3.25,0.375),(3.175,0.375),(3.175,1.025),(3.25,1.025),(3.25,1.03),(3.75,1.03),(3.75,1.025),(3.825,1.025),(3.825,0.375),(3.75,0.375),(3.75,0.37),(3.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.25,5.375),(0.25,8.625),(4.25,8.625),(4.25,9.625),(5.75,9.625),(5.75,5.375),(0.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((1.25,0.37),(1.25,0.375),(1.175,0.375),(1.175,1.025),(1.25,1.025),(1.25,1.03),(1.75,1.03),(1.75,1.025),(1.825,1.025),(1.825,0.375),(1.75,0.375),(1.75,0.37),(1.25,0.37)),
        )
        createRL(
            tech, net, 'nsdm',
            ((3.25,8.87),(3.25,8.875),(3.175,8.875),(3.175,9.525),(3.25,9.525),(3.25,9.53),(3.75,9.53),(3.75,9.525),(3.825,9.525),(3.825,8.875),(3.75,8.875),(3.75,8.87),(3.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((4.25,0.475),(4.25,1.375),(2.75,1.375),(2.75,0.57),(2.25,0.57),(2.25,1.375),(0.75,1.375),(0.75,0.57),(0.25,0.57),(0.25,2.625),(5.75,2.625),(5.75,0.475),(4.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,8.87),(1.25,8.875),(1.175,8.875),(1.175,9.525),(1.25,9.525),(1.25,9.53),(1.75,9.53),(1.75,9.525),(1.825,9.525),(1.825,8.875),(1.75,8.875),(1.75,8.87),(1.25,8.87)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'poly',
            ((4.925,0.47),(4.925,3.835),(4.465,3.835),(4.465,4.165),(4.925,4.165),(4.925,9.63),(5.075,9.63),(5.075,0.47),(4.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,5.5),(0.375,8.5),(0.925,8.5),(0.925,5.5),(0.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,1.5),(1.075,2.5),(1.925,2.5),(1.925,1.5),(1.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,1.5),(3.075,2.5),(3.925,2.5),(3.925,1.5),(3.075,1.5)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.6),(1.415,2.33),(1.585,2.33),(1.585,2.085),(3.415,2.085),(3.415,2.33),(3.585,2.33),(3.585,2.085),(4.415,2.085),(4.415,7.915),(0.585,7.915),(0.585,7.67),(0.415,7.67),(0.415,8.4),(0.585,8.4),(0.585,8.085),(4.585,8.085),(4.585,4.165),(4.685,4.165),(4.685,3.835),(4.585,3.835),(4.585,1.915),(3.585,1.915),(3.585,1.6),(3.415,1.6),(3.415,1.915),(1.585,1.915),(1.585,1.6),(1.415,1.6)),
        )
        createRL(
            tech, net, 'licon',
            ((4.515,3.915),(4.515,4.085),(4.685,4.085),(4.685,3.915),(4.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,1.68),(1.415,1.85),(1.585,1.85),(1.585,1.68),(1.415,1.68)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,2.02),(1.415,2.19),(1.585,2.19),(1.585,2.02),(1.415,2.02)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.81),(0.415,7.98),(0.585,7.98),(0.585,7.81),(0.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.15),(0.415,8.32),(0.585,8.32),(0.585,8.15),(0.415,8.15)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.68),(3.415,1.85),(3.585,1.85),(3.585,1.68),(3.415,1.68)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.02),(3.415,2.19),(3.585,2.19),(3.585,2.02),(3.415,2.02)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((2.375,0.695),(2.375,1.5),(1.925,1.5),(1.925,2.5),(3.075,2.5),(3.075,1.5),(2.625,1.5),(2.625,0.695),(2.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,0.695),(0.375,2.5),(1.075,2.5),(1.075,1.5),(0.625,1.5),(0.625,0.695),(0.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,0.6),(4.375,1.5),(3.925,1.5),(3.925,2.5),(5.075,2.5),(5.075,0.6),(4.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.495),(1.375,0.5),(1.3,0.5),(1.3,0.9),(1.375,0.9),(1.375,0.905),(1.625,0.905),(1.625,0.9),(1.7,0.9),(1.7,0.5),(1.625,0.5),(1.625,0.495),(1.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,0.495),(3.375,0.5),(3.3,0.5),(3.3,0.9),(3.375,0.9),(3.375,0.905),(3.625,0.905),(3.625,0.9),(3.7,0.9),(3.7,0.5),(3.625,0.5),(3.625,0.495),(3.375,0.495)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(4.415,1.2),(4.415,1.745),(4.585,1.745),(4.585,1.2),(6.0,1.2),(6.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.0), u(6.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.0), u(6.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,0.615),(3.415,0.785),(3.585,0.785),(3.585,0.615),(3.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,0.615),(1.415,0.785),(1.585,0.785),(1.585,0.615),(1.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.815),(0.415,0.985),(0.585,0.985),(0.585,0.815),(0.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,0.765),(4.415,0.935),(4.585,0.935),(4.585,0.765),(4.415,0.765)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.105),(4.415,1.275),(4.585,1.275),(4.585,1.105),(4.415,1.105)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.445),(4.415,1.615),(4.585,1.615),(4.585,1.445),(4.415,1.445)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.815),(2.415,0.985),(2.585,0.985),(2.585,0.815),(2.415,0.815)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((2.925,1.37),(2.925,3.835),(2.465,3.835),(2.465,4.165),(2.925,4.165),(2.925,8.63),(3.075,8.63),(3.075,1.37),(2.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,2.255),(2.415,7.745),(2.585,7.745),(2.585,4.165),(2.685,4.165),(2.685,3.835),(2.585,3.835),(2.585,2.255),(2.415,2.255)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(2.255), u(7.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(2.255), u(7.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((2.515,3.915),(2.515,4.085),(2.685,4.085),(2.685,3.915),(2.515,3.915)),
        )
        net = nets['_net3']
        createRL(
            tech, net, 'difftap',
            ((2.075,5.5),(2.075,8.5),(2.925,8.5),(2.925,5.5),(2.075,5.5)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((3.075,5.5),(3.075,8.5),(3.925,8.5),(3.925,5.5),(3.075,5.5)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((1.075,5.5),(1.075,8.5),(1.925,8.5),(1.925,5.5),(1.075,5.5)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,3.835),(1.465,3.835),(1.465,4.165),(1.925,4.165),(1.925,8.63),(2.075,8.63),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,2.5),(1.415,7.745),(1.585,7.745),(1.585,4.165),(1.685,4.165),(1.685,3.835),(1.585,3.835),(1.585,2.5),(1.415,2.5)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(2.5), u(7.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(2.5), u(7.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.515,3.915),(1.515,4.085),(1.685,4.085),(1.685,3.915),(1.515,3.915)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((3.925,1.37),(3.925,3.835),(3.465,3.835),(3.465,4.165),(3.925,4.165),(3.925,8.63),(4.075,8.63),(4.075,1.37),(3.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,2.5),(3.415,7.745),(3.585,7.745),(3.585,4.165),(3.685,4.165),(3.685,3.835),(3.585,3.835),(3.585,2.5),(3.415,2.5)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(0.17), u(2.5), u(7.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(0.17), u(2.5), u(7.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.515,3.915),(3.515,4.085),(3.685,4.085),(3.685,3.915),(3.515,3.915)),
        )
        net = nets['i3']
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,3.835),(0.465,3.835),(0.465,4.165),(0.925,4.165),(0.925,8.63),(1.075,8.63),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,7.5),(0.585,7.5),(0.585,4.165),(0.685,4.165),(0.685,3.835),(0.585,3.835),(0.585,1.37),(0.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(1.37), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(1.37), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,3.915),(0.515,4.085),(0.685,4.085),(0.685,3.915),(0.515,3.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)
        nets['i3'].setDirection(Net.Direction.IN)

    cell = new_cells['oa22_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(5.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.375,8.995),(1.375,9.0),(1.3,9.0),(1.3,9.4),(1.375,9.4),(1.375,9.405),(1.625,9.405),(1.625,9.4),(1.7,9.4),(1.7,9.0),(1.625,9.0),(1.625,8.995),(1.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,6.5),(1.925,8.5),(2.075,8.5),(2.075,6.5),(1.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,6.5),(0.925,8.5),(1.075,8.5),(1.075,6.5),(0.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,5.5),(3.375,6.5),(2.925,6.5),(2.925,8.5),(3.375,8.5),(3.375,9.5),(4.075,9.5),(4.075,5.5),(3.375,5.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(5.3,10.8),(5.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(5.0,10.0),(5.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,9.115),(1.415,9.285),(1.585,9.285),(1.585,9.115),(1.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,9.045),(3.415,9.215),(3.585,9.215),(3.585,9.045),(3.415,9.045)),
        )
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((4.075,5.5),(4.075,9.5),(4.625,9.5),(4.625,5.5),(4.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.075,0.6),(4.075,2.5),(4.625,2.5),(4.625,0.6),(4.075,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,1.37),(4.415,8.63),(4.585,8.63),(4.585,1.37),(4.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((4.415,1.49),(4.415,1.66),(4.585,1.66),(4.585,1.49),(4.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.83),(4.415,2.0),(4.585,2.0),(4.585,1.83),(4.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,2.17),(4.415,2.34),(4.585,2.34),(4.585,2.17),(4.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.64),(4.415,5.81),(4.585,5.81),(4.585,5.64),(4.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.98),(4.415,6.15),(4.585,6.15),(4.585,5.98),(4.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.32),(4.415,6.49),(4.585,6.49),(4.585,6.32),(4.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.66),(4.415,6.83),(4.585,6.83),(4.585,6.66),(4.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.0),(4.415,7.17),(4.585,7.17),(4.585,7.0),(4.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.34),(4.415,7.51),(4.585,7.51),(4.585,7.34),(4.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.68),(4.415,7.85),(4.585,7.85),(4.585,7.68),(4.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.02),(4.415,8.19),(4.585,8.19),(4.585,8.02),(4.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.36),(4.415,8.53),(4.585,8.53),(4.585,8.36),(4.415,8.36)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((3.25,5.375),(3.25,6.375),(0.25,6.375),(0.25,8.625),(3.25,8.625),(3.25,9.625),(4.75,9.625),(4.75,5.375),(3.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((1.25,0.37),(1.25,0.375),(1.175,0.375),(1.175,1.025),(1.25,1.025),(1.25,1.03),(1.75,1.03),(1.75,1.025),(1.825,1.025),(1.825,0.375),(1.75,0.375),(1.75,0.37),(1.25,0.37)),
        )
        createRL(
            tech, net, 'nsdm',
            ((2.25,0.475),(2.25,1.475),(0.75,1.475),(0.75,0.57),(0.25,0.57),(0.25,3.625),(2.75,3.625),(2.75,2.625),(4.75,2.625),(4.75,0.475),(2.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,8.87),(1.25,8.875),(1.175,8.875),(1.175,9.525),(1.25,9.525),(1.25,9.53),(1.75,9.53),(1.75,9.525),(1.825,9.525),(1.825,8.875),(1.75,8.875),(1.75,8.87),(1.25,8.87)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'poly',
            ((3.925,0.47),(3.925,3.835),(2.465,3.835),(2.465,4.165),(3.925,4.165),(3.925,9.63),(4.075,9.63),(4.075,0.47),(3.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,0.6),(2.375,1.6),(2.075,1.6),(2.075,3.5),(2.625,3.5),(2.625,2.5),(2.925,2.5),(2.925,0.6),(2.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.075,6.5),(1.075,8.5),(1.925,8.5),(1.925,6.5),(1.075,6.5)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.48),(2.415,1.65),(2.585,1.65),(2.585,1.48),(2.415,1.48)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.82),(2.415,1.99),(2.585,1.99),(2.585,1.82),(2.415,1.82)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.16),(2.415,2.33),(2.585,2.33),(2.585,2.16),(2.415,2.16)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.5),(2.415,2.67),(2.585,2.67),(2.585,2.5),(2.415,2.5)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.84),(2.415,3.01),(2.585,3.01),(2.585,2.84),(2.415,2.84)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,3.18),(2.415,3.35),(2.585,3.35),(2.585,3.18),(2.415,3.18)),
        )
        createRL(
            tech, net, 'licon',
            ((2.515,3.915),(2.515,4.085),(2.685,4.085),(2.685,3.915),(2.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,6.715),(1.415,6.885),(1.585,6.885),(1.585,6.715),(1.415,6.715)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.055),(1.415,7.225),(1.585,7.225),(1.585,7.055),(1.415,7.055)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,7.395),(1.415,7.565),(1.585,7.565),(1.585,7.395),(1.415,7.395)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,1.37),(2.415,6.915),(1.585,6.915),(1.585,6.635),(1.415,6.635),(1.415,7.745),(1.585,7.745),(1.585,7.085),(2.585,7.085),(2.585,4.165),(2.685,4.165),(2.685,3.835),(2.585,3.835),(2.585,1.37),(2.415,1.37)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.375,0.695),(0.375,3.5),(1.075,3.5),(1.075,1.6),(0.625,1.6),(0.625,0.695),(0.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.925,1.6),(1.925,3.5),(2.075,3.5),(2.075,1.6),(1.925,1.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.495),(1.375,0.5),(1.3,0.5),(1.3,0.9),(1.375,0.9),(1.375,0.905),(1.625,0.905),(1.625,0.9),(1.7,0.9),(1.7,0.5),(1.625,0.5),(1.625,0.495),(1.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,0.6),(2.925,2.5),(4.075,2.5),(4.075,0.6),(2.925,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(5.0,1.2),(5.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(5.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,0.835),(3.415,1.005),(3.585,1.005),(3.585,0.835),(3.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,0.615),(1.415,0.785),(1.585,0.785),(1.585,0.615),(1.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.815),(0.415,0.985),(0.585,0.985),(0.585,0.815),(0.415,0.815)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((1.075,1.6),(1.075,3.5),(1.925,3.5),(1.925,1.6),(1.075,1.6)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.925,1.47),(0.925,3.835),(0.465,3.835),(0.465,4.165),(0.925,4.165),(0.925,8.63),(1.075,8.63),(1.075,1.47),(0.925,1.47)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,7.5),(0.585,7.5),(0.585,4.165),(0.685,4.165),(0.685,3.835),(0.585,3.835),(0.585,1.37),(0.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(1.37), u(7.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(1.37), u(7.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((0.515,3.915),(0.515,4.085),(0.685,4.085),(0.685,3.915),(0.515,3.915)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((2.075,6.5),(2.075,8.5),(2.925,8.5),(2.925,6.5),(2.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,8.5),(0.925,8.5),(0.925,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,7.255),(2.415,7.915),(0.585,7.915),(0.585,7.67),(0.415,7.67),(0.415,8.4),(0.585,8.4),(0.585,8.085),(2.415,8.085),(2.415,8.365),(2.585,8.365),(2.585,7.255),(2.415,7.255)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.81),(0.415,7.98),(0.585,7.98),(0.585,7.81),(0.415,7.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.15),(0.415,8.32),(0.585,8.32),(0.585,8.15),(0.415,8.15)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.435),(2.415,7.605),(2.585,7.605),(2.585,7.435),(2.415,7.435)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.775),(2.415,7.945),(2.585,7.945),(2.585,7.775),(2.415,7.775)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,8.115),(2.415,8.285),(2.585,8.285),(2.585,8.115),(2.415,8.115)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.925,1.47),(1.925,4.835),(1.465,4.835),(1.465,5.165),(1.925,5.165),(1.925,8.63),(2.075,8.63),(2.075,1.47),(1.925,1.47)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.37),(1.415,6.465),(1.585,6.465),(1.585,5.165),(1.685,5.165),(1.685,4.835),(1.585,4.835),(1.585,1.37),(1.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(1.37), u(6.465),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(1.37), u(6.465),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.515,4.915),(1.515,5.085),(1.685,5.085),(1.685,4.915),(1.515,4.915)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((2.925,4.835),(2.925,8.63),(3.075,8.63),(3.075,5.165),(3.535,5.165),(3.535,4.835),(2.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((2.925,0.47),(2.925,3.165),(3.535,3.165),(3.535,2.835),(3.075,2.835),(3.075,0.47),(2.925,0.47)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,1.37),(3.415,2.835),(3.315,2.835),(3.315,3.165),(3.415,3.165),(3.415,4.835),(3.315,4.835),(3.315,5.165),(3.415,5.165),(3.415,8.63),(3.585,8.63),(3.585,1.37),(3.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.315,2.915),(3.315,3.085),(3.485,3.085),(3.485,2.915),(3.315,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.315,4.915),(3.315,5.085),(3.485,5.085),(3.485,4.915),(3.315,4.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)

    cell = new_cells['powmid_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(7.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.415,1.2),(0.415,2.165),(0.585,2.165),(0.585,1.2),(1.415,1.2),(1.415,2.165),(1.585,2.165),(1.585,1.2),(2.415,1.2),(2.415,2.165),(2.585,2.165),(2.585,1.2),(3.415,1.2),(3.415,2.165),(3.585,2.165),(3.585,1.2),(4.415,1.2),(4.415,2.165),(4.585,2.165),(4.585,1.2),(5.415,1.2),(5.415,2.165),(5.585,2.165),(5.585,1.2),(6.415,1.2),(6.415,2.165),(6.585,2.165),(6.585,1.2),(7.0,1.2),(7.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'm2',
            ((3.8,0.0),(3.8,10.0),(6.2,10.0),(6.2,0.0),(3.8,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('m2.pin'),
            u(5.0), u(2.4), u(0.0), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('m2'),
            u(5.0), u(2.4), u(0.0), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'm1',
            ((4.12,0.34),(4.12,0.35),(4.0,0.35),(4.0,0.65),(4.12,0.65),(4.12,0.66),(4.38,0.66),(4.38,0.65),(4.62,0.65),(4.62,0.66),(4.88,0.66),(4.88,0.65),(5.12,0.65),(5.12,0.66),(5.38,0.66),(5.38,0.65),(5.62,0.65),(5.62,0.66),(5.88,0.66),(5.88,0.65),(6.0,0.65),(6.0,0.35),(5.88,0.35),(5.88,0.34),(5.62,0.34),(5.62,0.35),(5.38,0.35),(5.38,0.34),(5.12,0.34),(5.12,0.35),(4.88,0.35),(4.88,0.34),(4.62,0.34),(4.62,0.35),(4.38,0.35),(4.38,0.34),(4.12,0.34)),
        )
        createRL(
            tech, net, 'via',
            ((5.175,0.425),(5.175,0.575),(5.325,0.575),(5.325,0.425),(5.175,0.425)),
        )
        createRL(
            tech, net, 'via',
            ((4.175,0.425),(4.175,0.575),(4.325,0.575),(4.325,0.425),(4.175,0.425)),
        )
        createRL(
            tech, net, 'via',
            ((4.675,0.425),(4.675,0.575),(4.825,0.575),(4.825,0.425),(4.675,0.425)),
        )
        createRL(
            tech, net, 'via',
            ((5.675,0.425),(5.675,0.575),(5.825,0.575),(5.825,0.425),(5.675,0.425)),
        )
        Horizontal.create(
            net, tech.getLayer('m1.pin'),
            u(0.5), u(0.3), u(4.0), u(6.0),
        )
        pin = Horizontal.create(
            net, tech.getLayer('m1'),
            u(0.5), u(0.3), u(4.0), u(6.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'mcon',
            ((5.165,0.415),(5.165,0.585),(5.335,0.585),(5.335,0.415),(5.165,0.415)),
        )
        createRL(
            tech, net, 'mcon',
            ((4.165,0.415),(4.165,0.585),(4.335,0.585),(4.335,0.415),(4.165,0.415)),
        )
        createRL(
            tech, net, 'mcon',
            ((4.665,0.415),(4.665,0.585),(4.835,0.585),(4.835,0.415),(4.665,0.415)),
        )
        createRL(
            tech, net, 'mcon',
            ((5.665,0.415),(5.665,0.585),(5.835,0.585),(5.835,0.415),(5.665,0.415)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,0.695),(0.375,1.105),(0.625,1.105),(0.625,1.05),(1.375,1.05),(1.375,1.105),(1.625,1.105),(1.625,1.05),(2.375,1.05),(2.375,1.105),(2.625,1.105),(2.625,1.05),(3.375,1.05),(3.375,1.105),(3.625,1.105),(3.625,1.05),(4.375,1.05),(4.375,1.105),(4.625,1.105),(4.625,1.05),(5.375,1.05),(5.375,1.105),(5.625,1.105),(5.625,1.05),(6.375,1.05),(6.375,1.105),(6.625,1.105),(6.625,0.695),(6.375,0.695),(6.375,0.75),(5.625,0.75),(5.625,0.695),(5.375,0.695),(5.375,0.75),(4.625,0.75),(4.625,0.695),(4.375,0.695),(4.375,0.75),(3.625,0.75),(3.625,0.695),(3.375,0.695),(3.375,0.75),(2.625,0.75),(2.625,0.695),(2.375,0.695),(2.375,0.75),(1.625,0.75),(1.625,0.695),(1.375,0.695),(1.375,0.75),(0.625,0.75),(0.625,0.695),(0.375,0.695)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,0.815),(3.415,0.985),(3.585,0.985),(3.585,0.815),(3.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,0.815),(5.415,0.985),(5.585,0.985),(5.585,0.815),(5.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.915),(2.415,2.085),(2.585,2.085),(2.585,1.915),(2.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,0.815),(1.415,0.985),(1.585,0.985),(1.585,0.815),(1.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,0.815),(4.415,0.985),(4.585,0.985),(4.585,0.815),(4.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.915),(3.415,2.085),(3.585,2.085),(3.585,1.915),(3.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,0.815),(6.415,0.985),(6.585,0.985),(6.585,0.815),(6.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,1.915),(5.415,2.085),(5.585,2.085),(5.585,1.915),(5.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.815),(0.415,0.985),(0.585,0.985),(0.585,0.815),(0.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,1.915),(4.415,2.085),(4.585,2.085),(4.585,1.915),(4.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,1.915),(1.415,2.085),(1.585,2.085),(1.585,1.915),(1.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,1.915),(6.415,2.085),(6.585,2.085),(6.585,1.915),(6.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.915),(0.415,2.085),(0.585,2.085),(0.585,1.915),(0.415,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.815),(2.415,0.985),(2.585,0.985),(2.585,0.815),(2.415,0.815)),
        )
        createRL(
            tech, net, 'poly',
            ((0.25,1.5),(0.25,8.5),(6.75,8.5),(6.75,1.5),(0.25,1.5)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(7.0,10.0),(7.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(7.3,10.8),(7.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'm1',
            ((1.12,9.34),(1.12,9.35),(1.0,9.35),(1.0,9.65),(1.12,9.65),(1.12,9.66),(1.38,9.66),(1.38,9.65),(1.62,9.65),(1.62,9.66),(1.88,9.66),(1.88,9.65),(2.12,9.65),(2.12,9.66),(2.38,9.66),(2.38,9.65),(2.62,9.65),(2.62,9.66),(2.88,9.66),(2.88,9.65),(3.0,9.65),(3.0,9.35),(2.88,9.35),(2.88,9.34),(2.62,9.34),(2.62,9.35),(2.38,9.35),(2.38,9.34),(2.12,9.34),(2.12,9.35),(1.88,9.35),(1.88,9.34),(1.62,9.34),(1.62,9.35),(1.38,9.35),(1.38,9.34),(1.12,9.34)),
        )
        Horizontal.create(
            net, tech.getLayer('m1.pin'),
            u(9.5), u(0.3), u(1.0), u(3.0),
        )
        pin = Horizontal.create(
            net, tech.getLayer('m1'),
            u(9.5), u(0.3), u(1.0), u(3.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'via',
            ((2.675,9.425),(2.675,9.575),(2.825,9.575),(2.825,9.425),(2.675,9.425)),
        )
        createRL(
            tech, net, 'via',
            ((1.675,9.425),(1.675,9.575),(1.825,9.575),(1.825,9.425),(1.675,9.425)),
        )
        createRL(
            tech, net, 'via',
            ((2.175,9.425),(2.175,9.575),(2.325,9.575),(2.325,9.425),(2.175,9.425)),
        )
        createRL(
            tech, net, 'via',
            ((1.175,9.425),(1.175,9.575),(1.325,9.575),(1.325,9.425),(1.175,9.425)),
        )
        createRL(
            tech, net, 'm2',
            ((0.8,0.0),(0.8,10.0),(3.2,10.0),(3.2,0.0),(0.8,0.0)),
        )
        createRL(
            tech, net, 'mcon',
            ((2.665,9.415),(2.665,9.585),(2.835,9.585),(2.835,9.415),(2.665,9.415)),
        )
        createRL(
            tech, net, 'mcon',
            ((1.665,9.415),(1.665,9.585),(1.835,9.585),(1.835,9.415),(1.665,9.415)),
        )
        createRL(
            tech, net, 'mcon',
            ((2.165,9.415),(2.165,9.585),(2.335,9.585),(2.335,9.415),(2.165,9.415)),
        )
        createRL(
            tech, net, 'mcon',
            ((1.165,9.415),(1.165,9.585),(1.335,9.585),(1.335,9.415),(1.165,9.415)),
        )
        Vertical.create(
            net, tech.getLayer('m2.pin'),
            u(2.0), u(2.4), u(0.0), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('m2'),
            u(2.0), u(2.4), u(0.0), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.375,8.895),(0.375,9.305),(0.625,9.305),(0.625,9.25),(1.375,9.25),(1.375,9.305),(1.625,9.305),(1.625,9.25),(2.375,9.25),(2.375,9.305),(2.625,9.305),(2.625,9.25),(3.375,9.25),(3.375,9.305),(3.625,9.305),(3.625,9.25),(4.375,9.25),(4.375,9.305),(4.625,9.305),(4.625,9.25),(5.375,9.25),(5.375,9.305),(5.625,9.305),(5.625,9.25),(6.375,9.25),(6.375,9.305),(6.625,9.305),(6.625,8.895),(6.375,8.895),(6.375,8.95),(5.625,8.95),(5.625,8.895),(5.375,8.895),(5.375,8.95),(4.625,8.95),(4.625,8.895),(4.375,8.895),(4.375,8.95),(3.625,8.95),(3.625,8.895),(3.375,8.895),(3.375,8.95),(2.625,8.95),(2.625,8.895),(2.375,8.895),(2.375,8.95),(1.625,8.95),(1.625,8.895),(1.375,8.895),(1.375,8.95),(0.625,8.95),(0.625,8.895),(0.375,8.895)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,9.015),(2.415,9.185),(2.585,9.185),(2.585,9.015),(2.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,9.015),(3.415,9.185),(3.585,9.185),(3.585,9.015),(3.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,9.015),(5.415,9.185),(5.585,9.185),(5.585,9.015),(5.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.015),(0.415,9.185),(0.585,9.185),(0.585,9.015),(0.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.015),(1.415,9.185),(1.585,9.185),(1.585,9.015),(1.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,9.015),(6.415,9.185),(6.585,9.185),(6.585,9.015),(6.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.015),(4.415,9.185),(4.585,9.185),(4.585,9.015),(4.415,9.015)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.25,0.57),(0.25,1.23),(0.75,1.23),(0.75,1.175),(1.25,1.175),(1.25,1.23),(1.75,1.23),(1.75,1.175),(2.25,1.175),(2.25,1.23),(2.75,1.23),(2.75,1.175),(3.25,1.175),(3.25,1.23),(3.75,1.23),(3.75,1.175),(4.25,1.175),(4.25,1.23),(4.75,1.23),(4.75,1.175),(5.25,1.175),(5.25,1.23),(5.75,1.23),(5.75,1.175),(6.25,1.175),(6.25,1.23),(6.75,1.23),(6.75,0.57),(6.25,0.57),(6.25,0.625),(5.75,0.625),(5.75,0.57),(5.25,0.57),(5.25,0.625),(4.75,0.625),(4.75,0.57),(4.25,0.57),(4.25,0.625),(3.75,0.625),(3.75,0.57),(3.25,0.57),(3.25,0.625),(2.75,0.625),(2.75,0.57),(2.25,0.57),(2.25,0.625),(1.75,0.625),(1.75,0.57),(1.25,0.57),(1.25,0.625),(0.75,0.625),(0.75,0.57),(0.25,0.57)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,8.77),(0.25,9.43),(0.75,9.43),(0.75,9.375),(1.25,9.375),(1.25,9.43),(1.75,9.43),(1.75,9.375),(2.25,9.375),(2.25,9.43),(2.75,9.43),(2.75,9.375),(3.25,9.375),(3.25,9.43),(3.75,9.43),(3.75,9.375),(4.25,9.375),(4.25,9.43),(4.75,9.43),(4.75,9.375),(5.25,9.375),(5.25,9.43),(5.75,9.43),(5.75,9.375),(6.25,9.375),(6.25,9.43),(6.75,9.43),(6.75,8.77),(6.25,8.77),(6.25,8.825),(5.75,8.825),(5.75,8.77),(5.25,8.77),(5.25,8.825),(4.75,8.825),(4.75,8.77),(4.25,8.77),(4.25,8.825),(3.75,8.825),(3.75,8.77),(3.25,8.77),(3.25,8.825),(2.75,8.825),(2.75,8.77),(2.25,8.77),(2.25,8.825),(1.75,8.825),(1.75,8.77),(1.25,8.77),(1.25,8.825),(0.75,8.825),(0.75,8.77),(0.25,8.77)),
        )
        nets['vss'].setGlobal(True)
        nets['vdd'].setGlobal(True)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)

    cell = new_cells['sff1_x4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(15.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            '_net4': Net.create(cell, '_net4'),
            '_net5': Net.create(cell, '_net5'),
            '_net6': Net.create(cell, '_net6'),
            'ck': Net.create(cell, 'ck'),
            'ckr': Net.create(cell, 'ckr'),
            'i': Net.create(cell, 'i'),
            'nckr': Net.create(cell, 'nckr'),
            'q': Net.create(cell, 'q'),
            'sff_m': Net.create(cell, 'sff_m'),
            'sff_s': Net.create(cell, 'sff_s'),
            'u': Net.create(cell, 'u'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
            'y': Net.create(cell, 'y'),
        }
        net = nets['nckr']
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,8.5),(0.925,8.5),(0.925,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,2.5),(0.925,2.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'poly',
            ((5.925,1.37),(5.925,3.165),(6.235,3.165),(6.235,2.835),(6.075,2.835),(6.075,1.37),(5.925,1.37)),
        )
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,4.835),(0.465,4.835),(0.465,5.165),(1.925,5.165),(1.925,8.63),(2.075,8.63),(2.075,5.165),(6.925,5.165),(6.925,8.63),(7.075,8.63),(7.075,5.165),(9.925,5.165),(9.925,8.63),(10.075,8.63),(10.075,5.165),(11.075,5.165),(11.075,1.37),(10.925,1.37),(10.925,4.835),(2.075,4.835),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.735),(0.415,6.905),(0.585,6.905),(0.585,6.735),(0.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.075),(0.415,7.245),(0.585,7.245),(0.585,7.075),(0.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.415),(0.415,7.585),(0.585,7.585),(0.585,7.415),(0.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.755),(0.415,7.925),(0.585,7.925),(0.585,7.755),(0.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.095),(0.415,8.265),(0.585,8.265),(0.585,8.095),(0.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.745),(0.415,1.915),(0.585,1.915),(0.585,1.745),(0.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.085),(0.415,2.255),(0.585,2.255),(0.585,2.085),(0.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((6.015,2.915),(6.015,3.085),(6.185,3.085),(6.185,2.915),(6.015,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((5.915,4.915),(5.915,5.085),(6.085,5.085),(6.085,4.915),(5.915,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.515,4.915),(0.515,5.085),(0.685,5.085),(0.685,4.915),(0.515,4.915)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.665),(0.415,8.345),(0.585,8.345),(0.585,5.165),(0.685,5.165),(0.685,4.835),(0.585,4.835),(0.585,1.665),(0.415,1.665)),
        )
        createRL(
            tech, net, 'li',
            ((6.015,2.835),(6.015,2.995),(5.915,2.995),(5.915,5.165),(6.085,5.165),(6.085,3.165),(6.185,3.165),(6.185,2.835),(6.015,2.835)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((13.925,5.5),(13.925,9.5),(14.625,9.5),(14.625,5.5),(13.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((9.925,6.5),(9.925,8.5),(10.075,8.5),(10.075,6.5),(9.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((7.925,7.5),(7.925,9.5),(9.075,9.5),(9.075,7.5),(7.925,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,7.5),(3.925,9.5),(5.075,9.5),(5.075,7.5),(3.925,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.925,6.5),(6.925,8.5),(7.075,8.5),(7.075,6.5),(6.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.375,8.995),(6.375,9.0),(6.25,9.0),(6.25,9.4),(6.375,9.4),(6.375,9.405),(6.625,9.405),(6.625,9.4),(6.75,9.4),(6.75,9.0),(6.625,9.0),(6.625,8.995),(6.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.925,6.5),(5.925,8.5),(6.075,8.5),(6.075,6.5),(5.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,8.995),(2.375,9.0),(2.3,9.0),(2.3,9.4),(2.375,9.4),(2.375,9.405),(2.625,9.405),(2.625,9.4),(2.7,9.4),(2.7,9.0),(2.625,9.0),(2.625,8.995),(2.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,6.5),(0.925,8.5),(1.375,8.5),(1.375,9.405),(1.625,9.405),(1.625,8.5),(2.075,8.5),(2.075,6.5),(0.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,8.995),(0.375,9.0),(0.35,9.0),(0.35,9.5),(0.65,9.5),(0.65,9.0),(0.625,9.0),(0.625,8.995),(0.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((12.375,5.5),(12.375,7.5),(11.925,7.5),(11.925,9.5),(13.075,9.5),(13.075,5.5),(12.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((10.375,8.995),(10.375,9.0),(10.25,9.0),(10.25,9.4),(10.375,9.4),(10.375,9.405),(10.625,9.405),(10.625,9.4),(10.75,9.4),(10.75,9.0),(10.625,9.0),(10.625,8.995),(10.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((10.925,6.5),(10.925,8.5),(11.075,8.5),(11.075,6.5),(10.925,6.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(15.3,10.8),(15.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((12.415,5.635),(12.415,8.8),(8.585,8.8),(8.585,7.655),(8.415,7.655),(8.415,8.8),(0.0,8.8),(0.0,10.0),(15.0,10.0),(15.0,8.8),(14.585,8.8),(14.585,5.635),(14.415,5.635),(14.415,8.8),(12.585,8.8),(12.585,5.635),(12.415,5.635)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(7.5), u(15.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(7.5), u(15.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((8.415,7.735),(8.415,7.905),(8.585,7.905),(8.585,7.735),(8.415,7.735)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.075),(8.415,8.245),(8.585,8.245),(8.585,8.075),(8.415,8.075)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.415),(8.415,8.585),(8.585,8.585),(8.585,8.415),(8.415,8.415)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.755),(8.415,8.925),(8.585,8.925),(8.585,8.755),(8.415,8.755)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,9.095),(8.415,9.265),(8.585,9.265),(8.585,9.095),(8.415,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,9.115),(2.415,9.285),(2.585,9.285),(2.585,9.115),(2.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.115),(0.415,9.285),(0.585,9.285),(0.585,9.115),(0.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,5.715),(12.415,5.885),(12.585,5.885),(12.585,5.715),(12.415,5.715)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,6.055),(12.415,6.225),(12.585,6.225),(12.585,6.055),(12.415,6.055)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,6.395),(12.415,6.565),(12.585,6.565),(12.585,6.395),(12.415,6.395)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,6.735),(12.415,6.905),(12.585,6.905),(12.585,6.735),(12.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,7.075),(12.415,7.245),(12.585,7.245),(12.585,7.075),(12.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,7.415),(12.415,7.585),(12.585,7.585),(12.585,7.415),(12.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,7.755),(12.415,7.925),(12.585,7.925),(12.585,7.755),(12.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,8.095),(12.415,8.265),(12.585,8.265),(12.585,8.095),(12.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,8.435),(12.415,8.605),(12.585,8.605),(12.585,8.435),(12.415,8.435)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,8.775),(12.415,8.945),(12.585,8.945),(12.585,8.775),(12.415,8.775)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,9.115),(12.415,9.285),(12.585,9.285),(12.585,9.115),(12.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.115),(1.415,9.285),(1.585,9.285),(1.585,9.115),(1.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,5.715),(14.415,5.885),(14.585,5.885),(14.585,5.715),(14.415,5.715)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,6.055),(14.415,6.225),(14.585,6.225),(14.585,6.055),(14.415,6.055)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,6.395),(14.415,6.565),(14.585,6.565),(14.585,6.395),(14.415,6.395)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,6.735),(14.415,6.905),(14.585,6.905),(14.585,6.735),(14.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,7.075),(14.415,7.245),(14.585,7.245),(14.585,7.075),(14.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,7.415),(14.415,7.585),(14.585,7.585),(14.585,7.415),(14.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,7.755),(14.415,7.925),(14.585,7.925),(14.585,7.755),(14.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,8.095),(14.415,8.265),(14.585,8.265),(14.585,8.095),(14.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,8.435),(14.415,8.605),(14.585,8.605),(14.585,8.435),(14.415,8.435)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,8.775),(14.415,8.945),(14.585,8.945),(14.585,8.775),(14.415,8.775)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,9.115),(14.415,9.285),(14.585,9.285),(14.585,9.115),(14.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,9.115),(6.415,9.285),(6.585,9.285),(6.585,9.115),(6.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,9.115),(10.415,9.285),(10.585,9.285),(10.585,9.115),(10.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.045),(4.415,9.215),(4.585,9.215),(4.585,9.045),(4.415,9.045)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((2.25,0.37),(2.25,0.375),(2.175,0.375),(2.175,1.025),(2.25,1.025),(2.25,1.03),(2.75,1.03),(2.75,1.025),(2.825,1.025),(2.825,0.375),(2.75,0.375),(2.75,0.37),(2.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.25,6.375),(0.25,8.625),(1.25,8.625),(1.25,9.53),(1.75,9.53),(1.75,8.625),(2.75,8.625),(2.75,6.375),(0.25,6.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((12.25,5.375),(12.25,7.375),(11.7,7.375),(11.7,6.375),(9.25,6.375),(9.25,7.375),(7.7,7.375),(7.7,6.375),(5.3,6.375),(5.3,7.375),(3.25,7.375),(3.25,9.625),(5.7,9.625),(5.7,8.625),(7.3,8.625),(7.3,9.625),(9.75,9.625),(9.75,8.625),(11.3,8.625),(11.3,9.625),(14.75,9.625),(14.75,5.375),(12.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((11.25,0.37),(11.25,0.375),(11.125,0.375),(11.125,1.025),(11.25,1.025),(11.25,1.03),(11.75,1.03),(11.75,1.025),(11.875,1.025),(11.875,0.375),(11.75,0.375),(11.75,0.37),(11.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((6.25,0.37),(6.25,0.375),(6.125,0.375),(6.125,1.025),(6.25,1.025),(6.25,1.03),(6.75,1.03),(6.75,1.025),(6.875,1.025),(6.875,0.375),(6.75,0.375),(6.75,0.37),(6.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.225,0.275),(0.225,1.025),(0.25,1.025),(0.25,1.03),(0.75,1.03),(0.75,1.025),(0.775,1.025),(0.775,0.275),(0.225,0.275)),
        )
        createRL(
            tech, net, 'nsdm',
            ((6.25,8.87),(6.25,8.875),(6.125,8.875),(6.125,9.525),(6.25,9.525),(6.25,9.53),(6.75,9.53),(6.75,9.525),(6.875,9.525),(6.875,8.875),(6.75,8.875),(6.75,8.87),(6.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,8.87),(0.25,8.875),(0.225,8.875),(0.225,9.625),(0.775,9.625),(0.775,8.875),(0.75,8.875),(0.75,8.87),(0.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((4.25,0.47),(4.25,1.375),(3.25,1.375),(3.25,2.625),(7.7,2.625),(7.7,1.625),(9.25,1.625),(9.25,2.625),(14.75,2.625),(14.75,0.475),(12.25,0.475),(12.25,1.375),(9.75,1.375),(9.75,0.475),(7.3,0.475),(7.3,1.375),(4.75,1.375),(4.75,0.47),(4.25,0.47)),
        )
        createRL(
            tech, net, 'nsdm',
            ((2.25,8.87),(2.25,8.875),(2.175,8.875),(2.175,9.525),(2.25,9.525),(2.25,9.53),(2.75,9.53),(2.75,9.525),(2.825,9.525),(2.825,8.875),(2.75,8.875),(2.75,8.87),(2.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((10.25,8.87),(10.25,8.875),(10.125,8.875),(10.125,9.525),(10.25,9.525),(10.25,9.53),(10.75,9.53),(10.75,9.525),(10.875,9.525),(10.875,8.875),(10.75,8.875),(10.75,8.87),(10.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,0.47),(1.25,1.375),(0.25,1.375),(0.25,2.625),(2.75,2.625),(2.75,1.375),(1.75,1.375),(1.75,0.47),(1.25,0.47)),
        )
        net = nets['ck']
        createRL(
            tech, net, 'poly',
            ((0.925,5.835),(0.925,8.63),(1.075,8.63),(1.075,6.165),(1.535,6.165),(1.535,5.835),(0.925,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,3.165),(1.535,3.165),(1.535,2.835),(1.075,2.835),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.37),(1.415,2.835),(1.315,2.835),(1.315,3.165),(1.415,3.165),(1.415,5.835),(1.315,5.835),(1.315,6.165),(1.415,6.165),(1.415,8.63),(1.585,8.63),(1.585,1.37),(1.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.315,2.915),(1.315,3.085),(1.485,3.085),(1.485,2.915),(1.315,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.315,5.915),(1.315,6.085),(1.485,6.085),(1.485,5.915),(1.315,5.915)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((13.925,0.6),(13.925,2.5),(14.625,2.5),(14.625,0.6),(13.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.375,0.495),(6.375,0.5),(6.25,0.5),(6.25,0.9),(6.375,0.9),(6.375,0.905),(6.625,0.905),(6.625,0.9),(6.75,0.9),(6.75,0.5),(6.625,0.5),(6.625,0.495),(6.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,0.595),(4.375,1.5),(3.925,1.5),(3.925,2.5),(5.075,2.5),(5.075,1.5),(4.625,1.5),(4.625,0.595),(4.375,0.595)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.595),(1.375,1.5),(0.925,1.5),(0.925,2.5),(2.075,2.5),(2.075,1.5),(1.625,1.5),(1.625,0.595),(1.375,0.595)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,0.495),(2.375,0.5),(2.3,0.5),(2.3,0.9),(2.375,0.9),(2.375,0.905),(2.625,0.905),(2.625,0.9),(2.7,0.9),(2.7,0.5),(2.625,0.5),(2.625,0.495),(2.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((12.375,0.6),(12.375,1.5),(11.925,1.5),(11.925,2.5),(13.075,2.5),(13.075,0.6),(12.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.925,1.5),(5.925,2.5),(6.075,2.5),(6.075,1.5),(5.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((7.925,0.6),(7.925,1.5),(9.075,1.5),(9.075,0.6),(7.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((10.925,1.5),(10.925,2.5),(11.075,2.5),(11.075,1.5),(10.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((11.375,0.495),(11.375,0.5),(11.25,0.5),(11.25,0.9),(11.375,0.9),(11.375,0.905),(11.625,0.905),(11.625,0.9),(11.75,0.9),(11.75,0.5),(11.625,0.5),(11.625,0.495),(11.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.925,1.5),(6.925,2.5),(7.075,2.5),(7.075,1.5),(6.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((9.925,1.5),(9.925,2.5),(10.075,2.5),(10.075,1.5),(9.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.35,0.4),(0.35,0.9),(0.375,0.9),(0.375,0.905),(0.625,0.905),(0.625,0.9),(0.65,0.9),(0.65,0.4),(0.35,0.4)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(8.415,1.2),(8.415,1.385),(8.585,1.385),(8.585,1.2),(12.415,1.2),(12.415,2.395),(12.585,2.395),(12.585,1.2),(14.415,1.2),(14.415,2.395),(14.585,2.395),(14.585,1.2),(15.0,1.2),(15.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(7.5), u(15.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(7.5), u(15.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((11.415,0.615),(11.415,0.785),(11.585,0.785),(11.585,0.615),(11.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,0.795),(8.415,0.965),(8.585,0.965),(8.585,0.795),(8.415,0.795)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,1.135),(8.415,1.305),(8.585,1.305),(8.585,1.135),(8.415,1.135)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,0.715),(1.415,0.885),(1.585,0.885),(1.585,0.715),(1.415,0.715)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,0.785),(14.415,0.955),(14.585,0.955),(14.585,0.785),(14.415,0.785)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,1.125),(14.415,1.295),(14.585,1.295),(14.585,1.125),(14.415,1.125)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,1.465),(14.415,1.635),(14.585,1.635),(14.585,1.465),(14.415,1.465)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,1.805),(14.415,1.975),(14.585,1.975),(14.585,1.805),(14.415,1.805)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,2.145),(14.415,2.315),(14.585,2.315),(14.585,2.145),(14.415,2.145)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,0.715),(4.415,0.885),(4.585,0.885),(4.585,0.715),(4.415,0.715)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,0.615),(6.415,0.785),(6.585,0.785),(6.585,0.615),(6.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,0.785),(12.415,0.955),(12.585,0.955),(12.585,0.785),(12.415,0.785)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,1.125),(12.415,1.295),(12.585,1.295),(12.585,1.125),(12.415,1.125)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,1.465),(12.415,1.635),(12.585,1.635),(12.585,1.465),(12.415,1.465)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,1.805),(12.415,1.975),(12.585,1.975),(12.585,1.805),(12.415,1.805)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,2.145),(12.415,2.315),(12.585,2.315),(12.585,2.145),(12.415,2.145)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.615),(0.415,0.785),(0.585,0.785),(0.585,0.615),(0.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.615),(2.415,0.785),(2.585,0.785),(2.585,0.615),(2.415,0.615)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((5.425,6.5),(5.425,7.5),(5.075,7.5),(5.075,9.5),(5.575,9.5),(5.575,8.5),(5.925,8.5),(5.925,6.5),(5.425,6.5)),
        )
        net = nets['sff_m']
        createRL(
            tech, net, 'difftap',
            ((6.075,6.5),(6.075,8.5),(6.925,8.5),(6.925,6.5),(6.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.075,1.5),(6.075,2.5),(6.925,2.5),(6.925,1.5),(6.075,1.5)),
        )
        createRL(
            tech, net, 'poly',
            ((8.925,0.47),(8.925,2.835),(8.615,2.835),(8.615,3.165),(9.075,3.165),(9.075,0.47),(8.925,0.47)),
        )
        createRL(
            tech, net, 'poly',
            ((8.615,5.835),(8.615,6.165),(8.925,6.165),(8.925,9.63),(9.075,9.63),(9.075,5.835),(8.615,5.835)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,1.745),(6.415,1.915),(6.585,1.915),(6.585,1.745),(6.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,2.085),(6.415,2.255),(6.585,2.255),(6.585,2.085),(6.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((8.665,2.915),(8.665,3.085),(8.835,3.085),(8.835,2.915),(8.665,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((8.665,5.915),(8.665,6.085),(8.835,6.085),(8.835,5.915),(8.665,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,6.735),(6.415,6.905),(6.585,6.905),(6.585,6.735),(6.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.075),(6.415,7.245),(6.585,7.245),(6.585,7.075),(6.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.415),(6.415,7.585),(6.585,7.585),(6.585,7.415),(6.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.755),(6.415,7.925),(6.585,7.925),(6.585,7.755),(6.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.095),(6.415,8.265),(6.585,8.265),(6.585,8.095),(6.415,8.095)),
        )
        createRL(
            tech, net, 'li',
            ((6.415,1.665),(6.415,2.335),(6.585,2.335),(6.585,2.085),(7.415,2.085),(7.415,7.415),(6.585,7.415),(6.585,6.655),(6.415,6.655),(6.415,8.345),(6.585,8.345),(6.585,7.585),(7.55,7.585),(7.55,7.55),(7.585,7.55),(7.585,6.085),(8.665,6.085),(8.665,6.165),(8.835,6.165),(8.835,5.835),(8.665,5.835),(8.665,5.915),(7.585,5.915),(7.585,3.085),(8.665,3.085),(8.665,3.165),(8.835,3.165),(8.835,2.835),(8.665,2.835),(8.665,2.915),(7.585,2.915),(7.585,1.95),(7.55,1.95),(7.55,1.915),(6.585,1.915),(6.585,1.665),(6.415,1.665)),
        )
        net = nets['ckr']
        createRL(
            tech, net, 'poly',
            ((5.925,5.835),(5.925,8.63),(6.075,8.63),(6.075,6.165),(6.335,6.165),(6.335,5.835),(5.925,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((10.465,5.835),(10.465,6.165),(10.925,6.165),(10.925,8.63),(11.075,8.63),(11.075,5.835),(10.465,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((6.925,1.37),(6.925,3.835),(2.465,3.835),(2.465,4.165),(10.535,4.165),(10.535,3.835),(10.075,3.835),(10.075,1.37),(9.925,1.37),(9.925,3.835),(7.075,3.835),(7.075,1.37),(6.925,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,1.5),(2.075,2.5),(2.625,2.5),(2.625,1.5),(2.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,6.5),(2.075,8.5),(2.625,8.5),(2.625,6.5),(2.075,6.5)),
        )
        createRL(
            tech, net, 'li',
            ((6.715,3.835),(6.715,3.995),(6.665,3.995),(6.665,5.915),(6.285,5.915),(6.285,5.835),(6.115,5.835),(6.115,6.165),(6.285,6.165),(6.285,6.085),(6.8,6.085),(6.8,6.05),(6.835,6.05),(6.835,4.165),(6.885,4.165),(6.885,3.835),(6.715,3.835)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,1.37),(2.415,8.63),(2.585,8.63),(2.585,4.165),(2.685,4.165),(2.685,3.835),(2.585,3.835),(2.585,1.37),(2.415,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((10.315,3.835),(10.315,4.165),(10.415,4.165),(10.415,6.05),(10.515,6.05),(10.515,6.165),(10.685,6.165),(10.685,5.835),(10.585,5.835),(10.585,3.95),(10.485,3.95),(10.485,3.835),(10.315,3.835)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.745),(2.415,1.915),(2.585,1.915),(2.585,1.745),(2.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.085),(2.415,2.255),(2.585,2.255),(2.585,2.085),(2.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((2.515,3.915),(2.515,4.085),(2.685,4.085),(2.685,3.915),(2.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((10.315,3.915),(10.315,4.085),(10.485,4.085),(10.485,3.915),(10.315,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.115,5.915),(6.115,6.085),(6.285,6.085),(6.285,5.915),(6.115,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,6.735),(2.415,6.905),(2.585,6.905),(2.585,6.735),(2.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.075),(2.415,7.245),(2.585,7.245),(2.585,7.075),(2.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.415),(2.415,7.585),(2.585,7.585),(2.585,7.415),(2.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.755),(2.415,7.925),(2.585,7.925),(2.585,7.755),(2.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,8.095),(2.415,8.265),(2.585,8.265),(2.585,8.095),(2.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((10.515,5.915),(10.515,6.085),(10.685,6.085),(10.685,5.915),(10.515,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.715,3.915),(6.715,4.085),(6.885,4.085),(6.885,3.915),(6.715,3.915)),
        )
        net = nets['y']
        createRL(
            tech, net, 'difftap',
            ((9.075,0.6),(9.075,1.5),(9.375,1.5),(9.375,2.5),(9.925,2.5),(9.925,1.5),(9.625,1.5),(9.625,0.6),(9.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((9.375,6.5),(9.375,7.5),(9.075,7.5),(9.075,9.5),(9.625,9.5),(9.625,8.5),(9.925,8.5),(9.925,6.5),(9.375,6.5)),
        )
        createRL(
            tech, net, 'poly',
            ((7.925,0.47),(7.925,2.165),(8.335,2.165),(8.335,1.835),(8.075,1.835),(8.075,0.47),(7.925,0.47)),
        )
        createRL(
            tech, net, 'poly',
            ((7.925,6.835),(7.925,9.63),(8.075,9.63),(8.075,7.165),(8.335,7.165),(8.335,6.835),(7.925,6.835)),
        )
        createRL(
            tech, net, 'licon',
            ((8.115,6.915),(8.115,7.085),(8.285,7.085),(8.285,6.915),(8.115,6.915)),
        )
        createRL(
            tech, net, 'licon',
            ((8.115,1.915),(8.115,2.085),(8.285,2.085),(8.285,1.915),(8.115,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,1.49),(9.415,1.66),(9.585,1.66),(9.585,1.49),(9.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,1.83),(9.415,2.0),(9.585,2.0),(9.585,1.83),(9.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,2.17),(9.415,2.34),(9.585,2.34),(9.585,2.17),(9.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,6.65),(9.415,6.82),(9.585,6.82),(9.585,6.65),(9.415,6.65)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,6.99),(9.415,7.16),(9.585,7.16),(9.585,6.99),(9.415,6.99)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,7.33),(9.415,7.5),(9.585,7.5),(9.585,7.33),(9.415,7.33)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,7.67),(9.415,7.84),(9.585,7.84),(9.585,7.67),(9.415,7.67)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,8.01),(9.415,8.18),(9.585,8.18),(9.585,8.01),(9.415,8.01)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,8.35),(9.415,8.52),(9.585,8.52),(9.585,8.35),(9.415,8.35)),
        )
        createRL(
            tech, net, 'li',
            ((9.415,1.37),(9.415,1.915),(8.285,1.915),(8.285,1.835),(8.115,1.835),(8.115,2.165),(8.285,2.165),(8.285,2.085),(9.415,2.085),(9.415,6.915),(8.285,6.915),(8.285,6.835),(8.115,6.835),(8.115,7.165),(8.285,7.165),(8.285,7.085),(9.415,7.085),(9.415,8.63),(9.585,8.63),(9.585,1.37),(9.415,1.37)),
        )
        net = nets['sff_s']
        createRL(
            tech, net, 'difftap',
            ((10.075,6.5),(10.075,8.5),(10.925,8.5),(10.925,6.5),(10.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((10.075,1.5),(10.075,2.5),(10.925,2.5),(10.925,1.5),(10.075,1.5)),
        )
        createRL(
            tech, net, 'poly',
            ((12.925,0.47),(12.925,3.835),(12.615,3.835),(12.615,4.165),(12.925,4.165),(12.925,9.63),(13.075,9.63),(13.075,4.165),(13.925,4.165),(13.925,9.63),(14.075,9.63),(14.075,0.47),(13.925,0.47),(13.925,3.835),(13.075,3.835),(13.075,0.47),(12.925,0.47)),
        )
        createRL(
            tech, net, 'li',
            ((10.415,1.37),(10.415,2.335),(10.585,2.335),(10.585,2.085),(11.415,2.085),(11.415,7.915),(10.585,7.915),(10.585,6.655),(10.415,6.655),(10.415,8.63),(10.585,8.63),(10.585,8.085),(11.55,8.085),(11.55,8.05),(11.585,8.05),(11.585,4.085),(12.665,4.085),(12.665,4.165),(12.835,4.165),(12.835,3.835),(12.665,3.835),(12.665,3.915),(11.585,3.915),(11.585,1.95),(11.55,1.95),(11.55,1.915),(10.585,1.915),(10.585,1.37),(10.415,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((12.665,3.915),(12.665,4.085),(12.835,4.085),(12.835,3.915),(12.665,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,1.745),(10.415,1.915),(10.585,1.915),(10.585,1.745),(10.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,2.085),(10.415,2.255),(10.585,2.255),(10.585,2.085),(10.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,6.735),(10.415,6.905),(10.585,6.905),(10.585,6.735),(10.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,7.075),(10.415,7.245),(10.585,7.245),(10.585,7.075),(10.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,7.415),(10.415,7.585),(10.585,7.585),(10.585,7.415),(10.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,7.755),(10.415,7.925),(10.585,7.925),(10.585,7.755),(10.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,8.095),(10.415,8.265),(10.585,8.265),(10.585,8.095),(10.415,8.095)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((11.075,6.5),(11.075,8.5),(11.425,8.5),(11.425,9.5),(11.925,9.5),(11.925,7.5),(11.575,7.5),(11.575,6.5),(11.075,6.5)),
        )
        net = nets['_net4']
        createRL(
            tech, net, 'difftap',
            ((7.425,0.6),(7.425,1.5),(7.075,1.5),(7.075,2.5),(7.575,2.5),(7.575,1.5),(7.925,1.5),(7.925,0.6),(7.425,0.6)),
        )
        net = nets['u']
        createRL(
            tech, net, 'difftap',
            ((3.375,1.5),(3.375,2.5),(3.925,2.5),(3.925,1.5),(3.375,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,7.5),(3.375,9.5),(3.925,9.5),(3.925,7.5),(3.375,7.5)),
        )
        createRL(
            tech, net, 'poly',
            ((3.465,5.835),(3.465,6.165),(4.925,6.165),(4.925,9.63),(5.075,9.63),(5.075,6.165),(5.335,6.165),(5.335,5.835),(3.465,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((4.925,1.37),(4.925,3.165),(5.335,3.165),(5.335,2.835),(5.075,2.835),(5.075,1.37),(4.925,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.745),(3.415,1.915),(3.585,1.915),(3.585,1.745),(3.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.085),(3.415,2.255),(3.585,2.255),(3.585,2.085),(3.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((5.115,2.915),(5.115,3.085),(5.285,3.085),(5.285,2.915),(5.115,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((5.115,5.915),(5.115,6.085),(5.285,6.085),(5.285,5.915),(5.115,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.515,5.915),(3.515,6.085),(3.685,6.085),(3.685,5.915),(3.515,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.66),(3.415,7.83),(3.585,7.83),(3.585,7.66),(3.415,7.66)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.0),(3.415,8.17),(3.585,8.17),(3.585,8.0),(3.415,8.0)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.34),(3.415,8.51),(3.585,8.51),(3.585,8.34),(3.415,8.34)),
        )
        createRL(
            tech, net, 'li',
            ((5.115,2.835),(5.115,6.165),(5.285,6.165),(5.285,2.835),(5.115,2.835)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,1.37),(3.415,8.63),(3.585,8.63),(3.585,6.165),(3.685,6.165),(3.685,5.835),(3.585,5.835),(3.585,1.37),(3.415,1.37)),
        )
        net = nets['i']
        createRL(
            tech, net, 'poly',
            ((3.925,6.835),(3.925,9.63),(4.075,9.63),(4.075,7.165),(4.535,7.165),(4.535,6.835),(3.925,6.835)),
        )
        createRL(
            tech, net, 'poly',
            ((3.925,1.37),(3.925,3.165),(4.535,3.165),(4.535,2.835),(4.075,2.835),(4.075,1.37),(3.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,1.37),(4.415,2.835),(4.315,2.835),(4.315,3.165),(4.415,3.165),(4.415,6.835),(4.315,6.835),(4.315,7.165),(4.415,7.165),(4.415,8.63),(4.585,8.63),(4.585,1.37),(4.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((4.315,6.915),(4.315,7.085),(4.485,7.085),(4.485,6.915),(4.315,6.915)),
        )
        createRL(
            tech, net, 'licon',
            ((4.315,2.915),(4.315,3.085),(4.485,3.085),(4.485,2.915),(4.315,2.915)),
        )
        net = nets['_net5']
        createRL(
            tech, net, 'difftap',
            ((7.075,6.5),(7.075,8.5),(7.425,8.5),(7.425,9.5),(7.925,9.5),(7.925,7.5),(7.575,7.5),(7.575,6.5),(7.075,6.5)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((5.075,1.5),(5.075,2.5),(5.925,2.5),(5.925,1.5),(5.075,1.5)),
        )
        net = nets['_net6']
        createRL(
            tech, net, 'difftap',
            ((11.075,1.5),(11.075,2.5),(11.925,2.5),(11.925,1.5),(11.075,1.5)),
        )
        net = nets['q']
        createRL(
            tech, net, 'poly',
            ((11.925,1.37),(11.925,3.165),(12.385,3.165),(12.385,2.835),(12.075,2.835),(12.075,1.37),(11.925,1.37)),
        )
        createRL(
            tech, net, 'poly',
            ((11.925,4.835),(11.925,9.63),(12.075,9.63),(12.075,5.165),(12.385,5.165),(12.385,4.835),(11.925,4.835)),
        )
        createRL(
            tech, net, 'difftap',
            ((13.075,5.5),(13.075,9.5),(13.925,9.5),(13.925,5.5),(13.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((13.075,0.6),(13.075,2.5),(13.925,2.5),(13.925,0.6),(13.075,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((13.415,1.37),(13.415,2.915),(12.335,2.915),(12.335,2.835),(12.165,2.835),(12.165,3.165),(12.335,3.165),(12.335,3.085),(13.415,3.085),(13.415,4.915),(12.335,4.915),(12.335,4.835),(12.165,4.835),(12.165,5.165),(12.335,5.165),(12.335,5.085),(13.415,5.085),(13.415,8.63),(13.585,8.63),(13.585,1.37),(13.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(13.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(13.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((13.415,5.64),(13.415,5.81),(13.585,5.81),(13.585,5.64),(13.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,5.98),(13.415,6.15),(13.585,6.15),(13.585,5.98),(13.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,6.32),(13.415,6.49),(13.585,6.49),(13.585,6.32),(13.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,6.66),(13.415,6.83),(13.585,6.83),(13.585,6.66),(13.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,7.0),(13.415,7.17),(13.585,7.17),(13.585,7.0),(13.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,7.34),(13.415,7.51),(13.585,7.51),(13.585,7.34),(13.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,7.68),(13.415,7.85),(13.585,7.85),(13.585,7.68),(13.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,8.02),(13.415,8.19),(13.585,8.19),(13.585,8.02),(13.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,8.36),(13.415,8.53),(13.585,8.53),(13.585,8.36),(13.415,8.36)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,1.49),(13.415,1.66),(13.585,1.66),(13.585,1.49),(13.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,1.83),(13.415,2.0),(13.585,2.0),(13.585,1.83),(13.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,2.17),(13.415,2.34),(13.585,2.34),(13.585,2.17),(13.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((12.165,2.915),(12.165,3.085),(12.335,3.085),(12.335,2.915),(12.165,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((12.165,4.915),(12.165,5.085),(12.335,5.085),(12.335,4.915),(12.165,4.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['ck'].setType(Net.Type.CLOCK)
        nets['ck'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)

    cell = new_cells['sff1r_x4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(18.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net2': Net.create(cell, '_net2'),
            '_net3': Net.create(cell, '_net3'),
            '_net4': Net.create(cell, '_net4'),
            '_net5': Net.create(cell, '_net5'),
            '_net6': Net.create(cell, '_net6'),
            '_net7': Net.create(cell, '_net7'),
            '_net8': Net.create(cell, '_net8'),
            'ck': Net.create(cell, 'ck'),
            'ckr': Net.create(cell, 'ckr'),
            'i': Net.create(cell, 'i'),
            'nckr': Net.create(cell, 'nckr'),
            'nrst': Net.create(cell, 'nrst'),
            'q': Net.create(cell, 'q'),
            'sff_m': Net.create(cell, 'sff_m'),
            'sff_s': Net.create(cell, 'sff_s'),
            'u': Net.create(cell, 'u'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
            'y': Net.create(cell, 'y'),
        }
        net = nets['nckr']
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,8.5),(0.925,8.5),(0.925,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,2.5),(0.925,2.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'poly',
            ((5.925,1.37),(5.925,3.165),(6.235,3.165),(6.235,2.835),(6.075,2.835),(6.075,1.37),(5.925,1.37)),
        )
        createRL(
            tech, net, 'poly',
            ((1.925,1.37),(1.925,4.835),(0.465,4.835),(0.465,5.165),(1.925,5.165),(1.925,8.63),(2.075,8.63),(2.075,5.165),(6.925,5.165),(6.925,8.63),(7.075,8.63),(7.075,5.165),(12.925,5.165),(12.925,8.63),(13.075,8.63),(13.075,1.37),(12.925,1.37),(12.925,4.835),(2.075,4.835),(2.075,1.37),(1.925,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.735),(0.415,6.905),(0.585,6.905),(0.585,6.735),(0.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.075),(0.415,7.245),(0.585,7.245),(0.585,7.075),(0.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.415),(0.415,7.585),(0.585,7.585),(0.585,7.415),(0.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.755),(0.415,7.925),(0.585,7.925),(0.585,7.755),(0.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.095),(0.415,8.265),(0.585,8.265),(0.585,8.095),(0.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.745),(0.415,1.915),(0.585,1.915),(0.585,1.745),(0.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.085),(0.415,2.255),(0.585,2.255),(0.585,2.085),(0.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((6.015,2.915),(6.015,3.085),(6.185,3.085),(6.185,2.915),(6.015,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((5.915,4.915),(5.915,5.085),(6.085,5.085),(6.085,4.915),(5.915,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.515,4.915),(0.515,5.085),(0.685,5.085),(0.685,4.915),(0.515,4.915)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.665),(0.415,8.345),(0.585,8.345),(0.585,5.165),(0.685,5.165),(0.685,4.835),(0.585,4.835),(0.585,1.665),(0.415,1.665)),
        )
        createRL(
            tech, net, 'li',
            ((6.015,2.835),(6.015,2.995),(5.915,2.995),(5.915,5.165),(6.085,5.165),(6.085,3.165),(6.185,3.165),(6.185,2.835),(6.015,2.835)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((6.375,0.495),(6.375,0.5),(6.3,0.5),(6.3,0.9),(6.375,0.9),(6.375,0.905),(6.625,0.905),(6.625,0.9),(6.7,0.9),(6.7,0.5),(6.625,0.5),(6.625,0.495),(6.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.695),(1.375,1.5),(0.925,1.5),(0.925,2.5),(2.075,2.5),(2.075,1.5),(1.625,1.5),(1.625,0.695),(1.375,0.695)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,0.495),(2.375,0.5),(2.3,0.5),(2.3,0.9),(2.375,0.9),(2.375,0.905),(2.625,0.905),(2.625,0.9),(2.7,0.9),(2.7,0.5),(2.625,0.5),(2.625,0.495),(2.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((9.925,0.6),(9.925,1.5),(10.075,1.5),(10.075,0.6),(9.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.925,1.5),(6.925,2.5),(7.075,2.5),(7.075,1.5),(6.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((7.925,0.6),(7.925,1.5),(9.075,1.5),(9.075,0.6),(7.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((16.925,0.6),(16.925,2.5),(17.625,2.5),(17.625,0.6),(16.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((12.925,1.5),(12.925,2.5),(13.075,2.5),(13.075,1.5),(12.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.925,1.5),(5.925,2.5),(6.075,2.5),(6.075,1.5),(5.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((13.925,1.5),(13.925,2.5),(14.075,2.5),(14.075,1.5),(13.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.35,0.4),(0.35,0.9),(0.375,0.9),(0.375,0.905),(0.625,0.905),(0.625,0.9),(0.65,0.9),(0.65,0.4),(0.35,0.4)),
        )
        createRL(
            tech, net, 'difftap',
            ((11.925,1.5),(11.925,2.5),(12.075,2.5),(12.075,1.5),(11.925,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((15.375,0.6),(15.375,1.5),(14.925,1.5),(14.925,2.5),(16.075,2.5),(16.075,0.6),(15.375,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((14.375,0.495),(14.375,0.5),(14.3,0.5),(14.3,0.9),(14.375,0.9),(14.375,0.905),(14.625,0.905),(14.625,0.9),(14.7,0.9),(14.7,0.5),(14.625,0.5),(14.625,0.495),(14.375,0.495)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.375,0.695),(4.375,1.5),(3.925,1.5),(3.925,2.5),(5.075,2.5),(5.075,1.5),(4.625,1.5),(4.625,0.695),(4.375,0.695)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(8.415,1.2),(8.415,1.385),(8.585,1.385),(8.585,1.2),(15.415,1.2),(15.415,2.395),(15.585,2.395),(15.585,1.2),(17.415,1.2),(17.415,2.395),(17.585,2.395),(17.585,1.2),(18.0,1.2),(18.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(9.0), u(18.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(9.0), u(18.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((17.415,0.785),(17.415,0.955),(17.585,0.955),(17.585,0.785),(17.415,0.785)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,1.125),(17.415,1.295),(17.585,1.295),(17.585,1.125),(17.415,1.125)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,1.465),(17.415,1.635),(17.585,1.635),(17.585,1.465),(17.415,1.465)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,1.805),(17.415,1.975),(17.585,1.975),(17.585,1.805),(17.415,1.805)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,2.145),(17.415,2.315),(17.585,2.315),(17.585,2.145),(17.415,2.145)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,0.795),(8.415,0.965),(8.585,0.965),(8.585,0.795),(8.415,0.795)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,1.135),(8.415,1.305),(8.585,1.305),(8.585,1.135),(8.415,1.135)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,0.815),(1.415,0.985),(1.585,0.985),(1.585,0.815),(1.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,0.815),(4.415,0.985),(4.585,0.985),(4.585,0.815),(4.415,0.815)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,0.615),(14.415,0.785),(14.585,0.785),(14.585,0.615),(14.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,0.785),(15.415,0.955),(15.585,0.955),(15.585,0.785),(15.415,0.785)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,1.125),(15.415,1.295),(15.585,1.295),(15.585,1.125),(15.415,1.125)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,1.465),(15.415,1.635),(15.585,1.635),(15.585,1.465),(15.415,1.465)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,1.805),(15.415,1.975),(15.585,1.975),(15.585,1.805),(15.415,1.805)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,2.145),(15.415,2.315),(15.585,2.315),(15.585,2.145),(15.415,2.145)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,0.615),(6.415,0.785),(6.585,0.785),(6.585,0.615),(6.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.615),(0.415,0.785),(0.585,0.785),(0.585,0.615),(0.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,0.615),(2.415,0.785),(2.585,0.785),(2.585,0.615),(2.415,0.615)),
        )
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((7.3,0.475),(7.3,1.375),(4.75,1.375),(4.75,0.57),(4.25,0.57),(4.25,1.375),(3.25,1.375),(3.25,2.625),(7.7,2.625),(7.7,1.625),(10.25,1.625),(10.25,2.625),(17.75,2.625),(17.75,0.475),(15.25,0.475),(15.25,1.375),(10.75,1.375),(10.75,0.475),(7.3,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((6.25,8.87),(6.25,8.875),(6.175,8.875),(6.175,9.525),(6.25,9.525),(6.25,9.53),(6.75,9.53),(6.75,9.525),(6.825,9.525),(6.825,8.875),(6.75,8.875),(6.75,8.87),(6.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((13.25,8.87),(13.25,8.875),(13.175,8.875),(13.175,9.525),(13.25,9.525),(13.25,9.53),(13.75,9.53),(13.75,9.525),(13.825,9.525),(13.825,8.875),(13.75,8.875),(13.75,8.87),(13.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,8.87),(0.25,8.875),(0.225,8.875),(0.225,9.625),(0.775,9.625),(0.775,8.875),(0.75,8.875),(0.75,8.87),(0.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((2.25,8.87),(2.25,8.875),(2.175,8.875),(2.175,9.525),(2.25,9.525),(2.25,9.53),(2.75,9.53),(2.75,9.525),(2.825,9.525),(2.825,8.875),(2.75,8.875),(2.75,8.87),(2.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,0.57),(1.25,1.375),(0.25,1.375),(0.25,2.625),(2.75,2.625),(2.75,1.375),(1.75,1.375),(1.75,0.57),(1.25,0.57)),
        )
        createRL(
            tech, net, 'psdm',
            ((15.25,5.375),(15.25,7.375),(14.75,7.375),(14.75,6.375),(12.25,6.375),(12.25,8.625),(14.25,8.625),(14.25,9.625),(17.75,9.625),(17.75,5.375),(15.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((2.25,0.37),(2.25,0.375),(2.175,0.375),(2.175,1.025),(2.25,1.025),(2.25,1.03),(2.75,1.03),(2.75,1.025),(2.825,1.025),(2.825,0.375),(2.75,0.375),(2.75,0.37),(2.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.25,6.375),(0.25,8.625),(1.25,8.625),(1.25,9.43),(1.75,9.43),(1.75,8.625),(2.75,8.625),(2.75,6.375),(0.25,6.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((14.25,0.37),(14.25,0.375),(14.175,0.375),(14.175,1.025),(14.25,1.025),(14.25,1.03),(14.75,1.03),(14.75,1.025),(14.825,1.025),(14.825,0.375),(14.75,0.375),(14.75,0.37),(14.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((6.25,0.37),(6.25,0.375),(6.175,0.375),(6.175,1.025),(6.25,1.025),(6.25,1.03),(6.75,1.03),(6.75,1.025),(6.825,1.025),(6.825,0.375),(6.75,0.375),(6.75,0.37),(6.25,0.37)),
        )
        createRL(
            tech, net, 'psdm',
            ((5.3,6.375),(5.3,7.375),(3.25,7.375),(3.25,9.625),(5.7,9.625),(5.7,8.625),(7.3,8.625),(7.3,9.625),(11.75,9.625),(11.75,7.375),(7.7,7.375),(7.7,6.375),(5.3,6.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.225,0.275),(0.225,1.025),(0.25,1.025),(0.25,1.03),(0.75,1.03),(0.75,1.025),(0.775,1.025),(0.775,0.275),(0.225,0.275)),
        )
        net = nets['ck']
        createRL(
            tech, net, 'poly',
            ((0.925,5.835),(0.925,8.63),(1.075,8.63),(1.075,6.165),(1.635,6.165),(1.635,5.835),(0.925,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((0.925,1.37),(0.925,3.165),(1.635,3.165),(1.635,2.835),(1.075,2.835),(1.075,1.37),(0.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.37),(1.415,8.63),(1.585,8.63),(1.585,1.37),(1.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,2.915),(1.415,3.085),(1.585,3.085),(1.585,2.915),(1.415,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,5.915),(1.415,6.085),(1.585,6.085),(1.585,5.915),(1.415,5.915)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((16.925,5.5),(16.925,9.5),(17.625,9.5),(17.625,5.5),(16.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((15.375,5.5),(15.375,7.5),(14.925,7.5),(14.925,9.5),(16.075,9.5),(16.075,5.5),(15.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((13.375,8.995),(13.375,9.0),(13.3,9.0),(13.3,9.4),(13.375,9.4),(13.375,9.405),(13.625,9.405),(13.625,9.4),(13.7,9.4),(13.7,9.0),(13.625,9.0),(13.625,8.995),(13.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,7.5),(3.925,9.5),(5.075,9.5),(5.075,7.5),(3.925,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((9.925,7.5),(9.925,9.5),(11.075,9.5),(11.075,7.5),(9.925,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.375,8.995),(6.375,9.0),(6.3,9.0),(6.3,9.4),(6.375,9.4),(6.375,9.405),(6.625,9.405),(6.625,9.4),(6.7,9.4),(6.7,9.0),(6.625,9.0),(6.625,8.995),(6.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.375,8.995),(2.375,9.0),(2.3,9.0),(2.3,9.4),(2.375,9.4),(2.375,9.405),(2.625,9.405),(2.625,9.4),(2.7,9.4),(2.7,9.0),(2.625,9.0),(2.625,8.995),(2.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((12.925,6.5),(12.925,8.5),(13.075,8.5),(13.075,6.5),(12.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,8.995),(0.375,9.0),(0.35,9.0),(0.35,9.5),(0.65,9.5),(0.65,9.0),(0.625,9.0),(0.625,8.995),(0.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((5.925,6.5),(5.925,8.5),(6.075,8.5),(6.075,6.5),(5.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,6.5),(0.925,8.5),(1.375,8.5),(1.375,9.305),(1.625,9.305),(1.625,8.5),(2.075,8.5),(2.075,6.5),(0.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((13.925,6.5),(13.925,8.5),(14.075,8.5),(14.075,6.5),(13.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.925,6.5),(6.925,8.5),(7.075,8.5),(7.075,6.5),(6.925,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((7.925,7.5),(7.925,9.5),(9.075,9.5),(9.075,7.5),(7.925,7.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(18.3,10.8),(18.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((15.415,5.635),(15.415,8.8),(10.585,8.8),(10.585,7.655),(10.415,7.655),(10.415,8.8),(8.585,8.8),(8.585,7.655),(8.415,7.655),(8.415,8.8),(0.0,8.8),(0.0,10.0),(18.0,10.0),(18.0,8.8),(17.585,8.8),(17.585,5.635),(17.415,5.635),(17.415,8.8),(15.585,8.8),(15.585,5.635),(15.415,5.635)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(9.0), u(18.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(9.0), u(18.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((8.415,7.735),(8.415,7.905),(8.585,7.905),(8.585,7.735),(8.415,7.735)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.075),(8.415,8.245),(8.585,8.245),(8.585,8.075),(8.415,8.075)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.415),(8.415,8.585),(8.585,8.585),(8.585,8.415),(8.415,8.415)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,8.755),(8.415,8.925),(8.585,8.925),(8.585,8.755),(8.415,8.755)),
        )
        createRL(
            tech, net, 'licon',
            ((8.415,9.095),(8.415,9.265),(8.585,9.265),(8.585,9.095),(8.415,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,9.115),(2.415,9.285),(2.585,9.285),(2.585,9.115),(2.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,9.115),(13.415,9.285),(13.585,9.285),(13.585,9.115),(13.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.115),(0.415,9.285),(0.585,9.285),(0.585,9.115),(0.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,7.735),(10.415,7.905),(10.585,7.905),(10.585,7.735),(10.415,7.735)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,8.075),(10.415,8.245),(10.585,8.245),(10.585,8.075),(10.415,8.075)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,8.415),(10.415,8.585),(10.585,8.585),(10.585,8.415),(10.415,8.415)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,8.755),(10.415,8.925),(10.585,8.925),(10.585,8.755),(10.415,8.755)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,9.095),(10.415,9.265),(10.585,9.265),(10.585,9.095),(10.415,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.015),(1.415,9.185),(1.585,9.185),(1.585,9.015),(1.415,9.015)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,5.715),(15.415,5.885),(15.585,5.885),(15.585,5.715),(15.415,5.715)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,6.055),(15.415,6.225),(15.585,6.225),(15.585,6.055),(15.415,6.055)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,6.395),(15.415,6.565),(15.585,6.565),(15.585,6.395),(15.415,6.395)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,6.735),(15.415,6.905),(15.585,6.905),(15.585,6.735),(15.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,7.075),(15.415,7.245),(15.585,7.245),(15.585,7.075),(15.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,7.415),(15.415,7.585),(15.585,7.585),(15.585,7.415),(15.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,7.755),(15.415,7.925),(15.585,7.925),(15.585,7.755),(15.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,8.095),(15.415,8.265),(15.585,8.265),(15.585,8.095),(15.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,8.435),(15.415,8.605),(15.585,8.605),(15.585,8.435),(15.415,8.435)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,8.775),(15.415,8.945),(15.585,8.945),(15.585,8.775),(15.415,8.775)),
        )
        createRL(
            tech, net, 'licon',
            ((15.415,9.115),(15.415,9.285),(15.585,9.285),(15.585,9.115),(15.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,9.115),(6.415,9.285),(6.585,9.285),(6.585,9.115),(6.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,5.715),(17.415,5.885),(17.585,5.885),(17.585,5.715),(17.415,5.715)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,6.055),(17.415,6.225),(17.585,6.225),(17.585,6.055),(17.415,6.055)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,6.395),(17.415,6.565),(17.585,6.565),(17.585,6.395),(17.415,6.395)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,6.735),(17.415,6.905),(17.585,6.905),(17.585,6.735),(17.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,7.075),(17.415,7.245),(17.585,7.245),(17.585,7.075),(17.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,7.415),(17.415,7.585),(17.585,7.585),(17.585,7.415),(17.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,7.755),(17.415,7.925),(17.585,7.925),(17.585,7.755),(17.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,8.095),(17.415,8.265),(17.585,8.265),(17.585,8.095),(17.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,8.435),(17.415,8.605),(17.585,8.605),(17.585,8.435),(17.415,8.435)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,8.775),(17.415,8.945),(17.585,8.945),(17.585,8.775),(17.415,8.775)),
        )
        createRL(
            tech, net, 'licon',
            ((17.415,9.115),(17.415,9.285),(17.585,9.285),(17.585,9.115),(17.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,9.045),(4.415,9.215),(4.585,9.215),(4.585,9.045),(4.415,9.045)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'difftap',
            ((5.425,6.5),(5.425,7.5),(5.075,7.5),(5.075,9.5),(5.575,9.5),(5.575,8.5),(5.925,8.5),(5.925,6.5),(5.425,6.5)),
        )
        net = nets['sff_m']
        createRL(
            tech, net, 'difftap',
            ((6.075,1.5),(6.075,2.5),(6.925,2.5),(6.925,1.5),(6.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.075,6.5),(6.075,8.5),(6.925,8.5),(6.925,6.5),(6.075,6.5)),
        )
        createRL(
            tech, net, 'poly',
            ((8.925,0.47),(8.925,2.835),(8.765,2.835),(8.765,3.165),(9.075,3.165),(9.075,0.47),(8.925,0.47)),
        )
        createRL(
            tech, net, 'poly',
            ((8.765,5.835),(8.765,6.165),(8.925,6.165),(8.925,9.63),(9.075,9.63),(9.075,5.835),(8.765,5.835)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,1.745),(6.415,1.915),(6.585,1.915),(6.585,1.745),(6.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,2.085),(6.415,2.255),(6.585,2.255),(6.585,2.085),(6.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((8.815,2.915),(8.815,3.085),(8.985,3.085),(8.985,2.915),(8.815,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((8.815,5.915),(8.815,6.085),(8.985,6.085),(8.985,5.915),(8.815,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,6.735),(6.415,6.905),(6.585,6.905),(6.585,6.735),(6.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.075),(6.415,7.245),(6.585,7.245),(6.585,7.075),(6.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.415),(6.415,7.585),(6.585,7.585),(6.585,7.415),(6.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.755),(6.415,7.925),(6.585,7.925),(6.585,7.755),(6.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.095),(6.415,8.265),(6.585,8.265),(6.585,8.095),(6.415,8.095)),
        )
        createRL(
            tech, net, 'li',
            ((6.415,1.665),(6.415,2.335),(6.585,2.335),(6.585,2.085),(7.415,2.085),(7.415,6.915),(6.585,6.915),(6.585,6.655),(6.415,6.655),(6.415,8.345),(6.585,8.345),(6.585,7.085),(7.585,7.085),(7.585,6.085),(8.815,6.085),(8.815,6.165),(8.985,6.165),(8.985,5.835),(8.815,5.835),(8.815,5.915),(7.585,5.915),(7.585,3.085),(8.815,3.085),(8.815,3.165),(8.985,3.165),(8.985,2.835),(8.815,2.835),(8.815,2.915),(7.585,2.915),(7.585,1.915),(6.585,1.915),(6.585,1.665),(6.415,1.665)),
        )
        net = nets['ckr']
        createRL(
            tech, net, 'poly',
            ((13.465,4.835),(13.465,5.165),(13.925,5.165),(13.925,8.63),(14.075,8.63),(14.075,4.835),(13.465,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((5.925,5.835),(5.925,8.63),(6.075,8.63),(6.075,6.165),(6.235,6.165),(6.235,5.835),(5.925,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((6.925,1.37),(6.925,3.835),(2.465,3.835),(2.465,4.165),(12.385,4.165),(12.385,3.835),(12.075,3.835),(12.075,1.37),(11.925,1.37),(11.925,3.835),(7.075,3.835),(7.075,1.37),(6.925,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,1.5),(2.075,2.5),(2.625,2.5),(2.625,1.5),(2.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,6.5),(2.075,8.5),(2.625,8.5),(2.625,6.5),(2.075,6.5)),
        )
        createRL(
            tech, net, 'licon',
            ((12.165,3.915),(12.165,4.085),(12.335,4.085),(12.335,3.915),(12.165,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,1.745),(2.415,1.915),(2.585,1.915),(2.585,1.745),(2.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,2.085),(2.415,2.255),(2.585,2.255),(2.585,2.085),(2.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((13.515,4.915),(13.515,5.085),(13.685,5.085),(13.685,4.915),(13.515,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.515,3.915),(2.515,4.085),(2.685,4.085),(2.685,3.915),(2.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.015,5.915),(6.015,6.085),(6.185,6.085),(6.185,5.915),(6.015,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,6.735),(2.415,6.905),(2.585,6.905),(2.585,6.735),(2.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.075),(2.415,7.245),(2.585,7.245),(2.585,7.075),(2.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.415),(2.415,7.585),(2.585,7.585),(2.585,7.415),(2.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.755),(2.415,7.925),(2.585,7.925),(2.585,7.755),(2.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,8.095),(2.415,8.265),(2.585,8.265),(2.585,8.095),(2.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((6.665,3.915),(6.665,4.085),(6.835,4.085),(6.835,3.915),(6.665,3.915)),
        )
        createRL(
            tech, net, 'li',
            ((6.665,3.835),(6.665,5.915),(6.185,5.915),(6.185,5.835),(6.015,5.835),(6.015,6.165),(6.185,6.165),(6.185,6.085),(6.835,6.085),(6.835,3.835),(6.665,3.835)),
        )
        createRL(
            tech, net, 'li',
            ((12.165,3.835),(12.165,4.165),(12.335,4.165),(12.335,4.085),(13.415,4.085),(13.415,5.005),(13.515,5.005),(13.515,5.165),(13.685,5.165),(13.685,4.835),(13.585,4.835),(13.585,3.915),(12.335,3.915),(12.335,3.835),(12.165,3.835)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,1.665),(2.415,8.345),(2.585,8.345),(2.585,4.165),(2.685,4.165),(2.685,3.835),(2.585,3.835),(2.585,1.665),(2.415,1.665)),
        )
        net = nets['sff_s']
        createRL(
            tech, net, 'difftap',
            ((12.075,1.5),(12.075,2.5),(12.925,2.5),(12.925,1.5),(12.075,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((13.075,6.5),(13.075,8.5),(13.925,8.5),(13.925,6.5),(13.075,6.5)),
        )
        createRL(
            tech, net, 'poly',
            ((15.925,0.47),(15.925,3.835),(15.465,3.835),(15.465,4.165),(15.925,4.165),(15.925,9.63),(16.075,9.63),(16.075,4.165),(16.925,4.165),(16.925,9.63),(17.075,9.63),(17.075,0.47),(16.925,0.47),(16.925,3.835),(16.075,3.835),(16.075,0.47),(15.925,0.47)),
        )
        createRL(
            tech, net, 'li',
            ((12.415,1.665),(12.415,2.335),(12.585,2.335),(12.585,2.085),(14.415,2.085),(14.415,5.915),(13.415,5.915),(13.415,7.745),(13.585,7.745),(13.585,6.085),(14.585,6.085),(14.585,4.085),(15.515,4.085),(15.515,4.165),(15.685,4.165),(15.685,3.835),(15.515,3.835),(15.515,3.915),(14.585,3.915),(14.585,1.915),(12.585,1.915),(12.585,1.665),(12.415,1.665)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,1.745),(12.415,1.915),(12.585,1.915),(12.585,1.745),(12.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,2.085),(12.415,2.255),(12.585,2.255),(12.585,2.085),(12.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((15.515,3.915),(15.515,4.085),(15.685,4.085),(15.685,3.915),(15.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,6.715),(13.415,6.885),(13.585,6.885),(13.585,6.715),(13.415,6.715)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,7.055),(13.415,7.225),(13.585,7.225),(13.585,7.055),(13.415,7.055)),
        )
        createRL(
            tech, net, 'licon',
            ((13.415,7.395),(13.415,7.565),(13.585,7.565),(13.585,7.395),(13.415,7.395)),
        )
        net = nets['_net3']
        createRL(
            tech, net, 'difftap',
            ((14.075,6.5),(14.075,8.5),(14.375,8.5),(14.375,9.5),(14.925,9.5),(14.925,7.5),(14.625,7.5),(14.625,6.5),(14.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((11.075,7.5),(11.075,9.5),(11.625,9.5),(11.625,7.5),(11.075,7.5)),
        )
        createRL(
            tech, net, 'licon',
            ((11.415,7.66),(11.415,7.83),(11.585,7.83),(11.585,7.66),(11.415,7.66)),
        )
        createRL(
            tech, net, 'licon',
            ((11.415,8.0),(11.415,8.17),(11.585,8.17),(11.585,8.0),(11.415,8.0)),
        )
        createRL(
            tech, net, 'licon',
            ((11.415,8.34),(11.415,8.51),(11.585,8.51),(11.585,8.34),(11.415,8.34)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,6.65),(14.415,6.82),(14.585,6.82),(14.585,6.65),(14.415,6.65)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,6.99),(14.415,7.16),(14.585,7.16),(14.585,6.99),(14.415,6.99)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,7.33),(14.415,7.5),(14.585,7.5),(14.585,7.33),(14.415,7.33)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,7.67),(14.415,7.84),(14.585,7.84),(14.585,7.67),(14.415,7.67)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,8.01),(14.415,8.18),(14.585,8.18),(14.585,8.01),(14.415,8.01)),
        )
        createRL(
            tech, net, 'licon',
            ((14.415,8.35),(14.415,8.52),(14.585,8.52),(14.585,8.35),(14.415,8.35)),
        )
        createRL(
            tech, net, 'li',
            ((14.415,6.57),(14.415,7.915),(11.585,7.915),(11.585,7.58),(11.415,7.58),(11.415,8.63),(11.585,8.63),(11.585,8.085),(14.415,8.085),(14.415,8.63),(14.585,8.63),(14.585,6.57),(14.415,6.57)),
        )
        net = nets['_net6']
        createRL(
            tech, net, 'difftap',
            ((7.425,0.6),(7.425,1.5),(7.075,1.5),(7.075,2.5),(7.575,2.5),(7.575,1.5),(7.925,1.5),(7.925,0.6),(7.425,0.6)),
        )
        net = nets['y']
        createRL(
            tech, net, 'difftap',
            ((10.075,0.6),(10.075,1.5),(10.375,1.5),(10.375,2.5),(11.925,2.5),(11.925,1.5),(10.625,1.5),(10.625,0.6),(10.075,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((9.075,7.5),(9.075,9.5),(9.925,9.5),(9.925,7.5),(9.075,7.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((12.375,6.5),(12.375,8.5),(12.925,8.5),(12.925,6.5),(12.375,6.5)),
        )
        createRL(
            tech, net, 'poly',
            ((7.925,0.47),(7.925,2.165),(8.235,2.165),(8.235,1.835),(8.075,1.835),(8.075,0.47),(7.925,0.47)),
        )
        createRL(
            tech, net, 'poly',
            ((7.925,6.835),(7.925,9.63),(8.075,9.63),(8.075,7.165),(8.235,7.165),(8.235,6.835),(7.925,6.835)),
        )
        createRL(
            tech, net, 'li',
            ((10.415,1.37),(10.415,1.915),(8.185,1.915),(8.185,1.835),(8.015,1.835),(8.015,2.165),(8.185,2.165),(8.185,2.1),(9.415,2.1),(9.415,6.915),(8.185,6.915),(8.185,6.835),(8.015,6.835),(8.015,7.165),(8.185,7.165),(8.185,7.085),(9.415,7.085),(9.415,8.63),(9.585,8.63),(9.585,7.085),(12.415,7.085),(12.415,7.745),(12.585,7.745),(12.585,6.635),(12.415,6.635),(12.415,6.915),(9.585,6.915),(9.585,2.085),(10.415,2.085),(10.415,2.42),(10.585,2.42),(10.585,1.37),(10.415,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,1.49),(10.415,1.66),(10.585,1.66),(10.585,1.49),(10.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,1.83),(10.415,2.0),(10.585,2.0),(10.585,1.83),(10.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((10.415,2.17),(10.415,2.34),(10.585,2.34),(10.585,2.17),(10.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,6.715),(12.415,6.885),(12.585,6.885),(12.585,6.715),(12.415,6.715)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,7.055),(12.415,7.225),(12.585,7.225),(12.585,7.055),(12.415,7.055)),
        )
        createRL(
            tech, net, 'licon',
            ((12.415,7.395),(12.415,7.565),(12.585,7.565),(12.585,7.395),(12.415,7.395)),
        )
        createRL(
            tech, net, 'licon',
            ((8.015,1.915),(8.015,2.085),(8.185,2.085),(8.185,1.915),(8.015,1.915)),
        )
        createRL(
            tech, net, 'licon',
            ((8.015,6.915),(8.015,7.085),(8.185,7.085),(8.185,6.915),(8.015,6.915)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,7.66),(9.415,7.83),(9.585,7.83),(9.585,7.66),(9.415,7.66)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,8.0),(9.415,8.17),(9.585,8.17),(9.585,8.0),(9.415,8.0)),
        )
        createRL(
            tech, net, 'licon',
            ((9.415,8.34),(9.415,8.51),(9.585,8.51),(9.585,8.34),(9.415,8.34)),
        )
        net = nets['u']
        createRL(
            tech, net, 'difftap',
            ((3.375,1.5),(3.375,2.5),(3.925,2.5),(3.925,1.5),(3.375,1.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.375,7.5),(3.375,9.5),(3.925,9.5),(3.925,7.5),(3.375,7.5)),
        )
        createRL(
            tech, net, 'poly',
            ((3.465,5.835),(3.465,6.165),(4.925,6.165),(4.925,9.63),(5.075,9.63),(5.075,6.165),(5.285,6.165),(5.285,5.835),(3.465,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((4.925,1.37),(4.925,3.165),(5.285,3.165),(5.285,2.835),(5.075,2.835),(5.075,1.37),(4.925,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.745),(3.415,1.915),(3.585,1.915),(3.585,1.745),(3.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.085),(3.415,2.255),(3.585,2.255),(3.585,2.085),(3.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((5.065,2.915),(5.065,3.085),(5.235,3.085),(5.235,2.915),(5.065,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((5.065,5.915),(5.065,6.085),(5.235,6.085),(5.235,5.915),(5.065,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.515,5.915),(3.515,6.085),(3.685,6.085),(3.685,5.915),(3.515,5.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.66),(3.415,7.83),(3.585,7.83),(3.585,7.66),(3.415,7.66)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.0),(3.415,8.17),(3.585,8.17),(3.585,8.0),(3.415,8.0)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,8.34),(3.415,8.51),(3.585,8.51),(3.585,8.34),(3.415,8.34)),
        )
        createRL(
            tech, net, 'li',
            ((5.065,2.835),(5.065,3.165),(5.165,3.165),(5.165,5.835),(5.065,5.835),(5.065,6.165),(5.235,6.165),(5.235,6.005),(5.335,6.005),(5.335,2.995),(5.235,2.995),(5.235,2.835),(5.065,2.835)),
        )
        createRL(
            tech, net, 'li',
            ((3.415,1.665),(3.415,8.63),(3.585,8.63),(3.585,6.165),(3.685,6.165),(3.685,5.835),(3.585,5.835),(3.585,1.665),(3.415,1.665)),
        )
        net = nets['i']
        createRL(
            tech, net, 'poly',
            ((3.925,6.835),(3.925,9.63),(4.075,9.63),(4.075,7.165),(4.535,7.165),(4.535,6.835),(3.925,6.835)),
        )
        createRL(
            tech, net, 'poly',
            ((3.925,1.37),(3.925,3.165),(4.535,3.165),(4.535,2.835),(4.075,2.835),(4.075,1.37),(3.925,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,1.37),(4.415,2.835),(4.315,2.835),(4.315,3.165),(4.415,3.165),(4.415,6.835),(4.315,6.835),(4.315,7.165),(4.415,7.165),(4.415,8.63),(4.585,8.63),(4.585,1.37),(4.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(4.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((4.315,6.915),(4.315,7.085),(4.485,7.085),(4.485,6.915),(4.315,6.915)),
        )
        createRL(
            tech, net, 'licon',
            ((4.315,2.915),(4.315,3.085),(4.485,3.085),(4.485,2.915),(4.315,2.915)),
        )
        net = nets['_net7']
        createRL(
            tech, net, 'difftap',
            ((5.075,1.5),(5.075,2.5),(5.925,2.5),(5.925,1.5),(5.075,1.5)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((7.075,6.5),(7.075,8.5),(7.425,8.5),(7.425,9.5),(7.925,9.5),(7.925,7.5),(7.575,7.5),(7.575,6.5),(7.075,6.5)),
        )
        net = nets['_net8']
        createRL(
            tech, net, 'difftap',
            ((13.075,1.5),(13.075,2.5),(13.925,2.5),(13.925,1.5),(13.075,1.5)),
        )
        net = nets['_net5']
        createRL(
            tech, net, 'difftap',
            ((14.075,1.5),(14.075,2.5),(14.925,2.5),(14.925,1.5),(14.075,1.5)),
        )
        net = nets['nrst']
        createRL(
            tech, net, 'poly',
            ((9.925,0.47),(9.925,3.165),(11.535,3.165),(11.535,2.835),(10.075,2.835),(10.075,0.47),(9.925,0.47)),
        )
        createRL(
            tech, net, 'poly',
            ((9.925,5.835),(9.925,9.63),(10.075,9.63),(10.075,6.165),(10.925,6.165),(10.925,9.63),(11.075,9.63),(11.075,6.165),(11.535,6.165),(11.535,5.835),(9.925,5.835)),
        )
        createRL(
            tech, net, 'poly',
            ((13.925,1.37),(13.925,2.835),(13.765,2.835),(13.765,3.165),(14.075,3.165),(14.075,1.37),(13.925,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((11.315,2.915),(11.315,3.085),(11.485,3.085),(11.485,2.915),(11.315,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((13.815,2.915),(13.815,3.085),(13.985,3.085),(13.985,2.915),(13.815,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((11.315,5.915),(11.315,6.085),(11.485,6.085),(11.485,5.915),(11.315,5.915)),
        )
        createRL(
            tech, net, 'li',
            ((11.415,1.37),(11.415,2.835),(11.315,2.835),(11.315,3.165),(11.415,3.165),(11.415,5.835),(11.315,5.835),(11.315,6.165),(11.415,6.165),(11.415,6.745),(11.585,6.745),(11.585,3.085),(13.815,3.085),(13.815,3.165),(13.985,3.165),(13.985,2.835),(13.815,2.835),(13.815,2.915),(11.585,2.915),(11.585,1.37),(11.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(11.5), u(0.17), u(1.37), u(6.745),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(11.5), u(0.17), u(1.37), u(6.745),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['_net4']
        createRL(
            tech, net, 'difftap',
            ((9.075,0.6),(9.075,1.5),(9.925,1.5),(9.925,0.6),(9.075,0.6)),
        )
        net = nets['q']
        createRL(
            tech, net, 'poly',
            ((14.925,4.835),(14.925,9.63),(15.075,9.63),(15.075,5.165),(15.235,5.165),(15.235,4.835),(14.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((14.925,1.37),(14.925,3.165),(15.235,3.165),(15.235,2.835),(15.075,2.835),(15.075,1.37),(14.925,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((16.075,5.5),(16.075,9.5),(16.925,9.5),(16.925,5.5),(16.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((16.075,0.6),(16.075,2.5),(16.925,2.5),(16.925,0.6),(16.075,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((16.415,1.37),(16.415,2.915),(15.185,2.915),(15.185,2.835),(15.015,2.835),(15.015,3.165),(15.185,3.165),(15.185,3.085),(16.415,3.085),(16.415,4.915),(15.185,4.915),(15.185,4.835),(15.015,4.835),(15.015,5.165),(15.185,5.165),(15.185,5.085),(16.415,5.085),(16.415,8.63),(16.585,8.63),(16.585,1.37),(16.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(16.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(16.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((15.015,2.915),(15.015,3.085),(15.185,3.085),(15.185,2.915),(15.015,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,5.64),(16.415,5.81),(16.585,5.81),(16.585,5.64),(16.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,5.98),(16.415,6.15),(16.585,6.15),(16.585,5.98),(16.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,6.32),(16.415,6.49),(16.585,6.49),(16.585,6.32),(16.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,6.66),(16.415,6.83),(16.585,6.83),(16.585,6.66),(16.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,7.0),(16.415,7.17),(16.585,7.17),(16.585,7.0),(16.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,7.34),(16.415,7.51),(16.585,7.51),(16.585,7.34),(16.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,7.68),(16.415,7.85),(16.585,7.85),(16.585,7.68),(16.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,8.02),(16.415,8.19),(16.585,8.19),(16.585,8.02),(16.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,8.36),(16.415,8.53),(16.585,8.53),(16.585,8.36),(16.415,8.36)),
        )
        createRL(
            tech, net, 'licon',
            ((15.015,4.915),(15.015,5.085),(15.185,5.085),(15.185,4.915),(15.015,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,1.49),(16.415,1.66),(16.585,1.66),(16.585,1.49),(16.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,1.83),(16.415,2.0),(16.585,2.0),(16.585,1.83),(16.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((16.415,2.17),(16.415,2.34),(16.585,2.34),(16.585,2.17),(16.415,2.17)),
        )
        nets['vss'].setGlobal(True)
        nets['vdd'].setGlobal(True)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['ck'].setType(Net.Type.CLOCK)
        nets['ck'].setDirection(Net.Direction.IN)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['nrst'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)

    cell = new_cells['xr2_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(7.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            '_net0': Net.create(cell, '_net0'),
            '_net1': Net.create(cell, '_net1'),
            '_net2': Net.create(cell, '_net2'),
            '_net3': Net.create(cell, '_net3'),
            '_net4': Net.create(cell, '_net4'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((3.075,5.5),(3.075,9.5),(3.925,9.5),(3.925,5.5),(3.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.075,0.6),(3.075,2.5),(3.925,2.5),(3.925,0.6),(3.075,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((2.415,1.37),(2.415,6.5),(2.585,6.5),(2.585,6.15),(3.415,6.15),(3.415,7.745),(3.585,7.745),(3.585,5.625),(3.415,5.625),(3.415,5.85),(2.585,5.85),(2.585,2.085),(3.415,2.085),(3.415,2.42),(3.585,2.42),(3.585,1.37),(3.415,1.37),(3.415,1.915),(2.585,1.915),(2.585,1.37),(2.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.5), u(0.17), u(1.37), u(6.5),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.5), u(0.17), u(1.37), u(6.5),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.415,1.49),(3.415,1.66),(3.585,1.66),(3.585,1.49),(3.415,1.49)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,1.83),(3.415,2.0),(3.585,2.0),(3.585,1.83),(3.415,1.83)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,2.17),(3.415,2.34),(3.585,2.34),(3.585,2.17),(3.415,2.17)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,5.705),(3.415,5.875),(3.585,5.875),(3.585,5.705),(3.415,5.705)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.045),(3.415,6.215),(3.585,6.215),(3.585,6.045),(3.415,6.045)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.385),(3.415,6.555),(3.585,6.555),(3.585,6.385),(3.415,6.385)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,6.725),(3.415,6.895),(3.585,6.895),(3.585,6.725),(3.415,6.725)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.065),(3.415,7.235),(3.585,7.235),(3.585,7.065),(3.415,7.065)),
        )
        createRL(
            tech, net, 'licon',
            ((3.415,7.405),(3.415,7.575),(3.585,7.575),(3.585,7.405),(3.415,7.405)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((6.375,8.995),(6.375,9.0),(6.35,9.0),(6.35,9.5),(6.65,9.5),(6.65,9.0),(6.625,9.0),(6.625,8.995),(6.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.925,5.5),(4.925,9.5),(5.625,9.5),(5.625,8.5),(6.075,8.5),(6.075,6.5),(5.625,6.5),(5.625,5.5),(4.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,5.5),(1.375,6.5),(0.925,6.5),(0.925,8.5),(1.375,8.5),(1.375,9.5),(2.075,9.5),(2.075,5.5),(1.375,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,8.995),(0.375,9.0),(0.35,9.0),(0.35,9.5),(0.65,9.5),(0.65,9.0),(0.625,9.0),(0.625,8.995),(0.375,8.995)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.925,5.5),(2.925,9.5),(3.075,9.5),(3.075,5.5),(2.925,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,5.5),(3.925,9.5),(4.075,9.5),(4.075,5.5),(3.925,5.5)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.3,4.8),(-0.3,10.8),(7.3,10.8),(7.3,4.8),(-0.3,4.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(7.0,10.0),(7.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((6.415,9.115),(6.415,9.285),(6.585,9.285),(6.585,9.115),(6.415,9.115)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,9.045),(1.415,9.215),(1.585,9.215),(1.585,9.045),(1.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,9.045),(5.415,9.215),(5.585,9.215),(5.585,9.045),(5.415,9.045)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.115),(0.415,9.285),(0.585,9.285),(0.585,9.115),(0.415,9.115)),
        )
        net = nets['_net2']
        createRL(
            tech, net, 'difftap',
            ((4.075,5.5),(4.075,9.5),(4.925,9.5),(4.925,5.5),(4.075,5.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.075,5.5),(2.075,9.5),(2.925,9.5),(2.925,5.5),(2.075,5.5)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,6.885),(2.415,7.055),(2.585,7.055),(2.585,6.885),(2.415,6.885)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.225),(2.415,7.395),(2.585,7.395),(2.585,7.225),(2.415,7.225)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.565),(2.415,7.735),(2.585,7.735),(2.585,7.565),(2.415,7.565)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,7.905),(2.415,8.075),(2.585,8.075),(2.585,7.905),(2.415,7.905)),
        )
        createRL(
            tech, net, 'licon',
            ((2.415,8.245),(2.415,8.415),(2.585,8.415),(2.585,8.245),(2.415,8.245)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.64),(4.415,5.81),(4.585,5.81),(4.585,5.64),(4.415,5.64)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,5.98),(4.415,6.15),(4.585,6.15),(4.585,5.98),(4.415,5.98)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.32),(4.415,6.49),(4.585,6.49),(4.585,6.32),(4.415,6.32)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,6.66),(4.415,6.83),(4.585,6.83),(4.585,6.66),(4.415,6.66)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.0),(4.415,7.17),(4.585,7.17),(4.585,7.0),(4.415,7.0)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.34),(4.415,7.51),(4.585,7.51),(4.585,7.34),(4.415,7.34)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,7.68),(4.415,7.85),(4.585,7.85),(4.585,7.68),(4.415,7.68)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.02),(4.415,8.19),(4.585,8.19),(4.585,8.02),(4.415,8.02)),
        )
        createRL(
            tech, net, 'licon',
            ((4.415,8.36),(4.415,8.53),(4.585,8.53),(4.585,8.36),(4.415,8.36)),
        )
        createRL(
            tech, net, 'li',
            ((4.415,5.56),(4.415,7.915),(2.585,7.915),(2.585,6.67),(2.415,6.67),(2.415,8.63),(2.585,8.63),(2.585,8.085),(4.415,8.085),(4.415,8.63),(4.585,8.63),(4.585,5.56),(4.415,5.56)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((1.25,5.375),(1.25,6.375),(0.25,6.375),(0.25,8.625),(1.25,8.625),(1.25,9.625),(5.75,9.625),(5.75,8.625),(6.75,8.625),(6.75,6.375),(5.75,6.375),(5.75,5.375),(1.25,5.375)),
        )
        createRL(
            tech, net, 'psdm',
            ((6.225,0.275),(6.225,1.025),(6.25,1.025),(6.25,1.03),(6.75,1.03),(6.75,1.025),(6.775,1.025),(6.775,0.275),(6.225,0.275)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.225,0.275),(0.225,1.025),(0.25,1.025),(0.25,1.03),(0.75,1.03),(0.75,1.025),(0.775,1.025),(0.775,0.275),(0.225,0.275)),
        )
        createRL(
            tech, net, 'nsdm',
            ((6.25,8.87),(6.25,8.875),(6.225,8.875),(6.225,9.625),(6.775,9.625),(6.775,8.875),(6.75,8.875),(6.75,8.87),(6.25,8.87)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.25,0.475),(1.25,1.375),(0.25,1.375),(0.25,2.625),(6.75,2.625),(6.75,1.375),(5.75,1.375),(5.75,0.475),(1.25,0.475)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,8.87),(0.25,8.875),(0.225,8.875),(0.225,9.625),(0.775,9.625),(0.775,8.875),(0.75,8.875),(0.75,8.87),(0.25,8.87)),
        )
        net = nets['_net0']
        createRL(
            tech, net, 'poly',
            ((3.925,0.47),(3.925,3.835),(0.465,3.835),(0.465,4.165),(3.925,4.165),(3.925,9.63),(4.075,9.63),(4.075,0.47),(3.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,6.5),(0.375,8.5),(0.925,8.5),(0.925,6.5),(0.375,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.5),(0.375,2.5),(0.925,2.5),(0.925,1.5),(0.375,1.5)),
        )
        createRL(
            tech, net, 'licon',
            ((0.515,3.915),(0.515,4.085),(0.685,4.085),(0.685,3.915),(0.515,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.745),(0.415,1.915),(0.585,1.915),(0.585,1.745),(0.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.085),(0.415,2.255),(0.585,2.255),(0.585,2.085),(0.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.735),(0.415,6.905),(0.585,6.905),(0.585,6.735),(0.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.075),(0.415,7.245),(0.585,7.245),(0.585,7.075),(0.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.415),(0.415,7.585),(0.585,7.585),(0.585,7.415),(0.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.755),(0.415,7.925),(0.585,7.925),(0.585,7.755),(0.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.095),(0.415,8.265),(0.585,8.265),(0.585,8.095),(0.415,8.095)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,1.665),(0.415,8.345),(0.585,8.345),(0.585,4.165),(0.685,4.165),(0.685,3.835),(0.585,3.835),(0.585,1.665),(0.415,1.665)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((2.925,0.6),(2.925,2.5),(3.075,2.5),(3.075,0.6),(2.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.35,0.4),(6.35,0.9),(6.375,0.9),(6.375,0.905),(6.625,0.905),(6.625,0.9),(6.65,0.9),(6.65,0.4),(6.35,0.4)),
        )
        createRL(
            tech, net, 'difftap',
            ((4.925,0.6),(4.925,2.5),(6.075,2.5),(6.075,1.5),(5.625,1.5),(5.625,0.6),(4.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.35,0.4),(0.35,0.9),(0.375,0.9),(0.375,0.905),(0.625,0.905),(0.625,0.9),(0.65,0.9),(0.65,0.4),(0.35,0.4)),
        )
        createRL(
            tech, net, 'difftap',
            ((3.925,0.6),(3.925,2.5),(4.075,2.5),(4.075,0.6),(3.925,0.6)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.375,0.6),(1.375,1.5),(0.925,1.5),(0.925,2.5),(2.075,2.5),(2.075,0.6),(1.375,0.6)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(7.0,1.2),(7.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(3.5), u(7.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,0.835),(1.415,1.005),(1.585,1.005),(1.585,0.835),(1.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,0.835),(5.415,1.005),(5.585,1.005),(5.585,0.835),(5.415,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.615),(0.415,0.785),(0.585,0.785),(0.585,0.615),(0.415,0.615)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,0.615),(6.415,0.785),(6.585,0.785),(6.585,0.615),(6.415,0.615)),
        )
        net = nets['_net4']
        createRL(
            tech, net, 'difftap',
            ((4.075,0.6),(4.075,2.5),(4.925,2.5),(4.925,0.6),(4.075,0.6)),
        )
        net = nets['_net3']
        createRL(
            tech, net, 'poly',
            ((2.925,4.835),(2.925,9.63),(3.075,9.63),(3.075,5.165),(3.535,5.165),(3.535,4.835),(2.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((4.925,0.47),(4.925,3.835),(4.765,3.835),(4.765,4.165),(6.535,4.165),(6.535,3.835),(5.075,3.835),(5.075,0.47),(4.925,0.47)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.075,6.5),(6.075,8.5),(6.625,8.5),(6.625,6.5),(6.075,6.5)),
        )
        createRL(
            tech, net, 'difftap',
            ((6.075,1.5),(6.075,2.5),(6.625,2.5),(6.625,1.5),(6.075,1.5)),
        )
        createRL(
            tech, net, 'licon',
            ((4.815,3.915),(4.815,4.085),(4.985,4.085),(4.985,3.915),(4.815,3.915)),
        )
        createRL(
            tech, net, 'licon',
            ((3.315,4.915),(3.315,5.085),(3.485,5.085),(3.485,4.915),(3.315,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,6.735),(6.415,6.905),(6.585,6.905),(6.585,6.735),(6.415,6.735)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.075),(6.415,7.245),(6.585,7.245),(6.585,7.075),(6.415,7.075)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.415),(6.415,7.585),(6.585,7.585),(6.585,7.415),(6.415,7.415)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,7.755),(6.415,7.925),(6.585,7.925),(6.585,7.755),(6.415,7.755)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,8.095),(6.415,8.265),(6.585,8.265),(6.585,8.095),(6.415,8.095)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,1.745),(6.415,1.915),(6.585,1.915),(6.585,1.745),(6.415,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((6.415,2.085),(6.415,2.255),(6.585,2.255),(6.585,2.085),(6.415,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((6.315,3.915),(6.315,4.085),(6.485,4.085),(6.485,3.915),(6.315,3.915)),
        )
        createRL(
            tech, net, 'li',
            ((6.415,1.665),(6.415,3.835),(6.315,3.835),(6.315,4.165),(6.415,4.165),(6.415,8.345),(6.585,8.345),(6.585,1.665),(6.415,1.665)),
        )
        createRL(
            tech, net, 'li',
            ((4.815,3.835),(4.815,3.915),(3.415,3.915),(3.415,4.835),(3.315,4.835),(3.315,5.165),(3.485,5.165),(3.485,5.005),(3.585,5.005),(3.585,4.085),(4.815,4.085),(4.815,4.165),(4.985,4.165),(4.985,3.835),(4.815,3.835)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.925,4.835),(0.925,8.63),(1.075,8.63),(1.075,5.165),(1.925,5.165),(1.925,9.63),(2.075,9.63),(2.075,4.835),(0.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((1.925,0.47),(1.925,2.835),(1.075,2.835),(1.075,1.37),(0.925,1.37),(0.925,3.165),(2.075,3.165),(2.075,0.47),(1.925,0.47)),
        )
        createRL(
            tech, net, 'li',
            ((1.415,1.37),(1.415,8.63),(1.585,8.63),(1.585,1.37),(1.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((1.415,2.915),(1.415,3.085),(1.585,3.085),(1.585,2.915),(1.415,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((1.415,4.915),(1.415,5.085),(1.585,5.085),(1.585,4.915),(1.415,4.915)),
        )
        net = nets['_net1']
        createRL(
            tech, net, 'difftap',
            ((2.075,0.6),(2.075,2.5),(2.925,2.5),(2.925,0.6),(2.075,0.6)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((5.925,1.37),(5.925,2.835),(5.365,2.835),(5.365,3.165),(6.075,3.165),(6.075,1.37),(5.925,1.37)),
        )
        createRL(
            tech, net, 'poly',
            ((4.925,4.835),(4.925,9.63),(5.075,9.63),(5.075,5.165),(5.925,5.165),(5.925,8.63),(6.075,8.63),(6.075,4.835),(4.925,4.835)),
        )
        createRL(
            tech, net, 'poly',
            ((2.925,0.47),(2.925,3.165),(3.235,3.165),(3.235,2.835),(3.075,2.835),(3.075,0.47),(2.925,0.47)),
        )
        createRL(
            tech, net, 'li',
            ((5.415,1.37),(5.415,2.915),(3.185,2.915),(3.185,2.835),(3.015,2.835),(3.015,3.165),(3.185,3.165),(3.185,3.085),(5.415,3.085),(5.415,8.63),(5.585,8.63),(5.585,1.37),(5.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(5.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'licon',
            ((3.015,2.915),(3.015,3.085),(3.185,3.085),(3.185,2.915),(3.015,2.915)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,4.915),(5.415,5.085),(5.585,5.085),(5.585,4.915),(5.415,4.915)),
        )
        createRL(
            tech, net, 'licon',
            ((5.415,2.915),(5.415,3.085),(5.585,3.085),(5.585,2.915),(5.415,2.915)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['q'].setDirection(Net.Direction.OUT)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)

    cell = new_cells['fill']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(1.0), u(10.0),
        ))
        nets = {
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(1.0,10.0),(1.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(1.18,10.18),(1.18,4.8),(-0.18,4.8)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(1.0,1.2),(1.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(1.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(1.0,10.0),(1.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(1.18,10.18),(1.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.295,9.095),(0.295,9.685),(0.705,9.685),(0.705,9.095),(0.295,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.135),(0.415,9.305),(0.585,9.305),(0.585,9.135),(0.415,9.135)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.475),(0.415,9.645),(0.585,9.645),(0.585,9.475),(0.415,9.475)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(1.0,1.2),(1.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.295,0.315),(0.295,0.905),(0.705,0.905),(0.705,0.315),(0.295,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.355),(0.415,0.525),(0.585,0.525),(0.585,0.355),(0.415,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.695),(0.415,0.865),(0.585,0.865),(0.585,0.695),(0.415,0.695)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.17,0.19),(0.17,1.03),(0.83,1.03),(0.83,0.19),(0.17,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.17,8.97),(0.17,9.81),(0.83,9.81),(0.83,8.97),(0.17,8.97)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_diff']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(1.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.415,5.13),(0.415,8.8),(0.0,8.8),(0.0,10.0),(1.0,10.0),(1.0,8.8),(0.585,8.8),(0.585,5.13),(0.415,5.13)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(1.18,10.18),(1.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,4.985),(0.315,9.685),(0.685,9.685),(0.685,4.985),(0.315,4.985)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,5.21),(0.415,5.38),(0.585,5.38),(0.585,5.21),(0.415,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,5.55),(0.415,5.72),(0.585,5.72),(0.585,5.55),(0.415,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,5.89),(0.415,6.06),(0.585,6.06),(0.585,5.89),(0.415,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.23),(0.415,6.4),(0.585,6.4),(0.585,6.23),(0.415,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.57),(0.415,6.74),(0.585,6.74),(0.585,6.57),(0.415,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.91),(0.415,7.08),(0.585,7.08),(0.585,6.91),(0.415,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.25),(0.415,7.42),(0.585,7.42),(0.585,7.25),(0.415,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.59),(0.415,7.76),(0.585,7.76),(0.585,7.59),(0.415,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.93),(0.415,8.1),(0.585,8.1),(0.585,7.93),(0.415,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.27),(0.415,8.44),(0.585,8.44),(0.585,8.27),(0.415,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.61),(0.415,8.78),(0.585,8.78),(0.585,8.61),(0.415,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.95),(0.415,9.12),(0.585,9.12),(0.585,8.95),(0.415,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.29),(0.415,9.46),(0.585,9.46),(0.585,9.29),(0.415,9.29)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.415,1.2),(0.415,4.25),(0.585,4.25),(0.585,1.2),(1.0,1.2),(1.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.315,0.315),(0.315,4.455),(0.685,4.455),(0.685,0.315),(0.315,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.6),(0.415,0.77),(0.585,0.77),(0.585,0.6),(0.415,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.94),(0.415,1.11),(0.585,1.11),(0.585,0.94),(0.415,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.28),(0.415,1.45),(0.585,1.45),(0.585,1.28),(0.415,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.62),(0.415,1.79),(0.585,1.79),(0.585,1.62),(0.415,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.96),(0.415,2.13),(0.585,2.13),(0.585,1.96),(0.415,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.3),(0.415,2.47),(0.585,2.47),(0.585,2.3),(0.415,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.64),(0.415,2.81),(0.585,2.81),(0.585,2.64),(0.415,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.98),(0.415,3.15),(0.585,3.15),(0.585,2.98),(0.415,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,3.32),(0.415,3.49),(0.585,3.49),(0.585,3.32),(0.415,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,3.66),(0.415,3.83),(0.585,3.83),(0.585,3.66),(0.415,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,4.0),(0.415,4.17),(0.585,4.17),(0.585,4.0),(0.415,4.0)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.19,0.19),(0.19,4.58),(0.81,4.58),(0.81,0.19),(0.19,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,4.86),(0.19,9.81),(0.81,9.81),(0.81,4.86),(0.19,4.86)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_poly']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(1.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(1.0,10.0),(1.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(1.18,10.18),(1.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.295,9.095),(0.295,9.685),(0.705,9.685),(0.705,9.095),(0.295,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.135),(0.415,9.305),(0.585,9.305),(0.585,9.135),(0.415,9.135)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.475),(0.415,9.645),(0.585,9.645),(0.585,9.475),(0.415,9.475)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.415,1.2),(0.415,1.44),(0.585,1.44),(0.585,1.2),(1.0,1.2),(1.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.295,0.315),(0.295,0.905),(0.705,0.905),(0.705,0.315),(0.295,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.355),(0.415,0.525),(0.585,0.525),(0.585,0.355),(0.415,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.695),(0.415,0.865),(0.585,0.865),(0.585,0.695),(0.415,0.695)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.19),(0.415,1.36),(0.585,1.36),(0.585,1.19),(0.415,1.19)),
        )
        createRL(
            tech, net, 'poly',
            ((0.105,0.98),(0.105,9.02),(0.895,9.02),(0.895,0.98),(0.105,0.98)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.17,0.19),(0.17,1.03),(0.83,1.03),(0.83,0.19),(0.17,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.17,8.97),(0.17,9.81),(0.83,9.81),(0.83,8.97),(0.17,8.97)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['fill_w2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_w2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.795,9.095),(0.795,9.685),(1.205,9.685),(1.205,9.095),(0.795,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,9.135),(0.915,9.305),(1.085,9.305),(1.085,9.135),(0.915,9.135)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,9.475),(0.915,9.645),(1.085,9.645),(1.085,9.475),(0.915,9.475)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.795,0.315),(0.795,0.905),(1.205,0.905),(1.205,0.315),(0.795,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,0.355),(0.915,0.525),(1.085,0.525),(1.085,0.355),(0.915,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,0.695),(0.915,0.865),(1.085,0.865),(1.085,0.695),(0.915,0.695)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.67,0.19),(0.67,1.03),(1.33,1.03),(1.33,0.19),(0.67,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.67,8.97),(0.67,9.81),(1.33,9.81),(1.33,8.97),(0.67,8.97)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_diff_w2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.405,5.13),(0.405,8.8),(0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(1.595,8.8),(1.595,5.13),(0.405,5.13)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,4.985),(0.315,9.685),(1.685,9.685),(1.685,4.985),(0.315,4.985)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,5.21),(0.405,5.38),(0.575,5.38),(0.575,5.21),(0.405,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,5.21),(0.745,5.38),(0.915,5.38),(0.915,5.21),(0.745,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,5.21),(1.085,5.38),(1.255,5.38),(1.255,5.21),(1.085,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,5.21),(1.425,5.38),(1.595,5.38),(1.595,5.21),(1.425,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,5.55),(0.405,5.72),(0.575,5.72),(0.575,5.55),(0.405,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,5.55),(0.745,5.72),(0.915,5.72),(0.915,5.55),(0.745,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,5.55),(1.085,5.72),(1.255,5.72),(1.255,5.55),(1.085,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,5.55),(1.425,5.72),(1.595,5.72),(1.595,5.55),(1.425,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,5.89),(0.405,6.06),(0.575,6.06),(0.575,5.89),(0.405,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,5.89),(0.745,6.06),(0.915,6.06),(0.915,5.89),(0.745,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,5.89),(1.085,6.06),(1.255,6.06),(1.255,5.89),(1.085,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,5.89),(1.425,6.06),(1.595,6.06),(1.595,5.89),(1.425,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,6.23),(0.405,6.4),(0.575,6.4),(0.575,6.23),(0.405,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,6.23),(0.745,6.4),(0.915,6.4),(0.915,6.23),(0.745,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,6.23),(1.085,6.4),(1.255,6.4),(1.255,6.23),(1.085,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,6.23),(1.425,6.4),(1.595,6.4),(1.595,6.23),(1.425,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,6.57),(0.405,6.74),(0.575,6.74),(0.575,6.57),(0.405,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,6.57),(0.745,6.74),(0.915,6.74),(0.915,6.57),(0.745,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,6.57),(1.085,6.74),(1.255,6.74),(1.255,6.57),(1.085,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,6.57),(1.425,6.74),(1.595,6.74),(1.595,6.57),(1.425,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,6.91),(0.405,7.08),(0.575,7.08),(0.575,6.91),(0.405,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,6.91),(0.745,7.08),(0.915,7.08),(0.915,6.91),(0.745,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,6.91),(1.085,7.08),(1.255,7.08),(1.255,6.91),(1.085,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,6.91),(1.425,7.08),(1.595,7.08),(1.595,6.91),(1.425,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,7.25),(0.405,7.42),(0.575,7.42),(0.575,7.25),(0.405,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,7.25),(0.745,7.42),(0.915,7.42),(0.915,7.25),(0.745,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,7.25),(1.085,7.42),(1.255,7.42),(1.255,7.25),(1.085,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,7.25),(1.425,7.42),(1.595,7.42),(1.595,7.25),(1.425,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,7.59),(0.405,7.76),(0.575,7.76),(0.575,7.59),(0.405,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,7.59),(0.745,7.76),(0.915,7.76),(0.915,7.59),(0.745,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,7.59),(1.085,7.76),(1.255,7.76),(1.255,7.59),(1.085,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,7.59),(1.425,7.76),(1.595,7.76),(1.595,7.59),(1.425,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,7.93),(0.405,8.1),(0.575,8.1),(0.575,7.93),(0.405,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,7.93),(0.745,8.1),(0.915,8.1),(0.915,7.93),(0.745,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,7.93),(1.085,8.1),(1.255,8.1),(1.255,7.93),(1.085,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,7.93),(1.425,8.1),(1.595,8.1),(1.595,7.93),(1.425,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,8.27),(0.405,8.44),(0.575,8.44),(0.575,8.27),(0.405,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,8.27),(0.745,8.44),(0.915,8.44),(0.915,8.27),(0.745,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,8.27),(1.085,8.44),(1.255,8.44),(1.255,8.27),(1.085,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,8.27),(1.425,8.44),(1.595,8.44),(1.595,8.27),(1.425,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,8.61),(0.405,8.78),(0.575,8.78),(0.575,8.61),(0.405,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,8.61),(0.745,8.78),(0.915,8.78),(0.915,8.61),(0.745,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,8.61),(1.085,8.78),(1.255,8.78),(1.255,8.61),(1.085,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,8.61),(1.425,8.78),(1.595,8.78),(1.595,8.61),(1.425,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,8.95),(0.405,9.12),(0.575,9.12),(0.575,8.95),(0.405,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,8.95),(0.745,9.12),(0.915,9.12),(0.915,8.95),(0.745,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,8.95),(1.085,9.12),(1.255,9.12),(1.255,8.95),(1.085,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,8.95),(1.425,9.12),(1.595,9.12),(1.595,8.95),(1.425,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,9.29),(0.405,9.46),(0.575,9.46),(0.575,9.29),(0.405,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,9.29),(0.745,9.46),(0.915,9.46),(0.915,9.29),(0.745,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,9.29),(1.085,9.46),(1.255,9.46),(1.255,9.29),(1.085,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,9.29),(1.425,9.46),(1.595,9.46),(1.595,9.29),(1.425,9.29)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.405,1.2),(0.405,4.25),(1.595,4.25),(1.595,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.315,0.315),(0.315,4.455),(1.685,4.455),(1.685,0.315),(0.315,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,0.6),(0.405,0.77),(0.575,0.77),(0.575,0.6),(0.405,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,0.6),(0.745,0.77),(0.915,0.77),(0.915,0.6),(0.745,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,0.6),(1.085,0.77),(1.255,0.77),(1.255,0.6),(1.085,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,0.6),(1.425,0.77),(1.595,0.77),(1.595,0.6),(1.425,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,0.94),(0.405,1.11),(0.575,1.11),(0.575,0.94),(0.405,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,0.94),(0.745,1.11),(0.915,1.11),(0.915,0.94),(0.745,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,0.94),(1.085,1.11),(1.255,1.11),(1.255,0.94),(1.085,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,0.94),(1.425,1.11),(1.595,1.11),(1.595,0.94),(1.425,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,1.28),(0.405,1.45),(0.575,1.45),(0.575,1.28),(0.405,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,1.28),(0.745,1.45),(0.915,1.45),(0.915,1.28),(0.745,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,1.28),(1.085,1.45),(1.255,1.45),(1.255,1.28),(1.085,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,1.28),(1.425,1.45),(1.595,1.45),(1.595,1.28),(1.425,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,1.62),(0.405,1.79),(0.575,1.79),(0.575,1.62),(0.405,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,1.62),(0.745,1.79),(0.915,1.79),(0.915,1.62),(0.745,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,1.62),(1.085,1.79),(1.255,1.79),(1.255,1.62),(1.085,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,1.62),(1.425,1.79),(1.595,1.79),(1.595,1.62),(1.425,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,1.96),(0.405,2.13),(0.575,2.13),(0.575,1.96),(0.405,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,1.96),(0.745,2.13),(0.915,2.13),(0.915,1.96),(0.745,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,1.96),(1.085,2.13),(1.255,2.13),(1.255,1.96),(1.085,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,1.96),(1.425,2.13),(1.595,2.13),(1.595,1.96),(1.425,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,2.3),(0.405,2.47),(0.575,2.47),(0.575,2.3),(0.405,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,2.3),(0.745,2.47),(0.915,2.47),(0.915,2.3),(0.745,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,2.3),(1.085,2.47),(1.255,2.47),(1.255,2.3),(1.085,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,2.3),(1.425,2.47),(1.595,2.47),(1.595,2.3),(1.425,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,2.64),(0.405,2.81),(0.575,2.81),(0.575,2.64),(0.405,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,2.64),(0.745,2.81),(0.915,2.81),(0.915,2.64),(0.745,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,2.64),(1.085,2.81),(1.255,2.81),(1.255,2.64),(1.085,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,2.64),(1.425,2.81),(1.595,2.81),(1.595,2.64),(1.425,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,2.98),(0.405,3.15),(0.575,3.15),(0.575,2.98),(0.405,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,2.98),(0.745,3.15),(0.915,3.15),(0.915,2.98),(0.745,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,2.98),(1.085,3.15),(1.255,3.15),(1.255,2.98),(1.085,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,2.98),(1.425,3.15),(1.595,3.15),(1.595,2.98),(1.425,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,3.32),(0.405,3.49),(0.575,3.49),(0.575,3.32),(0.405,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,3.32),(0.745,3.49),(0.915,3.49),(0.915,3.32),(0.745,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,3.32),(1.085,3.49),(1.255,3.49),(1.255,3.32),(1.085,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,3.32),(1.425,3.49),(1.595,3.49),(1.595,3.32),(1.425,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,3.66),(0.405,3.83),(0.575,3.83),(0.575,3.66),(0.405,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,3.66),(0.745,3.83),(0.915,3.83),(0.915,3.66),(0.745,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,3.66),(1.085,3.83),(1.255,3.83),(1.255,3.66),(1.085,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,3.66),(1.425,3.83),(1.595,3.83),(1.595,3.66),(1.425,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((0.405,4.0),(0.405,4.17),(0.575,4.17),(0.575,4.0),(0.405,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,4.0),(0.745,4.17),(0.915,4.17),(0.915,4.0),(0.745,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,4.0),(1.085,4.17),(1.255,4.17),(1.255,4.0),(1.085,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((1.425,4.0),(1.425,4.17),(1.595,4.17),(1.595,4.0),(1.425,4.0)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.19,0.19),(0.19,4.58),(1.81,4.58),(1.81,0.19),(0.19,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,4.86),(0.19,9.81),(1.81,9.81),(1.81,4.86),(0.19,4.86)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_poly_w2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.795,9.095),(0.795,9.685),(1.205,9.685),(1.205,9.095),(0.795,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,9.135),(0.915,9.305),(1.085,9.305),(1.085,9.135),(0.915,9.135)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,9.475),(0.915,9.645),(1.085,9.645),(1.085,9.475),(0.915,9.475)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.915,1.2),(0.915,1.44),(1.085,1.44),(1.085,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.795,0.315),(0.795,0.905),(1.205,0.905),(1.205,0.315),(0.795,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,0.355),(0.915,0.525),(1.085,0.525),(1.085,0.355),(0.915,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,0.695),(0.915,0.865),(1.085,0.865),(1.085,0.695),(0.915,0.695)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,1.19),(0.915,1.36),(1.085,1.36),(1.085,1.19),(0.915,1.19)),
        )
        createRL(
            tech, net, 'poly',
            ((0.105,0.98),(0.105,9.02),(1.895,9.02),(1.895,0.98),(0.105,0.98)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.67,0.19),(0.67,1.03),(1.33,1.03),(1.33,0.19),(0.67,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.67,8.97),(0.67,9.81),(1.33,9.81),(1.33,8.97),(0.67,8.97)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['fill_w4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(4.18,10.18),(4.18,4.8),(-0.18,4.8)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_w4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(4.18,10.18),(4.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.795,9.095),(1.795,9.685),(2.205,9.685),(2.205,9.095),(1.795,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,9.135),(1.915,9.305),(2.085,9.305),(2.085,9.135),(1.915,9.135)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,9.475),(1.915,9.645),(2.085,9.645),(2.085,9.475),(1.915,9.475)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((1.795,0.315),(1.795,0.905),(2.205,0.905),(2.205,0.315),(1.795,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,0.355),(1.915,0.525),(2.085,0.525),(2.085,0.355),(1.915,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,0.695),(1.915,0.865),(2.085,0.865),(2.085,0.695),(1.915,0.695)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((1.67,0.19),(1.67,1.03),(2.33,1.03),(2.33,0.19),(1.67,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.67,8.97),(1.67,9.81),(2.33,9.81),(2.33,8.97),(1.67,8.97)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_diff_w4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.385,5.13),(0.385,8.8),(0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(3.615,8.8),(3.615,5.13),(0.385,5.13)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(4.18,10.18),(4.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,4.985),(0.315,9.685),(3.685,9.685),(3.685,4.985),(0.315,4.985)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,5.21),(0.385,5.38),(0.555,5.38),(0.555,5.21),(0.385,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,5.21),(0.725,5.38),(0.895,5.38),(0.895,5.21),(0.725,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,5.21),(1.065,5.38),(1.235,5.38),(1.235,5.21),(1.065,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,5.21),(1.405,5.38),(1.575,5.38),(1.575,5.21),(1.405,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,5.21),(1.745,5.38),(1.915,5.38),(1.915,5.21),(1.745,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,5.21),(2.085,5.38),(2.255,5.38),(2.255,5.21),(2.085,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,5.21),(2.425,5.38),(2.595,5.38),(2.595,5.21),(2.425,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,5.21),(2.765,5.38),(2.935,5.38),(2.935,5.21),(2.765,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,5.21),(3.105,5.38),(3.275,5.38),(3.275,5.21),(3.105,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,5.21),(3.445,5.38),(3.615,5.38),(3.615,5.21),(3.445,5.21)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,5.55),(0.385,5.72),(0.555,5.72),(0.555,5.55),(0.385,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,5.55),(0.725,5.72),(0.895,5.72),(0.895,5.55),(0.725,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,5.55),(1.065,5.72),(1.235,5.72),(1.235,5.55),(1.065,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,5.55),(1.405,5.72),(1.575,5.72),(1.575,5.55),(1.405,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,5.55),(1.745,5.72),(1.915,5.72),(1.915,5.55),(1.745,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,5.55),(2.085,5.72),(2.255,5.72),(2.255,5.55),(2.085,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,5.55),(2.425,5.72),(2.595,5.72),(2.595,5.55),(2.425,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,5.55),(2.765,5.72),(2.935,5.72),(2.935,5.55),(2.765,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,5.55),(3.105,5.72),(3.275,5.72),(3.275,5.55),(3.105,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,5.55),(3.445,5.72),(3.615,5.72),(3.615,5.55),(3.445,5.55)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,5.89),(0.385,6.06),(0.555,6.06),(0.555,5.89),(0.385,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,5.89),(0.725,6.06),(0.895,6.06),(0.895,5.89),(0.725,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,5.89),(1.065,6.06),(1.235,6.06),(1.235,5.89),(1.065,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,5.89),(1.405,6.06),(1.575,6.06),(1.575,5.89),(1.405,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,5.89),(1.745,6.06),(1.915,6.06),(1.915,5.89),(1.745,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,5.89),(2.085,6.06),(2.255,6.06),(2.255,5.89),(2.085,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,5.89),(2.425,6.06),(2.595,6.06),(2.595,5.89),(2.425,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,5.89),(2.765,6.06),(2.935,6.06),(2.935,5.89),(2.765,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,5.89),(3.105,6.06),(3.275,6.06),(3.275,5.89),(3.105,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,5.89),(3.445,6.06),(3.615,6.06),(3.615,5.89),(3.445,5.89)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,6.23),(0.385,6.4),(0.555,6.4),(0.555,6.23),(0.385,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,6.23),(0.725,6.4),(0.895,6.4),(0.895,6.23),(0.725,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,6.23),(1.065,6.4),(1.235,6.4),(1.235,6.23),(1.065,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,6.23),(1.405,6.4),(1.575,6.4),(1.575,6.23),(1.405,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,6.23),(1.745,6.4),(1.915,6.4),(1.915,6.23),(1.745,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,6.23),(2.085,6.4),(2.255,6.4),(2.255,6.23),(2.085,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,6.23),(2.425,6.4),(2.595,6.4),(2.595,6.23),(2.425,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,6.23),(2.765,6.4),(2.935,6.4),(2.935,6.23),(2.765,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,6.23),(3.105,6.4),(3.275,6.4),(3.275,6.23),(3.105,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,6.23),(3.445,6.4),(3.615,6.4),(3.615,6.23),(3.445,6.23)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,6.57),(0.385,6.74),(0.555,6.74),(0.555,6.57),(0.385,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,6.57),(0.725,6.74),(0.895,6.74),(0.895,6.57),(0.725,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,6.57),(1.065,6.74),(1.235,6.74),(1.235,6.57),(1.065,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,6.57),(1.405,6.74),(1.575,6.74),(1.575,6.57),(1.405,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,6.57),(1.745,6.74),(1.915,6.74),(1.915,6.57),(1.745,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,6.57),(2.085,6.74),(2.255,6.74),(2.255,6.57),(2.085,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,6.57),(2.425,6.74),(2.595,6.74),(2.595,6.57),(2.425,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,6.57),(2.765,6.74),(2.935,6.74),(2.935,6.57),(2.765,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,6.57),(3.105,6.74),(3.275,6.74),(3.275,6.57),(3.105,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,6.57),(3.445,6.74),(3.615,6.74),(3.615,6.57),(3.445,6.57)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,6.91),(0.385,7.08),(0.555,7.08),(0.555,6.91),(0.385,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,6.91),(0.725,7.08),(0.895,7.08),(0.895,6.91),(0.725,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,6.91),(1.065,7.08),(1.235,7.08),(1.235,6.91),(1.065,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,6.91),(1.405,7.08),(1.575,7.08),(1.575,6.91),(1.405,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,6.91),(1.745,7.08),(1.915,7.08),(1.915,6.91),(1.745,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,6.91),(2.085,7.08),(2.255,7.08),(2.255,6.91),(2.085,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,6.91),(2.425,7.08),(2.595,7.08),(2.595,6.91),(2.425,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,6.91),(2.765,7.08),(2.935,7.08),(2.935,6.91),(2.765,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,6.91),(3.105,7.08),(3.275,7.08),(3.275,6.91),(3.105,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,6.91),(3.445,7.08),(3.615,7.08),(3.615,6.91),(3.445,6.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,7.25),(0.385,7.42),(0.555,7.42),(0.555,7.25),(0.385,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,7.25),(0.725,7.42),(0.895,7.42),(0.895,7.25),(0.725,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,7.25),(1.065,7.42),(1.235,7.42),(1.235,7.25),(1.065,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,7.25),(1.405,7.42),(1.575,7.42),(1.575,7.25),(1.405,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,7.25),(1.745,7.42),(1.915,7.42),(1.915,7.25),(1.745,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,7.25),(2.085,7.42),(2.255,7.42),(2.255,7.25),(2.085,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,7.25),(2.425,7.42),(2.595,7.42),(2.595,7.25),(2.425,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,7.25),(2.765,7.42),(2.935,7.42),(2.935,7.25),(2.765,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,7.25),(3.105,7.42),(3.275,7.42),(3.275,7.25),(3.105,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,7.25),(3.445,7.42),(3.615,7.42),(3.615,7.25),(3.445,7.25)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,7.59),(0.385,7.76),(0.555,7.76),(0.555,7.59),(0.385,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,7.59),(0.725,7.76),(0.895,7.76),(0.895,7.59),(0.725,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,7.59),(1.065,7.76),(1.235,7.76),(1.235,7.59),(1.065,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,7.59),(1.405,7.76),(1.575,7.76),(1.575,7.59),(1.405,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,7.59),(1.745,7.76),(1.915,7.76),(1.915,7.59),(1.745,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,7.59),(2.085,7.76),(2.255,7.76),(2.255,7.59),(2.085,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,7.59),(2.425,7.76),(2.595,7.76),(2.595,7.59),(2.425,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,7.59),(2.765,7.76),(2.935,7.76),(2.935,7.59),(2.765,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,7.59),(3.105,7.76),(3.275,7.76),(3.275,7.59),(3.105,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,7.59),(3.445,7.76),(3.615,7.76),(3.615,7.59),(3.445,7.59)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,7.93),(0.385,8.1),(0.555,8.1),(0.555,7.93),(0.385,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,7.93),(0.725,8.1),(0.895,8.1),(0.895,7.93),(0.725,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,7.93),(1.065,8.1),(1.235,8.1),(1.235,7.93),(1.065,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,7.93),(1.405,8.1),(1.575,8.1),(1.575,7.93),(1.405,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,7.93),(1.745,8.1),(1.915,8.1),(1.915,7.93),(1.745,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,7.93),(2.085,8.1),(2.255,8.1),(2.255,7.93),(2.085,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,7.93),(2.425,8.1),(2.595,8.1),(2.595,7.93),(2.425,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,7.93),(2.765,8.1),(2.935,8.1),(2.935,7.93),(2.765,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,7.93),(3.105,8.1),(3.275,8.1),(3.275,7.93),(3.105,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,7.93),(3.445,8.1),(3.615,8.1),(3.615,7.93),(3.445,7.93)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,8.27),(0.385,8.44),(0.555,8.44),(0.555,8.27),(0.385,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,8.27),(0.725,8.44),(0.895,8.44),(0.895,8.27),(0.725,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,8.27),(1.065,8.44),(1.235,8.44),(1.235,8.27),(1.065,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,8.27),(1.405,8.44),(1.575,8.44),(1.575,8.27),(1.405,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,8.27),(1.745,8.44),(1.915,8.44),(1.915,8.27),(1.745,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,8.27),(2.085,8.44),(2.255,8.44),(2.255,8.27),(2.085,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,8.27),(2.425,8.44),(2.595,8.44),(2.595,8.27),(2.425,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,8.27),(2.765,8.44),(2.935,8.44),(2.935,8.27),(2.765,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,8.27),(3.105,8.44),(3.275,8.44),(3.275,8.27),(3.105,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,8.27),(3.445,8.44),(3.615,8.44),(3.615,8.27),(3.445,8.27)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,8.61),(0.385,8.78),(0.555,8.78),(0.555,8.61),(0.385,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,8.61),(0.725,8.78),(0.895,8.78),(0.895,8.61),(0.725,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,8.61),(1.065,8.78),(1.235,8.78),(1.235,8.61),(1.065,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,8.61),(1.405,8.78),(1.575,8.78),(1.575,8.61),(1.405,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,8.61),(1.745,8.78),(1.915,8.78),(1.915,8.61),(1.745,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,8.61),(2.085,8.78),(2.255,8.78),(2.255,8.61),(2.085,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,8.61),(2.425,8.78),(2.595,8.78),(2.595,8.61),(2.425,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,8.61),(2.765,8.78),(2.935,8.78),(2.935,8.61),(2.765,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,8.61),(3.105,8.78),(3.275,8.78),(3.275,8.61),(3.105,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,8.61),(3.445,8.78),(3.615,8.78),(3.615,8.61),(3.445,8.61)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,8.95),(0.385,9.12),(0.555,9.12),(0.555,8.95),(0.385,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,8.95),(0.725,9.12),(0.895,9.12),(0.895,8.95),(0.725,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,8.95),(1.065,9.12),(1.235,9.12),(1.235,8.95),(1.065,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,8.95),(1.405,9.12),(1.575,9.12),(1.575,8.95),(1.405,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,8.95),(1.745,9.12),(1.915,9.12),(1.915,8.95),(1.745,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,8.95),(2.085,9.12),(2.255,9.12),(2.255,8.95),(2.085,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,8.95),(2.425,9.12),(2.595,9.12),(2.595,8.95),(2.425,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,8.95),(2.765,9.12),(2.935,9.12),(2.935,8.95),(2.765,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,8.95),(3.105,9.12),(3.275,9.12),(3.275,8.95),(3.105,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,8.95),(3.445,9.12),(3.615,9.12),(3.615,8.95),(3.445,8.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,9.29),(0.385,9.46),(0.555,9.46),(0.555,9.29),(0.385,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,9.29),(0.725,9.46),(0.895,9.46),(0.895,9.29),(0.725,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,9.29),(1.065,9.46),(1.235,9.46),(1.235,9.29),(1.065,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,9.29),(1.405,9.46),(1.575,9.46),(1.575,9.29),(1.405,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,9.29),(1.745,9.46),(1.915,9.46),(1.915,9.29),(1.745,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,9.29),(2.085,9.46),(2.255,9.46),(2.255,9.29),(2.085,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,9.29),(2.425,9.46),(2.595,9.46),(2.595,9.29),(2.425,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,9.29),(2.765,9.46),(2.935,9.46),(2.935,9.29),(2.765,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,9.29),(3.105,9.46),(3.275,9.46),(3.275,9.29),(3.105,9.29)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,9.29),(3.445,9.46),(3.615,9.46),(3.615,9.29),(3.445,9.29)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.385,1.2),(0.385,4.25),(3.615,4.25),(3.615,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.315,0.315),(0.315,4.455),(3.685,4.455),(3.685,0.315),(0.315,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,0.6),(0.385,0.77),(0.555,0.77),(0.555,0.6),(0.385,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,0.6),(0.725,0.77),(0.895,0.77),(0.895,0.6),(0.725,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,0.6),(1.065,0.77),(1.235,0.77),(1.235,0.6),(1.065,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,0.6),(1.405,0.77),(1.575,0.77),(1.575,0.6),(1.405,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,0.6),(1.745,0.77),(1.915,0.77),(1.915,0.6),(1.745,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,0.6),(2.085,0.77),(2.255,0.77),(2.255,0.6),(2.085,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,0.6),(2.425,0.77),(2.595,0.77),(2.595,0.6),(2.425,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,0.6),(2.765,0.77),(2.935,0.77),(2.935,0.6),(2.765,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,0.6),(3.105,0.77),(3.275,0.77),(3.275,0.6),(3.105,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,0.6),(3.445,0.77),(3.615,0.77),(3.615,0.6),(3.445,0.6)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,0.94),(0.385,1.11),(0.555,1.11),(0.555,0.94),(0.385,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,0.94),(0.725,1.11),(0.895,1.11),(0.895,0.94),(0.725,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,0.94),(1.065,1.11),(1.235,1.11),(1.235,0.94),(1.065,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,0.94),(1.405,1.11),(1.575,1.11),(1.575,0.94),(1.405,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,0.94),(1.745,1.11),(1.915,1.11),(1.915,0.94),(1.745,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,0.94),(2.085,1.11),(2.255,1.11),(2.255,0.94),(2.085,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,0.94),(2.425,1.11),(2.595,1.11),(2.595,0.94),(2.425,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,0.94),(2.765,1.11),(2.935,1.11),(2.935,0.94),(2.765,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,0.94),(3.105,1.11),(3.275,1.11),(3.275,0.94),(3.105,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,0.94),(3.445,1.11),(3.615,1.11),(3.615,0.94),(3.445,0.94)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,1.28),(0.385,1.45),(0.555,1.45),(0.555,1.28),(0.385,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,1.28),(0.725,1.45),(0.895,1.45),(0.895,1.28),(0.725,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,1.28),(1.065,1.45),(1.235,1.45),(1.235,1.28),(1.065,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,1.28),(1.405,1.45),(1.575,1.45),(1.575,1.28),(1.405,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,1.28),(1.745,1.45),(1.915,1.45),(1.915,1.28),(1.745,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,1.28),(2.085,1.45),(2.255,1.45),(2.255,1.28),(2.085,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,1.28),(2.425,1.45),(2.595,1.45),(2.595,1.28),(2.425,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,1.28),(2.765,1.45),(2.935,1.45),(2.935,1.28),(2.765,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,1.28),(3.105,1.45),(3.275,1.45),(3.275,1.28),(3.105,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,1.28),(3.445,1.45),(3.615,1.45),(3.615,1.28),(3.445,1.28)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,1.62),(0.385,1.79),(0.555,1.79),(0.555,1.62),(0.385,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,1.62),(0.725,1.79),(0.895,1.79),(0.895,1.62),(0.725,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,1.62),(1.065,1.79),(1.235,1.79),(1.235,1.62),(1.065,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,1.62),(1.405,1.79),(1.575,1.79),(1.575,1.62),(1.405,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,1.62),(1.745,1.79),(1.915,1.79),(1.915,1.62),(1.745,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,1.62),(2.085,1.79),(2.255,1.79),(2.255,1.62),(2.085,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,1.62),(2.425,1.79),(2.595,1.79),(2.595,1.62),(2.425,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,1.62),(2.765,1.79),(2.935,1.79),(2.935,1.62),(2.765,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,1.62),(3.105,1.79),(3.275,1.79),(3.275,1.62),(3.105,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,1.62),(3.445,1.79),(3.615,1.79),(3.615,1.62),(3.445,1.62)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,1.96),(0.385,2.13),(0.555,2.13),(0.555,1.96),(0.385,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,1.96),(0.725,2.13),(0.895,2.13),(0.895,1.96),(0.725,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,1.96),(1.065,2.13),(1.235,2.13),(1.235,1.96),(1.065,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,1.96),(1.405,2.13),(1.575,2.13),(1.575,1.96),(1.405,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,1.96),(1.745,2.13),(1.915,2.13),(1.915,1.96),(1.745,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,1.96),(2.085,2.13),(2.255,2.13),(2.255,1.96),(2.085,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,1.96),(2.425,2.13),(2.595,2.13),(2.595,1.96),(2.425,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,1.96),(2.765,2.13),(2.935,2.13),(2.935,1.96),(2.765,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,1.96),(3.105,2.13),(3.275,2.13),(3.275,1.96),(3.105,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,1.96),(3.445,2.13),(3.615,2.13),(3.615,1.96),(3.445,1.96)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,2.3),(0.385,2.47),(0.555,2.47),(0.555,2.3),(0.385,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,2.3),(0.725,2.47),(0.895,2.47),(0.895,2.3),(0.725,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,2.3),(1.065,2.47),(1.235,2.47),(1.235,2.3),(1.065,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,2.3),(1.405,2.47),(1.575,2.47),(1.575,2.3),(1.405,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,2.3),(1.745,2.47),(1.915,2.47),(1.915,2.3),(1.745,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,2.3),(2.085,2.47),(2.255,2.47),(2.255,2.3),(2.085,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,2.3),(2.425,2.47),(2.595,2.47),(2.595,2.3),(2.425,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,2.3),(2.765,2.47),(2.935,2.47),(2.935,2.3),(2.765,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,2.3),(3.105,2.47),(3.275,2.47),(3.275,2.3),(3.105,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,2.3),(3.445,2.47),(3.615,2.47),(3.615,2.3),(3.445,2.3)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,2.64),(0.385,2.81),(0.555,2.81),(0.555,2.64),(0.385,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,2.64),(0.725,2.81),(0.895,2.81),(0.895,2.64),(0.725,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,2.64),(1.065,2.81),(1.235,2.81),(1.235,2.64),(1.065,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,2.64),(1.405,2.81),(1.575,2.81),(1.575,2.64),(1.405,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,2.64),(1.745,2.81),(1.915,2.81),(1.915,2.64),(1.745,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,2.64),(2.085,2.81),(2.255,2.81),(2.255,2.64),(2.085,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,2.64),(2.425,2.81),(2.595,2.81),(2.595,2.64),(2.425,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,2.64),(2.765,2.81),(2.935,2.81),(2.935,2.64),(2.765,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,2.64),(3.105,2.81),(3.275,2.81),(3.275,2.64),(3.105,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,2.64),(3.445,2.81),(3.615,2.81),(3.615,2.64),(3.445,2.64)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,2.98),(0.385,3.15),(0.555,3.15),(0.555,2.98),(0.385,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,2.98),(0.725,3.15),(0.895,3.15),(0.895,2.98),(0.725,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,2.98),(1.065,3.15),(1.235,3.15),(1.235,2.98),(1.065,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,2.98),(1.405,3.15),(1.575,3.15),(1.575,2.98),(1.405,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,2.98),(1.745,3.15),(1.915,3.15),(1.915,2.98),(1.745,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,2.98),(2.085,3.15),(2.255,3.15),(2.255,2.98),(2.085,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,2.98),(2.425,3.15),(2.595,3.15),(2.595,2.98),(2.425,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,2.98),(2.765,3.15),(2.935,3.15),(2.935,2.98),(2.765,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,2.98),(3.105,3.15),(3.275,3.15),(3.275,2.98),(3.105,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,2.98),(3.445,3.15),(3.615,3.15),(3.615,2.98),(3.445,2.98)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,3.32),(0.385,3.49),(0.555,3.49),(0.555,3.32),(0.385,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,3.32),(0.725,3.49),(0.895,3.49),(0.895,3.32),(0.725,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,3.32),(1.065,3.49),(1.235,3.49),(1.235,3.32),(1.065,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,3.32),(1.405,3.49),(1.575,3.49),(1.575,3.32),(1.405,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,3.32),(1.745,3.49),(1.915,3.49),(1.915,3.32),(1.745,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,3.32),(2.085,3.49),(2.255,3.49),(2.255,3.32),(2.085,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,3.32),(2.425,3.49),(2.595,3.49),(2.595,3.32),(2.425,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,3.32),(2.765,3.49),(2.935,3.49),(2.935,3.32),(2.765,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,3.32),(3.105,3.49),(3.275,3.49),(3.275,3.32),(3.105,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,3.32),(3.445,3.49),(3.615,3.49),(3.615,3.32),(3.445,3.32)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,3.66),(0.385,3.83),(0.555,3.83),(0.555,3.66),(0.385,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,3.66),(0.725,3.83),(0.895,3.83),(0.895,3.66),(0.725,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,3.66),(1.065,3.83),(1.235,3.83),(1.235,3.66),(1.065,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,3.66),(1.405,3.83),(1.575,3.83),(1.575,3.66),(1.405,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,3.66),(1.745,3.83),(1.915,3.83),(1.915,3.66),(1.745,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,3.66),(2.085,3.83),(2.255,3.83),(2.255,3.66),(2.085,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,3.66),(2.425,3.83),(2.595,3.83),(2.595,3.66),(2.425,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,3.66),(2.765,3.83),(2.935,3.83),(2.935,3.66),(2.765,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,3.66),(3.105,3.83),(3.275,3.83),(3.275,3.66),(3.105,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,3.66),(3.445,3.83),(3.615,3.83),(3.615,3.66),(3.445,3.66)),
        )
        createRL(
            tech, net, 'licon',
            ((0.385,4.0),(0.385,4.17),(0.555,4.17),(0.555,4.0),(0.385,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((0.725,4.0),(0.725,4.17),(0.895,4.17),(0.895,4.0),(0.725,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((1.065,4.0),(1.065,4.17),(1.235,4.17),(1.235,4.0),(1.065,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((1.405,4.0),(1.405,4.17),(1.575,4.17),(1.575,4.0),(1.405,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((1.745,4.0),(1.745,4.17),(1.915,4.17),(1.915,4.0),(1.745,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((2.085,4.0),(2.085,4.17),(2.255,4.17),(2.255,4.0),(2.085,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((2.425,4.0),(2.425,4.17),(2.595,4.17),(2.595,4.0),(2.425,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((2.765,4.0),(2.765,4.17),(2.935,4.17),(2.935,4.0),(2.765,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((3.105,4.0),(3.105,4.17),(3.275,4.17),(3.275,4.0),(3.105,4.0)),
        )
        createRL(
            tech, net, 'licon',
            ((3.445,4.0),(3.445,4.17),(3.615,4.17),(3.615,4.0),(3.445,4.0)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.19,0.19),(0.19,4.58),(3.81,4.58),(3.81,0.19),(0.19,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,4.86),(0.19,9.81),(3.81,9.81),(3.81,4.86),(0.19,4.86)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['tie_poly_w4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(4.18,10.18),(4.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.795,9.095),(1.795,9.685),(2.205,9.685),(2.205,9.095),(1.795,9.095)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,9.135),(1.915,9.305),(2.085,9.305),(2.085,9.135),(1.915,9.135)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,9.475),(1.915,9.645),(2.085,9.645),(2.085,9.475),(1.915,9.475)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(1.915,1.2),(1.915,1.44),(2.085,1.44),(2.085,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((1.795,0.315),(1.795,0.905),(2.205,0.905),(2.205,0.315),(1.795,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,1.19),(1.915,1.36),(2.085,1.36),(2.085,1.19),(1.915,1.19)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,0.355),(1.915,0.525),(2.085,0.525),(2.085,0.355),(1.915,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,0.695),(1.915,0.865),(2.085,0.865),(2.085,0.695),(1.915,0.695)),
        )
        createRL(
            tech, net, 'poly',
            ((0.105,0.98),(0.105,9.02),(3.895,9.02),(3.895,0.98),(0.105,0.98)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((1.67,0.19),(1.67,1.03),(2.33,1.03),(2.33,0.19),(1.67,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((1.67,8.97),(1.67,9.81),(2.33,9.81),(2.33,8.97),(1.67,8.97)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['diode_w1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(1.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(1.0,10.0),(1.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.415,4.98),(0.415,8.63),(0.585,8.63),(0.585,4.98),(0.415,4.98)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(1.18,10.18),(1.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,8.935),(0.375,9.685),(0.625,9.685),(0.625,8.935),(0.375,8.935)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,4.98),(0.375,8.63),(0.625,8.63),(0.625,4.98),(0.375,4.98)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.055),(0.415,9.225),(0.585,9.225),(0.585,9.055),(0.415,9.055)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,9.395),(0.415,9.565),(0.585,9.565),(0.585,9.395),(0.415,9.395)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,5.19),(0.415,5.36),(0.585,5.36),(0.585,5.19),(0.415,5.19)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,5.53),(0.415,5.7),(0.585,5.7),(0.585,5.53),(0.415,5.53)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,5.87),(0.415,6.04),(0.585,6.04),(0.585,5.87),(0.415,5.87)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.21),(0.415,6.38),(0.585,6.38),(0.585,6.21),(0.415,6.21)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.55),(0.415,6.72),(0.585,6.72),(0.585,6.55),(0.415,6.55)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,6.89),(0.415,7.06),(0.585,7.06),(0.585,6.89),(0.415,6.89)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.23),(0.415,7.4),(0.585,7.4),(0.585,7.23),(0.415,7.23)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.57),(0.415,7.74),(0.585,7.74),(0.585,7.57),(0.415,7.57)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,7.91),(0.415,8.08),(0.585,8.08),(0.585,7.91),(0.415,7.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,8.25),(0.415,8.42),(0.585,8.42),(0.585,8.25),(0.415,8.25)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,4.46),(0.585,4.46),(0.585,1.37),(0.415,1.37)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(1.0,1.2),(1.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(1.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.375,0.315),(0.375,1.065),(0.625,1.065),(0.625,0.315),(0.375,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.375,1.37),(0.375,4.46),(0.625,4.46),(0.625,1.37),(0.375,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.64),(0.415,1.81),(0.585,1.81),(0.585,1.64),(0.415,1.64)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,1.98),(0.415,2.15),(0.585,2.15),(0.585,1.98),(0.415,1.98)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.32),(0.415,2.49),(0.585,2.49),(0.585,2.32),(0.415,2.32)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,2.66),(0.415,2.83),(0.585,2.83),(0.585,2.66),(0.415,2.66)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,3.0),(0.415,3.17),(0.585,3.17),(0.585,3.0),(0.415,3.0)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,3.34),(0.415,3.51),(0.585,3.51),(0.585,3.34),(0.415,3.34)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,3.68),(0.415,3.85),(0.585,3.85),(0.585,3.68),(0.415,3.68)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,4.02),(0.415,4.19),(0.585,4.19),(0.585,4.02),(0.415,4.02)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.435),(0.415,0.605),(0.585,0.605),(0.585,0.435),(0.415,0.435)),
        )
        createRL(
            tech, net, 'licon',
            ((0.415,0.775),(0.415,0.945),(0.585,0.945),(0.585,0.775),(0.415,0.775)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.25,0.19),(0.25,1.19),(0.75,1.19),(0.75,0.19),(0.25,0.19)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.25,4.855),(0.25,8.755),(0.75,8.755),(0.75,4.855),(0.25,4.855)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,1.245),(0.25,4.585),(0.75,4.585),(0.75,1.245),(0.25,1.245)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.25,8.81),(0.25,9.81),(0.75,9.81),(0.75,8.81),(0.25,8.81)),
        )
        net = nets['i']
        createRL(
            tech, net, 'li',
            ((0.415,1.37),(0.415,8.63),(0.585,8.63),(0.585,1.37),(0.415,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.5), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.5), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)

    cell = new_cells['zero_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'one': Net.create(cell, 'one'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
            'zero': Net.create(cell, 'zero'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((1.13,5.59),(1.13,8.8),(0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(1.3,8.8),(1.3,5.59),(1.13,5.59)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.625,9.435),(0.625,9.685),(1.375,9.685),(1.375,9.435),(0.625,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,5.41),(0.925,9.16),(1.34,9.16),(1.34,5.41),(0.925,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,9.475),(0.745,9.645),(0.915,9.645),(0.915,9.475),(0.745,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,9.475),(1.085,9.645),(1.255,9.645),(1.255,9.475),(1.085,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,5.67),(1.13,5.84),(1.3,5.84),(1.3,5.67),(1.13,5.67)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.01),(1.13,6.18),(1.3,6.18),(1.3,6.01),(1.13,6.01)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.35),(1.13,6.52),(1.3,6.52),(1.3,6.35),(1.13,6.35)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.69),(1.13,6.86),(1.3,6.86),(1.3,6.69),(1.13,6.69)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.03),(1.13,7.2),(1.3,7.2),(1.3,7.03),(1.13,7.03)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.37),(1.13,7.54),(1.3,7.54),(1.3,7.37),(1.13,7.37)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.71),(1.13,7.88),(1.3,7.88),(1.3,7.71),(1.13,7.71)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.05),(1.13,8.22),(1.3,8.22),(1.3,8.05),(1.13,8.05)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.39),(1.13,8.56),(1.3,8.56),(1.3,8.39),(1.13,8.39)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.73),(1.13,8.9),(1.3,8.9),(1.3,8.73),(1.13,8.73)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.7,1.2),(0.7,4.035),(0.87,4.035),(0.87,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.625,0.315),(0.625,0.565),(1.375,0.565),(1.375,0.315),(0.625,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,0.835),(0.66,4.185),(1.075,4.185),(1.075,0.835),(0.66,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,0.355),(0.745,0.525),(0.915,0.525),(0.915,0.355),(0.745,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,0.355),(1.085,0.525),(1.255,0.525),(1.255,0.355),(1.085,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.065),(0.7,1.235),(0.87,1.235),(0.87,1.065),(0.7,1.065)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.405),(0.7,1.575),(0.87,1.575),(0.87,1.405),(0.7,1.405)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.745),(0.7,1.915),(0.87,1.915),(0.87,1.745),(0.7,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.085),(0.7,2.255),(0.87,2.255),(0.87,2.085),(0.7,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.425),(0.7,2.595),(0.87,2.595),(0.87,2.425),(0.7,2.425)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.765),(0.7,2.935),(0.87,2.935),(0.87,2.765),(0.7,2.765)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.105),(0.7,3.275),(0.87,3.275),(0.87,3.105),(0.7,3.105)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.445),(0.7,3.615),(0.87,3.615),(0.87,3.445),(0.7,3.445)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.785),(0.7,3.955),(0.87,3.955),(0.87,3.785),(0.7,3.785)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.535,5.285),(0.535,9.285),(1.465,9.285),(1.465,5.285),(0.535,5.285)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.5,0.19),(0.5,0.69),(1.5,0.69),(1.5,0.19),(0.5,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.535,0.71),(0.535,4.31),(1.465,4.31),(1.465,0.71),(0.535,0.71)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.5,9.31),(0.5,9.81),(1.5,9.81),(1.5,9.31),(0.5,9.31)),
        )
        net = nets['zero']
        createRL(
            tech, net, 'difftap',
            ((1.075,0.835),(1.075,4.185),(1.34,4.185),(1.34,0.835),(1.075,0.835)),
        )
        createRL(
            tech, net, 'poly',
            ((0.925,4.905),(0.925,9.29),(1.075,9.29),(1.075,5.175),(1.38,5.175),(1.38,4.905),(0.925,4.905)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,4.955),(1.13,5.125),(1.3,5.125),(1.3,4.955),(1.13,4.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.595),(1.13,1.765),(1.3,1.765),(1.3,1.595),(1.13,1.595)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.935),(1.13,2.105),(1.3,2.105),(1.3,1.935),(1.13,1.935)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.275),(1.13,2.445),(1.3,2.445),(1.3,2.275),(1.13,2.275)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.615),(1.13,2.785),(1.3,2.785),(1.3,2.615),(1.13,2.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.955),(1.13,3.125),(1.3,3.125),(1.3,2.955),(1.13,2.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.295),(1.13,3.465),(1.3,3.465),(1.3,3.295),(1.13,3.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.635),(1.13,3.805),(1.3,3.805),(1.3,3.635),(1.13,3.635)),
        )
        createRL(
            tech, net, 'li',
            ((1.13,1.37),(1.13,5.205),(1.3,5.205),(1.3,1.37),(1.13,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.215), u(0.17), u(1.37), u(5.205),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.215), u(0.17), u(1.37), u(5.205),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['one']
        createRL(
            tech, net, 'poly',
            ((0.925,0.705),(0.925,4.425),(0.62,4.425),(0.62,4.695),(1.075,4.695),(1.075,0.705),(0.925,0.705)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,5.41),(0.66,9.16),(0.925,9.16),(0.925,5.41),(0.66,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,4.475),(0.7,4.645),(0.87,4.645),(0.87,4.475),(0.7,4.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,5.835),(0.7,6.005),(0.87,6.005),(0.87,5.835),(0.7,5.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.175),(0.7,6.345),(0.87,6.345),(0.87,6.175),(0.7,6.175)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.515),(0.7,6.685),(0.87,6.685),(0.87,6.515),(0.7,6.515)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.855),(0.7,7.025),(0.87,7.025),(0.87,6.855),(0.7,6.855)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.195),(0.7,7.365),(0.87,7.365),(0.87,7.195),(0.7,7.195)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.535),(0.7,7.705),(0.87,7.705),(0.87,7.535),(0.7,7.535)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.875),(0.7,8.045),(0.87,8.045),(0.87,7.875),(0.7,7.875)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,8.215),(0.7,8.385),(0.87,8.385),(0.87,8.215),(0.7,8.215)),
        )
        createRL(
            tech, net, 'li',
            ((0.7,4.395),(0.7,8.63),(0.87,8.63),(0.87,4.395),(0.7,4.395)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['zero'].setDirection(Net.Direction.OUT)

    cell = new_cells['one_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'one': Net.create(cell, 'one'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
            'zero': Net.create(cell, 'zero'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((1.13,5.59),(1.13,8.8),(0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(1.3,8.8),(1.3,5.59),(1.13,5.59)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.625,9.435),(0.625,9.685),(1.375,9.685),(1.375,9.435),(0.625,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,5.41),(0.925,9.16),(1.34,9.16),(1.34,5.41),(0.925,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,9.475),(0.745,9.645),(0.915,9.645),(0.915,9.475),(0.745,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,9.475),(1.085,9.645),(1.255,9.645),(1.255,9.475),(1.085,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,5.67),(1.13,5.84),(1.3,5.84),(1.3,5.67),(1.13,5.67)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.01),(1.13,6.18),(1.3,6.18),(1.3,6.01),(1.13,6.01)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.35),(1.13,6.52),(1.3,6.52),(1.3,6.35),(1.13,6.35)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.69),(1.13,6.86),(1.3,6.86),(1.3,6.69),(1.13,6.69)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.03),(1.13,7.2),(1.3,7.2),(1.3,7.03),(1.13,7.03)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.37),(1.13,7.54),(1.3,7.54),(1.3,7.37),(1.13,7.37)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.71),(1.13,7.88),(1.3,7.88),(1.3,7.71),(1.13,7.71)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.05),(1.13,8.22),(1.3,8.22),(1.3,8.05),(1.13,8.05)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.39),(1.13,8.56),(1.3,8.56),(1.3,8.39),(1.13,8.39)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.73),(1.13,8.9),(1.3,8.9),(1.3,8.73),(1.13,8.73)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.7,1.2),(0.7,4.035),(0.87,4.035),(0.87,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.625,0.315),(0.625,0.565),(1.375,0.565),(1.375,0.315),(0.625,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,0.835),(0.66,4.185),(1.075,4.185),(1.075,0.835),(0.66,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,0.355),(0.745,0.525),(0.915,0.525),(0.915,0.355),(0.745,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,0.355),(1.085,0.525),(1.255,0.525),(1.255,0.355),(1.085,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.065),(0.7,1.235),(0.87,1.235),(0.87,1.065),(0.7,1.065)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.405),(0.7,1.575),(0.87,1.575),(0.87,1.405),(0.7,1.405)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.745),(0.7,1.915),(0.87,1.915),(0.87,1.745),(0.7,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.085),(0.7,2.255),(0.87,2.255),(0.87,2.085),(0.7,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.425),(0.7,2.595),(0.87,2.595),(0.87,2.425),(0.7,2.425)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.765),(0.7,2.935),(0.87,2.935),(0.87,2.765),(0.7,2.765)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.105),(0.7,3.275),(0.87,3.275),(0.87,3.105),(0.7,3.105)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.445),(0.7,3.615),(0.87,3.615),(0.87,3.445),(0.7,3.445)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.785),(0.7,3.955),(0.87,3.955),(0.87,3.785),(0.7,3.785)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.535,5.285),(0.535,9.285),(1.465,9.285),(1.465,5.285),(0.535,5.285)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.5,0.19),(0.5,0.69),(1.5,0.69),(1.5,0.19),(0.5,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.535,0.71),(0.535,4.31),(1.465,4.31),(1.465,0.71),(0.535,0.71)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.5,9.31),(0.5,9.81),(1.5,9.81),(1.5,9.31),(0.5,9.31)),
        )
        net = nets['zero']
        createRL(
            tech, net, 'difftap',
            ((1.075,0.835),(1.075,4.185),(1.34,4.185),(1.34,0.835),(1.075,0.835)),
        )
        createRL(
            tech, net, 'poly',
            ((0.925,4.905),(0.925,9.29),(1.075,9.29),(1.075,5.175),(1.38,5.175),(1.38,4.905),(0.925,4.905)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,4.955),(1.13,5.125),(1.3,5.125),(1.3,4.955),(1.13,4.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.595),(1.13,1.765),(1.3,1.765),(1.3,1.595),(1.13,1.595)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.935),(1.13,2.105),(1.3,2.105),(1.3,1.935),(1.13,1.935)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.275),(1.13,2.445),(1.3,2.445),(1.3,2.275),(1.13,2.275)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.615),(1.13,2.785),(1.3,2.785),(1.3,2.615),(1.13,2.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.955),(1.13,3.125),(1.3,3.125),(1.3,2.955),(1.13,2.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.295),(1.13,3.465),(1.3,3.465),(1.3,3.295),(1.13,3.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.635),(1.13,3.805),(1.3,3.805),(1.3,3.635),(1.13,3.635)),
        )
        createRL(
            tech, net, 'li',
            ((1.13,1.37),(1.13,5.205),(1.3,5.205),(1.3,1.37),(1.13,1.37)),
        )
        net = nets['one']
        createRL(
            tech, net, 'poly',
            ((0.925,0.705),(0.925,4.425),(0.62,4.425),(0.62,4.695),(1.075,4.695),(1.075,0.705),(0.925,0.705)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,5.41),(0.66,9.16),(0.925,9.16),(0.925,5.41),(0.66,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,4.475),(0.7,4.645),(0.87,4.645),(0.87,4.475),(0.7,4.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,5.835),(0.7,6.005),(0.87,6.005),(0.87,5.835),(0.7,5.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.175),(0.7,6.345),(0.87,6.345),(0.87,6.175),(0.7,6.175)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.515),(0.7,6.685),(0.87,6.685),(0.87,6.515),(0.7,6.515)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.855),(0.7,7.025),(0.87,7.025),(0.87,6.855),(0.7,6.855)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.195),(0.7,7.365),(0.87,7.365),(0.87,7.195),(0.7,7.195)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.535),(0.7,7.705),(0.87,7.705),(0.87,7.535),(0.7,7.535)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.875),(0.7,8.045),(0.87,8.045),(0.87,7.875),(0.7,7.875)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,8.215),(0.7,8.385),(0.87,8.385),(0.87,8.215),(0.7,8.215)),
        )
        createRL(
            tech, net, 'li',
            ((0.7,4.395),(0.7,8.63),(0.87,8.63),(0.87,4.395),(0.7,4.395)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.785), u(0.17), u(4.395), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.785), u(0.17), u(4.395), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['one'].setDirection(Net.Direction.OUT)

    cell = new_cells['zeroone_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'one': Net.create(cell, 'one'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
            'zero': Net.create(cell, 'zero'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((1.13,5.59),(1.13,8.8),(0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(1.3,8.8),(1.3,5.59),(1.13,5.59)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.625,9.435),(0.625,9.685),(1.375,9.685),(1.375,9.435),(0.625,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,5.41),(0.925,9.16),(1.34,9.16),(1.34,5.41),(0.925,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,9.475),(0.745,9.645),(0.915,9.645),(0.915,9.475),(0.745,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,9.475),(1.085,9.645),(1.255,9.645),(1.255,9.475),(1.085,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,5.67),(1.13,5.84),(1.3,5.84),(1.3,5.67),(1.13,5.67)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.01),(1.13,6.18),(1.3,6.18),(1.3,6.01),(1.13,6.01)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.35),(1.13,6.52),(1.3,6.52),(1.3,6.35),(1.13,6.35)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.69),(1.13,6.86),(1.3,6.86),(1.3,6.69),(1.13,6.69)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.03),(1.13,7.2),(1.3,7.2),(1.3,7.03),(1.13,7.03)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.37),(1.13,7.54),(1.3,7.54),(1.3,7.37),(1.13,7.37)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.71),(1.13,7.88),(1.3,7.88),(1.3,7.71),(1.13,7.71)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.05),(1.13,8.22),(1.3,8.22),(1.3,8.05),(1.13,8.05)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.39),(1.13,8.56),(1.3,8.56),(1.3,8.39),(1.13,8.39)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.73),(1.13,8.9),(1.3,8.9),(1.3,8.73),(1.13,8.73)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.7,1.2),(0.7,4.035),(0.87,4.035),(0.87,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.625,0.315),(0.625,0.565),(1.375,0.565),(1.375,0.315),(0.625,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,0.835),(0.66,4.185),(1.075,4.185),(1.075,0.835),(0.66,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,0.355),(0.745,0.525),(0.915,0.525),(0.915,0.355),(0.745,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,0.355),(1.085,0.525),(1.255,0.525),(1.255,0.355),(1.085,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.065),(0.7,1.235),(0.87,1.235),(0.87,1.065),(0.7,1.065)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.405),(0.7,1.575),(0.87,1.575),(0.87,1.405),(0.7,1.405)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.745),(0.7,1.915),(0.87,1.915),(0.87,1.745),(0.7,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.085),(0.7,2.255),(0.87,2.255),(0.87,2.085),(0.7,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.425),(0.7,2.595),(0.87,2.595),(0.87,2.425),(0.7,2.425)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.765),(0.7,2.935),(0.87,2.935),(0.87,2.765),(0.7,2.765)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.105),(0.7,3.275),(0.87,3.275),(0.87,3.105),(0.7,3.105)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.445),(0.7,3.615),(0.87,3.615),(0.87,3.445),(0.7,3.445)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.785),(0.7,3.955),(0.87,3.955),(0.87,3.785),(0.7,3.785)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.535,5.285),(0.535,9.285),(1.465,9.285),(1.465,5.285),(0.535,5.285)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.5,0.19),(0.5,0.69),(1.5,0.69),(1.5,0.19),(0.5,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.535,0.71),(0.535,4.31),(1.465,4.31),(1.465,0.71),(0.535,0.71)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.5,9.31),(0.5,9.81),(1.5,9.81),(1.5,9.31),(0.5,9.31)),
        )
        net = nets['zero']
        createRL(
            tech, net, 'difftap',
            ((1.075,0.835),(1.075,4.185),(1.34,4.185),(1.34,0.835),(1.075,0.835)),
        )
        createRL(
            tech, net, 'poly',
            ((0.925,4.905),(0.925,9.29),(1.075,9.29),(1.075,5.175),(1.38,5.175),(1.38,4.905),(0.925,4.905)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,4.955),(1.13,5.125),(1.3,5.125),(1.3,4.955),(1.13,4.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.595),(1.13,1.765),(1.3,1.765),(1.3,1.595),(1.13,1.595)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.935),(1.13,2.105),(1.3,2.105),(1.3,1.935),(1.13,1.935)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.275),(1.13,2.445),(1.3,2.445),(1.3,2.275),(1.13,2.275)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.615),(1.13,2.785),(1.3,2.785),(1.3,2.615),(1.13,2.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.955),(1.13,3.125),(1.3,3.125),(1.3,2.955),(1.13,2.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.295),(1.13,3.465),(1.3,3.465),(1.3,3.295),(1.13,3.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.635),(1.13,3.805),(1.3,3.805),(1.3,3.635),(1.13,3.635)),
        )
        createRL(
            tech, net, 'li',
            ((1.13,1.37),(1.13,5.205),(1.3,5.205),(1.3,1.37),(1.13,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.215), u(0.17), u(1.37), u(5.205),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.215), u(0.17), u(1.37), u(5.205),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['one']
        createRL(
            tech, net, 'poly',
            ((0.925,0.705),(0.925,4.425),(0.62,4.425),(0.62,4.695),(1.075,4.695),(1.075,0.705),(0.925,0.705)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,5.41),(0.66,9.16),(0.925,9.16),(0.925,5.41),(0.66,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,4.475),(0.7,4.645),(0.87,4.645),(0.87,4.475),(0.7,4.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,5.835),(0.7,6.005),(0.87,6.005),(0.87,5.835),(0.7,5.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.175),(0.7,6.345),(0.87,6.345),(0.87,6.175),(0.7,6.175)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.515),(0.7,6.685),(0.87,6.685),(0.87,6.515),(0.7,6.515)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.855),(0.7,7.025),(0.87,7.025),(0.87,6.855),(0.7,6.855)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.195),(0.7,7.365),(0.87,7.365),(0.87,7.195),(0.7,7.195)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.535),(0.7,7.705),(0.87,7.705),(0.87,7.535),(0.7,7.535)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.875),(0.7,8.045),(0.87,8.045),(0.87,7.875),(0.7,7.875)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,8.215),(0.7,8.385),(0.87,8.385),(0.87,8.215),(0.7,8.215)),
        )
        createRL(
            tech, net, 'li',
            ((0.7,4.395),(0.7,8.63),(0.87,8.63),(0.87,4.395),(0.7,4.395)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.785), u(0.17), u(4.395), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.785), u(0.17), u(4.395), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['zero'].setDirection(Net.Direction.OUT)
        nets['one'].setDirection(Net.Direction.OUT)

    cell = new_cells['decap_w0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'one': Net.create(cell, 'one'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
            'zero': Net.create(cell, 'zero'),
        }
        net = nets['vdd']
        createRL(
            tech, net, 'li',
            ((1.13,5.59),(1.13,8.8),(0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(1.3,8.8),(1.3,5.59),(1.13,5.59)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.625,9.435),(0.625,9.685),(1.375,9.685),(1.375,9.435),(0.625,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.925,5.41),(0.925,9.16),(1.34,9.16),(1.34,5.41),(0.925,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,9.475),(0.745,9.645),(0.915,9.645),(0.915,9.475),(0.745,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,9.475),(1.085,9.645),(1.255,9.645),(1.255,9.475),(1.085,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,5.67),(1.13,5.84),(1.3,5.84),(1.3,5.67),(1.13,5.67)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.01),(1.13,6.18),(1.3,6.18),(1.3,6.01),(1.13,6.01)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.35),(1.13,6.52),(1.3,6.52),(1.3,6.35),(1.13,6.35)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,6.69),(1.13,6.86),(1.3,6.86),(1.3,6.69),(1.13,6.69)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.03),(1.13,7.2),(1.3,7.2),(1.3,7.03),(1.13,7.03)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.37),(1.13,7.54),(1.3,7.54),(1.3,7.37),(1.13,7.37)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,7.71),(1.13,7.88),(1.3,7.88),(1.3,7.71),(1.13,7.71)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.05),(1.13,8.22),(1.3,8.22),(1.3,8.05),(1.13,8.05)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.39),(1.13,8.56),(1.3,8.56),(1.3,8.39),(1.13,8.39)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,8.73),(1.13,8.9),(1.3,8.9),(1.3,8.73),(1.13,8.73)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(0.7,1.2),(0.7,4.035),(0.87,4.035),(0.87,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'difftap',
            ((0.625,0.315),(0.625,0.565),(1.375,0.565),(1.375,0.315),(0.625,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,0.835),(0.66,4.185),(1.075,4.185),(1.075,0.835),(0.66,0.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.745,0.355),(0.745,0.525),(0.915,0.525),(0.915,0.355),(0.745,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.085,0.355),(1.085,0.525),(1.255,0.525),(1.255,0.355),(1.085,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.065),(0.7,1.235),(0.87,1.235),(0.87,1.065),(0.7,1.065)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.405),(0.7,1.575),(0.87,1.575),(0.87,1.405),(0.7,1.405)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,1.745),(0.7,1.915),(0.87,1.915),(0.87,1.745),(0.7,1.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.085),(0.7,2.255),(0.87,2.255),(0.87,2.085),(0.7,2.085)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.425),(0.7,2.595),(0.87,2.595),(0.87,2.425),(0.7,2.425)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,2.765),(0.7,2.935),(0.87,2.935),(0.87,2.765),(0.7,2.765)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.105),(0.7,3.275),(0.87,3.275),(0.87,3.105),(0.7,3.105)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.445),(0.7,3.615),(0.87,3.615),(0.87,3.445),(0.7,3.445)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,3.785),(0.7,3.955),(0.87,3.955),(0.87,3.785),(0.7,3.785)),
        )
        net = nets['*']
        createRL(
            tech, net, 'psdm',
            ((0.535,5.285),(0.535,9.285),(1.465,9.285),(1.465,5.285),(0.535,5.285)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.5,0.19),(0.5,0.69),(1.5,0.69),(1.5,0.19),(0.5,0.19)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.535,0.71),(0.535,4.31),(1.465,4.31),(1.465,0.71),(0.535,0.71)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.5,9.31),(0.5,9.81),(1.5,9.81),(1.5,9.31),(0.5,9.31)),
        )
        net = nets['zero']
        createRL(
            tech, net, 'difftap',
            ((1.075,0.835),(1.075,4.185),(1.34,4.185),(1.34,0.835),(1.075,0.835)),
        )
        createRL(
            tech, net, 'poly',
            ((0.925,4.905),(0.925,9.29),(1.075,9.29),(1.075,5.175),(1.38,5.175),(1.38,4.905),(0.925,4.905)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,4.955),(1.13,5.125),(1.3,5.125),(1.3,4.955),(1.13,4.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.595),(1.13,1.765),(1.3,1.765),(1.3,1.595),(1.13,1.595)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,1.935),(1.13,2.105),(1.3,2.105),(1.3,1.935),(1.13,1.935)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.275),(1.13,2.445),(1.3,2.445),(1.3,2.275),(1.13,2.275)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.615),(1.13,2.785),(1.3,2.785),(1.3,2.615),(1.13,2.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,2.955),(1.13,3.125),(1.3,3.125),(1.3,2.955),(1.13,2.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.295),(1.13,3.465),(1.3,3.465),(1.3,3.295),(1.13,3.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.13,3.635),(1.13,3.805),(1.3,3.805),(1.3,3.635),(1.13,3.635)),
        )
        createRL(
            tech, net, 'li',
            ((1.13,1.37),(1.13,5.205),(1.3,5.205),(1.3,1.37),(1.13,1.37)),
        )
        net = nets['one']
        createRL(
            tech, net, 'poly',
            ((0.925,0.705),(0.925,4.425),(0.62,4.425),(0.62,4.695),(1.075,4.695),(1.075,0.705),(0.925,0.705)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.66,5.41),(0.66,9.16),(0.925,9.16),(0.925,5.41),(0.66,5.41)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,4.475),(0.7,4.645),(0.87,4.645),(0.87,4.475),(0.7,4.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,5.835),(0.7,6.005),(0.87,6.005),(0.87,5.835),(0.7,5.835)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.175),(0.7,6.345),(0.87,6.345),(0.87,6.175),(0.7,6.175)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.515),(0.7,6.685),(0.87,6.685),(0.87,6.515),(0.7,6.515)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,6.855),(0.7,7.025),(0.87,7.025),(0.87,6.855),(0.7,6.855)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.195),(0.7,7.365),(0.87,7.365),(0.87,7.195),(0.7,7.195)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.535),(0.7,7.705),(0.87,7.705),(0.87,7.535),(0.7,7.535)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,7.875),(0.7,8.045),(0.87,8.045),(0.87,7.875),(0.7,7.875)),
        )
        createRL(
            tech, net, 'licon',
            ((0.7,8.215),(0.7,8.385),(0.87,8.385),(0.87,8.215),(0.7,8.215)),
        )
        createRL(
            tech, net, 'li',
            ((0.7,4.395),(0.7,8.63),(0.87,8.63),(0.87,4.395),(0.7,4.395)),
        )
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)

    cell = new_cells['inv_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['i']
        createRL(
            tech, net, 'li',
            ((0.17,1.37),(0.17,8.63),(0.34,8.63),(0.34,4.965),(0.425,4.965),(0.425,4.635),(0.34,4.635),(0.34,1.37),(0.17,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'poly',
            ((0.64,1.075),(0.64,4.635),(0.205,4.635),(0.205,4.965),(0.64,4.965),(0.64,8.915),(0.79,8.915),(0.79,1.075),(0.64,1.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.255,4.715),(0.255,4.885),(0.425,4.885),(0.425,4.715),(0.255,4.715)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.935,0.315),(0.935,0.565),(1.685,0.565),(1.685,0.315),(0.935,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.91),(0.315,2.205),(0.79,2.205),(0.79,1.205),(0.565,1.205),(0.565,0.91),(0.315,0.91)),
        )
        createRL(
            tech, net, 'licon',
            ((1.055,0.355),(1.055,0.525),(1.225,0.525),(1.225,0.355),(1.055,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.395,0.355),(1.395,0.525),(1.565,0.525),(1.565,0.355),(1.395,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,1.03),(0.355,1.2),(0.525,1.2),(0.525,1.03),(0.355,1.03)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.81,9.31),(0.81,9.81),(1.81,9.81),(1.81,9.31),(0.81,9.31)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.785),(0.19,2.33),(1.18,2.33),(1.18,1.08),(0.69,1.08),(0.69,0.785),(0.19,0.785)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,6.66),(0.19,9.215),(0.69,9.215),(0.69,8.91),(1.18,8.91),(1.18,6.66),(0.19,6.66)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.81,0.19),(0.81,0.69),(1.81,0.69),(1.81,0.19),(0.81,0.19)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((0.935,9.435),(0.935,9.685),(1.685,9.685),(1.685,9.435),(0.935,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,6.785),(0.315,9.09),(0.565,9.09),(0.565,8.785),(0.79,8.785),(0.79,6.785),(0.315,6.785)),
        )
        createRL(
            tech, net, 'licon',
            ((1.055,9.475),(1.055,9.645),(1.225,9.645),(1.225,9.475),(1.055,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.395,9.475),(1.395,9.645),(1.565,9.645),(1.565,9.475),(1.395,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.8),(0.355,8.97),(0.525,8.97),(0.525,8.8),(0.355,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((0.79,6.785),(0.79,8.785),(1.055,8.785),(1.055,6.785),(0.79,6.785)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,1.205),(0.79,2.205),(1.055,2.205),(1.055,1.205),(0.79,1.205)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.45),(0.845,1.62),(1.015,1.62),(1.015,1.45),(0.845,1.45)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.79),(0.845,1.96),(1.015,1.96),(1.015,1.79),(0.845,1.79)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.02),(0.845,7.19),(1.015,7.19),(1.015,7.02),(0.845,7.02)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.36),(0.845,7.53),(1.015,7.53),(1.015,7.36),(0.845,7.36)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.7),(0.845,7.87),(1.015,7.87),(1.015,7.7),(0.845,7.7)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.04),(0.845,8.21),(1.015,8.21),(1.015,8.04),(0.845,8.04)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.38),(0.845,8.55),(1.015,8.55),(1.015,8.38),(0.845,8.38)),
        )
        createRL(
            tech, net, 'li',
            ((0.845,1.37),(0.845,8.63),(1.015,8.63),(1.015,1.37),(0.845,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)

    cell = new_cells['inv_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['i']
        createRL(
            tech, net, 'li',
            ((0.17,1.37),(0.17,8.63),(0.34,8.63),(0.34,4.965),(0.425,4.965),(0.425,4.635),(0.34,4.635),(0.34,1.37),(0.17,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'poly',
            ((0.64,1.185),(0.64,4.635),(0.205,4.635),(0.205,4.965),(0.64,4.965),(0.64,8.865),(0.79,8.865),(0.79,1.185),(0.64,1.185)),
        )
        createRL(
            tech, net, 'licon',
            ((0.255,4.715),(0.255,4.885),(0.425,4.885),(0.425,4.715),(0.255,4.715)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.935,0.315),(0.935,0.565),(1.685,0.565),(1.685,0.315),(0.935,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.91),(0.315,3.115),(0.79,3.115),(0.79,1.315),(0.565,1.315),(0.565,0.91),(0.315,0.91)),
        )
        createRL(
            tech, net, 'licon',
            ((1.055,0.355),(1.055,0.525),(1.225,0.525),(1.225,0.355),(1.055,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.395,0.355),(1.395,0.525),(1.565,0.525),(1.565,0.355),(1.395,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,1.03),(0.355,1.2),(0.525,1.2),(0.525,1.03),(0.355,1.03)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.785),(0.19,3.24),(1.18,3.24),(1.18,1.19),(0.69,1.19),(0.69,0.785),(0.19,0.785)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.81,9.31),(0.81,9.81),(1.81,9.81),(1.81,9.31),(0.81,9.31)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.01),(0.19,9.215),(0.69,9.215),(0.69,8.86),(1.18,8.86),(1.18,5.01),(0.19,5.01)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.81,0.19),(0.81,0.69),(1.81,0.69),(1.81,0.19),(0.81,0.19)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((0.935,9.435),(0.935,9.685),(1.685,9.685),(1.685,9.435),(0.935,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,5.135),(0.315,9.09),(0.565,9.09),(0.565,8.735),(0.79,8.735),(0.79,5.135),(0.315,5.135)),
        )
        createRL(
            tech, net, 'licon',
            ((1.055,9.475),(1.055,9.645),(1.225,9.645),(1.225,9.475),(1.055,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.395,9.475),(1.395,9.645),(1.565,9.645),(1.565,9.475),(1.395,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.8),(0.355,8.97),(0.525,8.97),(0.525,8.8),(0.355,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((0.79,1.315),(0.79,3.115),(1.055,3.115),(1.055,1.315),(0.79,1.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,5.135),(0.79,8.735),(1.055,8.735),(1.055,5.135),(0.79,5.135)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.45),(0.845,1.62),(1.015,1.62),(1.015,1.45),(0.845,1.45)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.79),(0.845,1.96),(1.015,1.96),(1.015,1.79),(0.845,1.79)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.13),(0.845,2.3),(1.015,2.3),(1.015,2.13),(0.845,2.13)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.47),(0.845,2.64),(1.015,2.64),(1.015,2.47),(0.845,2.47)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.81),(0.845,2.98),(1.015,2.98),(1.015,2.81),(0.845,2.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,5.49),(0.845,5.66),(1.015,5.66),(1.015,5.49),(0.845,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,5.83),(0.845,6.0),(1.015,6.0),(1.015,5.83),(0.845,5.83)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.17),(0.845,6.34),(1.015,6.34),(1.015,6.17),(0.845,6.17)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.51),(0.845,6.68),(1.015,6.68),(1.015,6.51),(0.845,6.51)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.85),(0.845,7.02),(1.015,7.02),(1.015,6.85),(0.845,6.85)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.19),(0.845,7.36),(1.015,7.36),(1.015,7.19),(0.845,7.19)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.53),(0.845,7.7),(1.015,7.7),(1.015,7.53),(0.845,7.53)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.87),(0.845,8.04),(1.015,8.04),(1.015,7.87),(0.845,7.87)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.21),(0.845,8.38),(1.015,8.38),(1.015,8.21),(0.845,8.21)),
        )
        createRL(
            tech, net, 'li',
            ((0.845,1.37),(0.845,8.63),(1.015,8.63),(1.015,1.37),(0.845,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)

    cell = new_cells['inv_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['i']
        createRL(
            tech, net, 'li',
            ((0.17,1.37),(0.17,8.63),(0.34,8.63),(0.34,4.965),(0.425,4.965),(0.425,4.635),(0.34,4.635),(0.34,1.37),(0.17,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'poly',
            ((0.64,1.185),(0.64,4.635),(0.205,4.635),(0.205,4.965),(0.64,4.965),(0.64,8.865),(0.79,8.865),(0.79,4.965),(1.07,4.965),(1.07,8.865),(1.22,8.865),(1.22,1.185),(1.07,1.185),(1.07,4.635),(0.79,4.635),(0.79,1.185),(0.64,1.185)),
        )
        createRL(
            tech, net, 'licon',
            ((0.255,4.715),(0.255,4.885),(0.425,4.885),(0.425,4.715),(0.255,4.715)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((1.295,0.91),(1.295,1.315),(1.07,1.315),(1.07,3.115),(1.545,3.115),(1.545,0.91),(1.295,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.91),(0.315,3.115),(0.79,3.115),(0.79,1.315),(0.565,1.315),(0.565,0.91),(0.315,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.935,0.315),(0.935,0.565),(1.685,0.565),(1.685,0.315),(0.935,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((1.055,0.355),(1.055,0.525),(1.225,0.525),(1.225,0.355),(1.055,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.395,0.355),(1.395,0.525),(1.565,0.525),(1.565,0.355),(1.395,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.335,1.03),(1.335,1.2),(1.505,1.2),(1.505,1.03),(1.335,1.03)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,1.03),(0.355,1.2),(0.525,1.2),(0.525,1.03),(0.355,1.03)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.81,9.31),(0.81,9.81),(1.81,9.81),(1.81,9.31),(0.81,9.31)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.785),(0.19,3.24),(1.67,3.24),(1.67,0.785),(1.17,0.785),(1.17,1.19),(0.69,1.19),(0.69,0.785),(0.19,0.785)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.01),(0.19,9.215),(0.69,9.215),(0.69,8.86),(1.17,8.86),(1.17,9.215),(1.67,9.215),(1.67,5.01),(0.19,5.01)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.81,0.19),(0.81,0.69),(1.81,0.69),(1.81,0.19),(0.81,0.19)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((0.935,9.435),(0.935,9.685),(1.685,9.685),(1.685,9.435),(0.935,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.07,5.135),(1.07,8.735),(1.295,8.735),(1.295,9.09),(1.545,9.09),(1.545,5.135),(1.07,5.135)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,5.135),(0.315,9.09),(0.565,9.09),(0.565,8.735),(0.79,8.735),(0.79,5.135),(0.315,5.135)),
        )
        createRL(
            tech, net, 'licon',
            ((1.335,8.8),(1.335,8.97),(1.505,8.97),(1.505,8.8),(1.335,8.8)),
        )
        createRL(
            tech, net, 'licon',
            ((1.055,9.475),(1.055,9.645),(1.225,9.645),(1.225,9.475),(1.055,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.395,9.475),(1.395,9.645),(1.565,9.645),(1.565,9.475),(1.395,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.8),(0.355,8.97),(0.525,8.97),(0.525,8.8),(0.355,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((0.79,1.315),(0.79,3.115),(1.07,3.115),(1.07,1.315),(0.79,1.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,5.135),(0.79,8.735),(1.07,8.735),(1.07,5.135),(0.79,5.135)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.45),(0.845,1.62),(1.015,1.62),(1.015,1.45),(0.845,1.45)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.79),(0.845,1.96),(1.015,1.96),(1.015,1.79),(0.845,1.79)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.13),(0.845,2.3),(1.015,2.3),(1.015,2.13),(0.845,2.13)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.47),(0.845,2.64),(1.015,2.64),(1.015,2.47),(0.845,2.47)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.81),(0.845,2.98),(1.015,2.98),(1.015,2.81),(0.845,2.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,5.49),(0.845,5.66),(1.015,5.66),(1.015,5.49),(0.845,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,5.83),(0.845,6.0),(1.015,6.0),(1.015,5.83),(0.845,5.83)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.17),(0.845,6.34),(1.015,6.34),(1.015,6.17),(0.845,6.17)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.51),(0.845,6.68),(1.015,6.68),(1.015,6.51),(0.845,6.51)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.85),(0.845,7.02),(1.015,7.02),(1.015,6.85),(0.845,6.85)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.19),(0.845,7.36),(1.015,7.36),(1.015,7.19),(0.845,7.19)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.53),(0.845,7.7),(1.015,7.7),(1.015,7.53),(0.845,7.53)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.87),(0.845,8.04),(1.015,8.04),(1.015,7.87),(0.845,7.87)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.21),(0.845,8.38),(1.015,8.38),(1.015,8.21),(0.845,8.21)),
        )
        createRL(
            tech, net, 'li',
            ((0.845,1.37),(0.845,8.63),(1.015,8.63),(1.015,1.37),(0.845,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)

    cell = new_cells['inv_x4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(3.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['i']
        createRL(
            tech, net, 'li',
            ((0.17,1.37),(0.17,8.63),(0.34,8.63),(0.34,4.965),(0.425,4.965),(0.425,4.635),(0.34,4.635),(0.34,1.37),(0.17,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.255), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        createRL(
            tech, net, 'poly',
            ((0.64,1.185),(0.64,4.635),(0.205,4.635),(0.205,4.965),(0.64,4.965),(0.64,8.865),(0.79,8.865),(0.79,4.965),(1.07,4.965),(1.07,8.865),(1.22,8.865),(1.22,4.965),(1.62,4.965),(1.62,8.865),(1.77,8.865),(1.77,4.965),(2.05,4.965),(2.05,8.865),(2.2,8.865),(2.2,1.185),(2.05,1.185),(2.05,4.635),(1.77,4.635),(1.77,1.185),(1.62,1.185),(1.62,4.635),(1.22,4.635),(1.22,1.185),(1.07,1.185),(1.07,4.635),(0.79,4.635),(0.79,1.185),(0.64,1.185)),
        )
        createRL(
            tech, net, 'licon',
            ((0.255,4.715),(0.255,4.885),(0.425,4.885),(0.425,4.715),(0.255,4.715)),
        )
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((1.295,0.91),(1.295,1.315),(1.07,1.315),(1.07,3.115),(1.77,3.115),(1.77,1.315),(1.545,1.315),(1.545,0.91),(1.295,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.91),(0.315,3.115),(0.79,3.115),(0.79,1.315),(0.565,1.315),(0.565,0.91),(0.315,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.935,0.315),(1.935,0.565),(2.685,0.565),(2.685,0.315),(1.935,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.275,0.91),(2.275,1.315),(2.05,1.315),(2.05,3.115),(2.525,3.115),(2.525,0.91),(2.275,0.91)),
        )
        createRL(
            tech, net, 'licon',
            ((2.315,1.03),(2.315,1.2),(2.485,1.2),(2.485,1.03),(2.315,1.03)),
        )
        createRL(
            tech, net, 'licon',
            ((2.055,0.355),(2.055,0.525),(2.225,0.525),(2.225,0.355),(2.055,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((2.395,0.355),(2.395,0.525),(2.565,0.525),(2.565,0.355),(2.395,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.335,1.03),(1.335,1.2),(1.505,1.2),(1.505,1.03),(1.335,1.03)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,1.03),(0.355,1.2),(0.525,1.2),(0.525,1.03),(0.355,1.03)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(3.0,1.2),(3.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((1.81,9.31),(1.81,9.81),(2.81,9.81),(2.81,9.31),(1.81,9.31)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.785),(0.19,3.24),(2.65,3.24),(2.65,0.785),(2.15,0.785),(2.15,1.19),(1.67,1.19),(1.67,0.785),(1.17,0.785),(1.17,1.19),(0.69,1.19),(0.69,0.785),(0.19,0.785)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.01),(0.19,9.215),(0.69,9.215),(0.69,8.86),(1.17,8.86),(1.17,9.215),(1.67,9.215),(1.67,8.86),(2.15,8.86),(2.15,9.215),(2.65,9.215),(2.65,5.01),(0.19,5.01)),
        )
        createRL(
            tech, net, 'psdm',
            ((1.81,0.19),(1.81,0.69),(2.81,0.69),(2.81,0.19),(1.81,0.19)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.07,5.135),(1.07,8.735),(1.295,8.735),(1.295,9.09),(1.545,9.09),(1.545,8.735),(1.77,8.735),(1.77,5.135),(1.07,5.135)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.935,9.435),(1.935,9.685),(2.685,9.685),(2.685,9.435),(1.935,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,5.135),(0.315,9.09),(0.565,9.09),(0.565,8.735),(0.79,8.735),(0.79,5.135),(0.315,5.135)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.05,5.135),(2.05,8.735),(2.275,8.735),(2.275,9.09),(2.525,9.09),(2.525,5.135),(2.05,5.135)),
        )
        createRL(
            tech, net, 'licon',
            ((1.335,8.8),(1.335,8.97),(1.505,8.97),(1.505,8.8),(1.335,8.8)),
        )
        createRL(
            tech, net, 'licon',
            ((2.055,9.475),(2.055,9.645),(2.225,9.645),(2.225,9.475),(2.055,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((2.395,9.475),(2.395,9.645),(2.565,9.645),(2.565,9.475),(2.395,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.8),(0.355,8.97),(0.525,8.97),(0.525,8.8),(0.355,8.8)),
        )
        createRL(
            tech, net, 'licon',
            ((2.315,8.8),(2.315,8.97),(2.485,8.97),(2.485,8.8),(2.315,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(3.0,10.0),(3.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(3.18,10.18),(3.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((0.79,1.315),(0.79,3.115),(1.07,3.115),(1.07,1.315),(0.79,1.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,5.135),(0.79,8.735),(1.07,8.735),(1.07,5.135),(0.79,5.135)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.77,5.135),(1.77,8.735),(2.05,8.735),(2.05,5.135),(1.77,5.135)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.77,1.315),(1.77,3.115),(2.05,3.115),(2.05,1.315),(1.77,1.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.45),(0.845,1.62),(1.015,1.62),(1.015,1.45),(0.845,1.45)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.79),(0.845,1.96),(1.015,1.96),(1.015,1.79),(0.845,1.79)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.13),(0.845,2.3),(1.015,2.3),(1.015,2.13),(0.845,2.13)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.47),(0.845,2.64),(1.015,2.64),(1.015,2.47),(0.845,2.47)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,2.81),(0.845,2.98),(1.015,2.98),(1.015,2.81),(0.845,2.81)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,5.49),(0.845,5.66),(1.015,5.66),(1.015,5.49),(0.845,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,5.83),(0.845,6.0),(1.015,6.0),(1.015,5.83),(0.845,5.83)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.17),(0.845,6.34),(1.015,6.34),(1.015,6.17),(0.845,6.17)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.51),(0.845,6.68),(1.015,6.68),(1.015,6.51),(0.845,6.51)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.85),(0.845,7.02),(1.015,7.02),(1.015,6.85),(0.845,6.85)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.19),(0.845,7.36),(1.015,7.36),(1.015,7.19),(0.845,7.19)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.53),(0.845,7.7),(1.015,7.7),(1.015,7.53),(0.845,7.53)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.87),(0.845,8.04),(1.015,8.04),(1.015,7.87),(0.845,7.87)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.21),(0.845,8.38),(1.015,8.38),(1.015,8.21),(0.845,8.21)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,5.49),(1.825,5.66),(1.995,5.66),(1.995,5.49),(1.825,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,5.83),(1.825,6.0),(1.995,6.0),(1.995,5.83),(1.825,5.83)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,6.17),(1.825,6.34),(1.995,6.34),(1.995,6.17),(1.825,6.17)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,6.51),(1.825,6.68),(1.995,6.68),(1.995,6.51),(1.825,6.51)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,6.85),(1.825,7.02),(1.995,7.02),(1.995,6.85),(1.825,6.85)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,7.19),(1.825,7.36),(1.995,7.36),(1.995,7.19),(1.825,7.19)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,7.53),(1.825,7.7),(1.995,7.7),(1.995,7.53),(1.825,7.53)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,7.87),(1.825,8.04),(1.995,8.04),(1.995,7.87),(1.825,7.87)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,8.21),(1.825,8.38),(1.995,8.38),(1.995,8.21),(1.825,8.21)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,1.45),(1.825,1.62),(1.995,1.62),(1.995,1.45),(1.825,1.45)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,1.79),(1.825,1.96),(1.995,1.96),(1.995,1.79),(1.825,1.79)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,2.13),(1.825,2.3),(1.995,2.3),(1.995,2.13),(1.825,2.13)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,2.47),(1.825,2.64),(1.995,2.64),(1.995,2.47),(1.825,2.47)),
        )
        createRL(
            tech, net, 'licon',
            ((1.825,2.81),(1.825,2.98),(1.995,2.98),(1.995,2.81),(1.825,2.81)),
        )
        createRL(
            tech, net, 'li',
            ((0.845,1.37),(0.845,8.63),(1.015,8.63),(1.015,4.885),(1.825,4.885),(1.825,8.63),(1.995,8.63),(1.995,1.37),(1.825,1.37),(1.825,4.715),(1.015,4.715),(1.015,1.37),(0.845,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.93), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)

    cell = new_cells['buf_x1']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'ni': Net.create(cell, 'ni'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['ni']
        createRL(
            tech, net, 'difftap',
            ((0.315,2.21),(0.315,2.71),(0.58,2.71),(0.58,2.21),(0.315,2.21)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,5.49),(0.315,6.49),(0.58,6.49),(0.58,5.49),(0.315,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,4.015),(0.355,4.185),(0.525,4.185),(0.525,4.015),(0.355,4.015)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,5.735),(0.355,5.905),(0.525,5.905),(0.525,5.735),(0.355,5.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,6.075),(0.355,6.245),(0.525,6.245),(0.525,6.075),(0.355,6.075)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,2.375),(0.355,2.545),(0.525,2.545),(0.525,2.375),(0.355,2.375)),
        )
        createRL(
            tech, net, 'li',
            ((0.355,2.21),(0.355,6.49),(0.525,6.49),(0.525,2.21),(0.355,2.21)),
        )
        createRL(
            tech, net, 'poly',
            ((1.155,0.78),(1.155,3.935),(0.305,3.935),(0.305,4.265),(1.155,4.265),(1.155,9.22),(1.305,9.22),(1.305,0.78),(1.155,0.78)),
        )
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.68,0.785),(0.68,2.085),(0.19,2.085),(0.19,2.835),(1.695,2.835),(1.695,0.785),(0.68,0.785)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,9.31),(0.19,9.81),(1.81,9.81),(1.81,9.31),(0.19,9.31)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,0.19),(0.19,0.69),(1.81,0.69),(1.81,0.19),(0.19,0.19)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.365),(0.19,6.615),(0.68,6.615),(0.68,9.215),(1.695,9.215),(1.695,5.365),(0.19,5.365)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,9.435),(0.315,9.685),(1.685,9.685),(1.685,9.435),(0.315,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.58,5.49),(0.58,6.49),(0.805,6.49),(0.805,9.09),(1.305,9.09),(1.305,5.49),(0.58,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((0.575,9.475),(0.575,9.645),(0.745,9.645),(0.745,9.475),(0.575,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,9.475),(0.915,9.645),(1.085,9.645),(1.085,9.475),(0.915,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.255,9.475),(1.255,9.645),(1.425,9.645),(1.425,9.475),(1.255,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.8),(0.845,8.97),(1.015,8.97),(1.015,8.8),(0.845,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.805,0.91),(0.805,2.21),(0.58,2.21),(0.58,2.71),(1.305,2.71),(1.305,0.91),(0.805,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.315),(0.315,0.565),(1.685,0.565),(1.685,0.315),(0.315,0.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.03),(0.845,1.2),(1.015,1.2),(1.015,1.03),(0.845,1.03)),
        )
        createRL(
            tech, net, 'licon',
            ((0.575,0.355),(0.575,0.525),(0.745,0.525),(0.745,0.355),(0.575,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.915,0.355),(0.915,0.525),(1.085,0.525),(1.085,0.355),(0.915,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.255,0.355),(1.255,0.525),(1.425,0.525),(1.425,0.355),(1.255,0.355)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i']
        createRL(
            tech, net, 'poly',
            ((0.58,2.08),(0.58,3.215),(0.945,3.215),(0.945,2.945),(0.73,2.945),(0.73,2.08),(0.58,2.08)),
        )
        createRL(
            tech, net, 'poly',
            ((0.58,4.985),(0.58,6.62),(0.73,6.62),(0.73,5.255),(0.945,5.255),(0.945,4.985),(0.58,4.985)),
        )
        createRL(
            tech, net, 'licon',
            ((0.695,2.995),(0.695,3.165),(0.865,3.165),(0.865,2.995),(0.695,2.995)),
        )
        createRL(
            tech, net, 'licon',
            ((0.695,5.035),(0.695,5.205),(0.865,5.205),(0.865,5.035),(0.695,5.035)),
        )
        createRL(
            tech, net, 'li',
            ((0.695,1.37),(0.695,8.63),(0.865,8.63),(0.865,1.37),(0.695,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.78), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.78), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((1.305,0.91),(1.305,2.71),(1.57,2.71),(1.57,0.91),(1.305,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.305,5.49),(1.305,9.09),(1.57,9.09),(1.57,5.49),(1.305,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,5.615),(1.36,5.785),(1.53,5.785),(1.53,5.615),(1.36,5.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,5.955),(1.36,6.125),(1.53,6.125),(1.53,5.955),(1.36,5.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.295),(1.36,6.465),(1.53,6.465),(1.53,6.295),(1.36,6.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.635),(1.36,6.805),(1.53,6.805),(1.53,6.635),(1.36,6.635)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.975),(1.36,7.145),(1.53,7.145),(1.53,6.975),(1.36,6.975)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.315),(1.36,7.485),(1.53,7.485),(1.53,7.315),(1.36,7.315)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.655),(1.36,7.825),(1.53,7.825),(1.53,7.655),(1.36,7.655)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.995),(1.36,8.165),(1.53,8.165),(1.53,7.995),(1.36,7.995)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,8.335),(1.36,8.505),(1.53,8.505),(1.53,8.335),(1.36,8.335)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,1.615),(1.36,1.785),(1.53,1.785),(1.53,1.615),(1.36,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,1.955),(1.36,2.125),(1.53,2.125),(1.53,1.955),(1.36,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,2.295),(1.36,2.465),(1.53,2.465),(1.53,2.295),(1.36,2.295)),
        )
        createRL(
            tech, net, 'li',
            ((1.36,1.37),(1.36,8.63),(1.53,8.63),(1.53,1.37),(1.36,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.445), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.445), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)

    cell = new_cells['buf_x2']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(3.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'ni': Net.create(cell, 'ni'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['ni']
        createRL(
            tech, net, 'difftap',
            ((0.315,1.71),(0.315,2.71),(0.58,2.71),(0.58,1.71),(0.315,1.71)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,5.49),(0.315,7.49),(0.58,7.49),(0.58,5.49),(0.315,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,5.725),(0.355,5.895),(0.525,5.895),(0.525,5.725),(0.355,5.725)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,6.065),(0.355,6.235),(0.525,6.235),(0.525,6.065),(0.355,6.065)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,6.405),(0.355,6.575),(0.525,6.575),(0.525,6.405),(0.355,6.405)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,6.745),(0.355,6.915),(0.525,6.915),(0.525,6.745),(0.355,6.745)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,7.085),(0.355,7.255),(0.525,7.255),(0.525,7.085),(0.355,7.085)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,4.015),(0.355,4.185),(0.525,4.185),(0.525,4.015),(0.355,4.015)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,1.955),(0.355,2.125),(0.525,2.125),(0.525,1.955),(0.355,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,2.295),(0.355,2.465),(0.525,2.465),(0.525,2.295),(0.355,2.295)),
        )
        createRL(
            tech, net, 'li',
            ((0.355,1.71),(0.355,7.49),(0.525,7.49),(0.525,1.71),(0.355,1.71)),
        )
        createRL(
            tech, net, 'poly',
            ((1.155,0.78),(1.155,3.935),(0.305,3.935),(0.305,4.265),(1.155,4.265),(1.155,9.22),(1.305,9.22),(1.305,4.265),(1.585,4.265),(1.585,9.22),(1.735,9.22),(1.735,0.78),(1.585,0.78),(1.585,3.935),(1.305,3.935),(1.305,0.78),(1.155,0.78)),
        )
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.68,0.785),(0.68,1.585),(0.19,1.585),(0.19,2.835),(2.125,2.835),(2.125,0.785),(0.68,0.785)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.19,9.31),(0.19,9.81),(2.81,9.81),(2.81,9.31),(0.19,9.31)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,0.19),(0.19,0.69),(2.81,0.69),(2.81,0.19),(0.19,0.19)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.365),(0.19,7.615),(0.68,7.615),(0.68,9.215),(2.125,9.215),(2.125,5.365),(0.19,5.365)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(3.18,10.18),(3.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.58,5.49),(0.58,7.49),(0.805,7.49),(0.805,9.09),(1.305,9.09),(1.305,5.49),(0.58,5.49)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.585,5.49),(1.585,9.09),(2.0,9.09),(2.0,5.49),(1.585,5.49)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,9.435),(0.315,9.685),(2.685,9.685),(2.685,9.435),(0.315,9.435)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,5.675),(1.79,5.845),(1.96,5.845),(1.96,5.675),(1.79,5.675)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,6.015),(1.79,6.185),(1.96,6.185),(1.96,6.015),(1.79,6.015)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,6.355),(1.79,6.525),(1.96,6.525),(1.96,6.355),(1.79,6.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,6.695),(1.79,6.865),(1.96,6.865),(1.96,6.695),(1.79,6.695)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,7.035),(1.79,7.205),(1.96,7.205),(1.96,7.035),(1.79,7.035)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,7.375),(1.79,7.545),(1.96,7.545),(1.96,7.375),(1.79,7.375)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,7.715),(1.79,7.885),(1.96,7.885),(1.96,7.715),(1.79,7.715)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,8.055),(1.79,8.225),(1.96,8.225),(1.96,8.055),(1.79,8.055)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,8.395),(1.79,8.565),(1.96,8.565),(1.96,8.395),(1.79,8.395)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,8.735),(1.79,8.905),(1.96,8.905),(1.96,8.735),(1.79,8.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.565,9.475),(0.565,9.645),(0.735,9.645),(0.735,9.475),(0.565,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.905,9.475),(0.905,9.645),(1.075,9.645),(1.075,9.475),(0.905,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.245,9.475),(1.245,9.645),(1.415,9.645),(1.415,9.475),(1.245,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.585,9.475),(1.585,9.645),(1.755,9.645),(1.755,9.475),(1.585,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.925,9.475),(1.925,9.645),(2.095,9.645),(2.095,9.475),(1.925,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((2.265,9.475),(2.265,9.645),(2.435,9.645),(2.435,9.475),(2.265,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.8),(0.845,8.97),(1.015,8.97),(1.015,8.8),(0.845,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((1.79,5.49),(1.79,8.8),(0.0,8.8),(0.0,10.0),(3.0,10.0),(3.0,8.8),(1.96,8.8),(1.96,5.49),(1.79,5.49)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.315,0.315),(0.315,0.565),(2.685,0.565),(2.685,0.315),(0.315,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.585,0.91),(1.585,2.71),(2.0,2.71),(2.0,0.91),(1.585,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.805,0.91),(0.805,1.71),(0.58,1.71),(0.58,2.71),(1.305,2.71),(1.305,0.91),(0.805,0.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.03),(0.845,1.2),(1.015,1.2),(1.015,1.03),(0.845,1.03)),
        )
        createRL(
            tech, net, 'licon',
            ((0.565,0.355),(0.565,0.525),(0.735,0.525),(0.735,0.355),(0.565,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.905,0.355),(0.905,0.525),(1.075,0.525),(1.075,0.355),(0.905,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.245,0.355),(1.245,0.525),(1.415,0.525),(1.415,0.355),(1.245,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.585,0.355),(1.585,0.525),(1.755,0.525),(1.755,0.355),(1.585,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.925,0.355),(1.925,0.525),(2.095,0.525),(2.095,0.355),(1.925,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((2.265,0.355),(2.265,0.525),(2.435,0.525),(2.435,0.355),(2.265,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,1.045),(1.79,1.215),(1.96,1.215),(1.96,1.045),(1.79,1.045)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,1.385),(1.79,1.555),(1.96,1.555),(1.96,1.385),(1.79,1.385)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,1.725),(1.79,1.895),(1.96,1.895),(1.96,1.725),(1.79,1.725)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,2.065),(1.79,2.235),(1.96,2.235),(1.96,2.065),(1.79,2.065)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,2.405),(1.79,2.575),(1.96,2.575),(1.96,2.405),(1.79,2.405)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(1.79,1.2),(1.79,2.71),(1.96,2.71),(1.96,1.2),(3.0,1.2),(3.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i']
        createRL(
            tech, net, 'poly',
            ((0.58,4.985),(0.58,7.62),(0.73,7.62),(0.73,5.255),(0.945,5.255),(0.945,4.985),(0.58,4.985)),
        )
        createRL(
            tech, net, 'poly',
            ((0.58,1.58),(0.58,3.215),(0.945,3.215),(0.945,2.945),(0.73,2.945),(0.73,1.58),(0.58,1.58)),
        )
        createRL(
            tech, net, 'licon',
            ((0.695,2.995),(0.695,3.165),(0.865,3.165),(0.865,2.995),(0.695,2.995)),
        )
        createRL(
            tech, net, 'licon',
            ((0.695,5.035),(0.695,5.205),(0.865,5.205),(0.865,5.035),(0.695,5.035)),
        )
        createRL(
            tech, net, 'li',
            ((0.695,1.37),(0.695,8.63),(0.865,8.63),(0.865,1.37),(0.695,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.78), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.78), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((1.305,0.91),(1.305,2.71),(1.585,2.71),(1.585,0.91),(1.305,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.305,5.49),(1.305,9.09),(1.585,9.09),(1.585,5.49),(1.305,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,5.615),(1.36,5.785),(1.53,5.785),(1.53,5.615),(1.36,5.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,5.955),(1.36,6.125),(1.53,6.125),(1.53,5.955),(1.36,5.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.295),(1.36,6.465),(1.53,6.465),(1.53,6.295),(1.36,6.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.635),(1.36,6.805),(1.53,6.805),(1.53,6.635),(1.36,6.635)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.975),(1.36,7.145),(1.53,7.145),(1.53,6.975),(1.36,6.975)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.315),(1.36,7.485),(1.53,7.485),(1.53,7.315),(1.36,7.315)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.655),(1.36,7.825),(1.53,7.825),(1.53,7.655),(1.36,7.655)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.995),(1.36,8.165),(1.53,8.165),(1.53,7.995),(1.36,7.995)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,8.335),(1.36,8.505),(1.53,8.505),(1.53,8.335),(1.36,8.335)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,1.615),(1.36,1.785),(1.53,1.785),(1.53,1.615),(1.36,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,1.955),(1.36,2.125),(1.53,2.125),(1.53,1.955),(1.36,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,2.295),(1.36,2.465),(1.53,2.465),(1.53,2.295),(1.36,2.295)),
        )
        createRL(
            tech, net, 'li',
            ((1.36,1.37),(1.36,8.63),(1.53,8.63),(1.53,1.37),(1.36,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.445), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.445), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)

    cell = new_cells['buf_x4']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i': Net.create(cell, 'i'),
            'ni': Net.create(cell, 'ni'),
            'q': Net.create(cell, 'q'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['ni']
        createRL(
            tech, net, 'difftap',
            ((0.315,5.49),(0.315,8.63),(0.33,8.63),(0.33,9.09),(0.58,9.09),(0.58,5.49),(0.315,5.49)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.33,0.91),(0.33,1.37),(0.315,1.37),(0.315,2.71),(0.58,2.71),(0.58,0.91),(0.33,0.91)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,4.015),(0.355,4.185),(0.525,4.185),(0.525,4.015),(0.355,4.015)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,5.615),(0.355,5.785),(0.525,5.785),(0.525,5.615),(0.355,5.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,5.955),(0.355,6.125),(0.525,6.125),(0.525,5.955),(0.355,5.955)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,6.295),(0.355,6.465),(0.525,6.465),(0.525,6.295),(0.355,6.295)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,6.635),(0.355,6.805),(0.525,6.805),(0.525,6.635),(0.355,6.635)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,6.975),(0.355,7.145),(0.525,7.145),(0.525,6.975),(0.355,6.975)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,7.315),(0.355,7.485),(0.525,7.485),(0.525,7.315),(0.355,7.315)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,7.655),(0.355,7.825),(0.525,7.825),(0.525,7.655),(0.355,7.655)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,7.995),(0.355,8.165),(0.525,8.165),(0.525,7.995),(0.355,7.995)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.335),(0.355,8.505),(0.525,8.505),(0.525,8.335),(0.355,8.335)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,1.615),(0.355,1.785),(0.525,1.785),(0.525,1.615),(0.355,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,1.955),(0.355,2.125),(0.525,2.125),(0.525,1.955),(0.355,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,2.295),(0.355,2.465),(0.525,2.465),(0.525,2.295),(0.355,2.295)),
        )
        createRL(
            tech, net, 'li',
            ((0.355,1.37),(0.355,8.63),(0.525,8.63),(0.525,1.37),(0.355,1.37)),
        )
        createRL(
            tech, net, 'poly',
            ((1.155,0.78),(1.155,3.935),(0.305,3.935),(0.305,4.265),(1.155,4.265),(1.155,9.22),(1.305,9.22),(1.305,4.265),(1.585,4.265),(1.585,9.22),(1.735,9.22),(1.735,4.265),(2.015,4.265),(2.015,9.22),(2.165,9.22),(2.165,4.265),(2.445,4.265),(2.445,9.22),(2.595,9.22),(2.595,0.78),(2.445,0.78),(2.445,3.935),(2.165,3.935),(2.165,0.78),(2.015,0.78),(2.015,3.935),(1.735,3.935),(1.735,0.78),(1.585,0.78),(1.585,3.935),(1.305,3.935),(1.305,0.78),(1.155,0.78)),
        )
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,9.31),(0.19,9.81),(3.81,9.81),(3.81,9.31),(0.19,9.31)),
        )
        createRL(
            tech, net, 'nsdm',
            ((0.205,0.785),(0.205,1.245),(0.19,1.245),(0.19,2.835),(2.985,2.835),(2.985,0.785),(0.205,0.785)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.365),(0.19,8.755),(0.205,8.755),(0.205,9.215),(2.985,9.215),(2.985,5.365),(0.19,5.365)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,0.19),(0.19,0.69),(3.81,0.69),(3.81,0.19),(0.19,0.19)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(4.18,10.18),(4.18,4.8),(-0.18,4.8)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,9.435),(0.315,9.685),(3.685,9.685),(3.685,9.435),(0.315,9.435)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.58,5.49),(0.58,9.09),(1.305,9.09),(1.305,5.49),(0.58,5.49)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.585,5.49),(1.585,9.09),(2.165,9.09),(2.165,5.49),(1.585,5.49)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.445,5.49),(2.445,9.09),(2.86,9.09),(2.86,5.49),(2.445,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,5.675),(1.79,5.845),(1.96,5.845),(1.96,5.675),(1.79,5.675)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,6.015),(1.79,6.185),(1.96,6.185),(1.96,6.015),(1.79,6.015)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,6.355),(1.79,6.525),(1.96,6.525),(1.96,6.355),(1.79,6.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,6.695),(1.79,6.865),(1.96,6.865),(1.96,6.695),(1.79,6.695)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,7.035),(1.79,7.205),(1.96,7.205),(1.96,7.035),(1.79,7.035)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,7.375),(1.79,7.545),(1.96,7.545),(1.96,7.375),(1.79,7.375)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,7.715),(1.79,7.885),(1.96,7.885),(1.96,7.715),(1.79,7.715)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,8.055),(1.79,8.225),(1.96,8.225),(1.96,8.055),(1.79,8.055)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,8.395),(1.79,8.565),(1.96,8.565),(1.96,8.395),(1.79,8.395)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,8.735),(1.79,8.905),(1.96,8.905),(1.96,8.735),(1.79,8.735)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,5.675),(2.65,5.845),(2.82,5.845),(2.82,5.675),(2.65,5.675)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,6.015),(2.65,6.185),(2.82,6.185),(2.82,6.015),(2.65,6.015)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,6.355),(2.65,6.525),(2.82,6.525),(2.82,6.355),(2.65,6.355)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,6.695),(2.65,6.865),(2.82,6.865),(2.82,6.695),(2.65,6.695)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,7.035),(2.65,7.205),(2.82,7.205),(2.82,7.035),(2.65,7.035)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,7.375),(2.65,7.545),(2.82,7.545),(2.82,7.375),(2.65,7.375)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,7.715),(2.65,7.885),(2.82,7.885),(2.82,7.715),(2.65,7.715)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,8.055),(2.65,8.225),(2.82,8.225),(2.82,8.055),(2.65,8.055)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,8.395),(2.65,8.565),(2.82,8.565),(2.82,8.395),(2.65,8.395)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,8.735),(2.65,8.905),(2.82,8.905),(2.82,8.735),(2.65,8.735)),
        )
        createRL(
            tech, net, 'licon',
            ((0.555,9.475),(0.555,9.645),(0.725,9.645),(0.725,9.475),(0.555,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.895,9.475),(0.895,9.645),(1.065,9.645),(1.065,9.475),(0.895,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.235,9.475),(1.235,9.645),(1.405,9.645),(1.405,9.475),(1.235,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.575,9.475),(1.575,9.645),(1.745,9.645),(1.745,9.475),(1.575,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,9.475),(1.915,9.645),(2.085,9.645),(2.085,9.475),(1.915,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((2.255,9.475),(2.255,9.645),(2.425,9.645),(2.425,9.475),(2.255,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((2.595,9.475),(2.595,9.645),(2.765,9.645),(2.765,9.475),(2.595,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((2.935,9.475),(2.935,9.645),(3.105,9.645),(3.105,9.475),(2.935,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((3.275,9.475),(3.275,9.645),(3.445,9.645),(3.445,9.475),(3.275,9.475)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.8),(0.845,8.97),(1.015,8.97),(1.015,8.8),(0.845,8.8)),
        )
        createRL(
            tech, net, 'li',
            ((1.79,5.49),(1.79,8.8),(0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(2.82,8.8),(2.82,5.49),(2.65,5.49),(2.65,8.8),(1.96,8.8),(1.96,5.49),(1.79,5.49)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((2.445,0.91),(2.445,2.71),(2.86,2.71),(2.86,0.91),(2.445,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.585,0.91),(1.585,2.71),(2.165,2.71),(2.165,0.91),(1.585,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.315),(0.315,0.565),(3.685,0.565),(3.685,0.315),(0.315,0.315)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.58,0.91),(0.58,2.71),(1.305,2.71),(1.305,0.91),(0.58,0.91)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,1.045),(2.65,1.215),(2.82,1.215),(2.82,1.045),(2.65,1.045)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,1.385),(2.65,1.555),(2.82,1.555),(2.82,1.385),(2.65,1.385)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,1.725),(2.65,1.895),(2.82,1.895),(2.82,1.725),(2.65,1.725)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,2.065),(2.65,2.235),(2.82,2.235),(2.82,2.065),(2.65,2.065)),
        )
        createRL(
            tech, net, 'licon',
            ((2.65,2.405),(2.65,2.575),(2.82,2.575),(2.82,2.405),(2.65,2.405)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.03),(0.845,1.2),(1.015,1.2),(1.015,1.03),(0.845,1.03)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,1.045),(1.79,1.215),(1.96,1.215),(1.96,1.045),(1.79,1.045)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,1.385),(1.79,1.555),(1.96,1.555),(1.96,1.385),(1.79,1.385)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,1.725),(1.79,1.895),(1.96,1.895),(1.96,1.725),(1.79,1.725)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,2.065),(1.79,2.235),(1.96,2.235),(1.96,2.065),(1.79,2.065)),
        )
        createRL(
            tech, net, 'licon',
            ((1.79,2.405),(1.79,2.575),(1.96,2.575),(1.96,2.405),(1.79,2.405)),
        )
        createRL(
            tech, net, 'licon',
            ((0.555,0.355),(0.555,0.525),(0.725,0.525),(0.725,0.355),(0.555,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((0.895,0.355),(0.895,0.525),(1.065,0.525),(1.065,0.355),(0.895,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.235,0.355),(1.235,0.525),(1.405,0.525),(1.405,0.355),(1.235,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.575,0.355),(1.575,0.525),(1.745,0.525),(1.745,0.355),(1.575,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((1.915,0.355),(1.915,0.525),(2.085,0.525),(2.085,0.355),(1.915,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((2.255,0.355),(2.255,0.525),(2.425,0.525),(2.425,0.355),(2.255,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((2.595,0.355),(2.595,0.525),(2.765,0.525),(2.765,0.355),(2.595,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((2.935,0.355),(2.935,0.525),(3.105,0.525),(3.105,0.355),(2.935,0.355)),
        )
        createRL(
            tech, net, 'licon',
            ((3.275,0.355),(3.275,0.525),(3.445,0.525),(3.445,0.355),(3.275,0.355)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(1.79,1.2),(1.79,2.71),(1.96,2.71),(1.96,1.2),(2.65,1.2),(2.65,2.71),(2.82,2.71),(2.82,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i']
        createRL(
            tech, net, 'poly',
            ((0.58,4.985),(0.58,9.22),(0.73,9.22),(0.73,5.255),(0.945,5.255),(0.945,4.985),(0.58,4.985)),
        )
        createRL(
            tech, net, 'poly',
            ((0.58,0.78),(0.58,3.215),(0.945,3.215),(0.945,2.945),(0.73,2.945),(0.73,0.78),(0.58,0.78)),
        )
        createRL(
            tech, net, 'licon',
            ((0.695,2.995),(0.695,3.165),(0.865,3.165),(0.865,2.995),(0.695,2.995)),
        )
        createRL(
            tech, net, 'licon',
            ((0.695,5.035),(0.695,5.205),(0.865,5.205),(0.865,5.035),(0.695,5.035)),
        )
        createRL(
            tech, net, 'li',
            ((0.695,1.37),(0.695,8.63),(0.865,8.63),(0.865,1.37),(0.695,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.78), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.78), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['q']
        createRL(
            tech, net, 'difftap',
            ((2.165,0.91),(2.165,2.71),(2.445,2.71),(2.445,0.91),(2.165,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.305,0.91),(1.305,2.71),(1.585,2.71),(1.585,0.91),(1.305,0.91)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.305,5.49),(1.305,9.09),(1.585,9.09),(1.585,5.49),(1.305,5.49)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.165,5.49),(2.165,9.09),(2.445,9.09),(2.445,5.49),(2.165,5.49)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,5.615),(2.22,5.785),(2.39,5.785),(2.39,5.615),(2.22,5.615)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,5.955),(2.22,6.125),(2.39,6.125),(2.39,5.955),(2.22,5.955)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,6.295),(2.22,6.465),(2.39,6.465),(2.39,6.295),(2.22,6.295)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,6.635),(2.22,6.805),(2.39,6.805),(2.39,6.635),(2.22,6.635)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,6.975),(2.22,7.145),(2.39,7.145),(2.39,6.975),(2.22,6.975)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,7.315),(2.22,7.485),(2.39,7.485),(2.39,7.315),(2.22,7.315)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,7.655),(2.22,7.825),(2.39,7.825),(2.39,7.655),(2.22,7.655)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,7.995),(2.22,8.165),(2.39,8.165),(2.39,7.995),(2.22,7.995)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,8.335),(2.22,8.505),(2.39,8.505),(2.39,8.335),(2.22,8.335)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,5.615),(1.36,5.785),(1.53,5.785),(1.53,5.615),(1.36,5.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,5.955),(1.36,6.125),(1.53,6.125),(1.53,5.955),(1.36,5.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.295),(1.36,6.465),(1.53,6.465),(1.53,6.295),(1.36,6.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.635),(1.36,6.805),(1.53,6.805),(1.53,6.635),(1.36,6.635)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,6.975),(1.36,7.145),(1.53,7.145),(1.53,6.975),(1.36,6.975)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.315),(1.36,7.485),(1.53,7.485),(1.53,7.315),(1.36,7.315)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.655),(1.36,7.825),(1.53,7.825),(1.53,7.655),(1.36,7.655)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,7.995),(1.36,8.165),(1.53,8.165),(1.53,7.995),(1.36,7.995)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,8.335),(1.36,8.505),(1.53,8.505),(1.53,8.335),(1.36,8.335)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,1.615),(1.36,1.785),(1.53,1.785),(1.53,1.615),(1.36,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,1.955),(1.36,2.125),(1.53,2.125),(1.53,1.955),(1.36,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.36,2.295),(1.36,2.465),(1.53,2.465),(1.53,2.295),(1.36,2.295)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,1.615),(2.22,1.785),(2.39,1.785),(2.39,1.615),(2.22,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,1.955),(2.22,2.125),(2.39,2.125),(2.39,1.955),(2.22,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((2.22,2.295),(2.22,2.465),(2.39,2.465),(2.39,2.295),(2.22,2.295)),
        )
        createRL(
            tech, net, 'li',
            ((1.36,1.37),(1.36,8.63),(1.53,8.63),(1.53,5.32),(2.22,5.32),(2.22,8.63),(2.39,8.63),(2.39,1.37),(2.22,1.37),(2.22,2.88),(1.53,2.88),(1.53,1.37),(1.36,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.445), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.445), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['i'].setDirection(Net.Direction.IN)
        nets['q'].setDirection(Net.Direction.OUT)

    cell = new_cells['nand2_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'int0': Net.create(cell, 'int0'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((1.195,1.37),(1.195,3.37),(1.345,3.37),(1.345,1.37),(1.195,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.83),(0.315,3.37),(0.79,3.37),(0.79,1.37),(0.565,1.37),(0.565,0.83),(0.315,0.83)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,0.95),(0.355,1.12),(0.525,1.12),(0.525,0.95),(0.355,0.95)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.705),(0.19,3.495),(1.735,3.495),(1.735,1.245),(0.69,1.245),(0.69,0.705),(0.19,0.705)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,6.505),(0.19,9.295),(0.69,9.295),(0.69,8.755),(1.295,8.755),(1.295,9.295),(1.795,9.295),(1.795,6.505),(0.19,6.505)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((0.315,6.63),(0.315,9.17),(0.565,9.17),(0.565,8.63),(0.79,8.63),(0.79,6.63),(0.315,6.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.195,6.63),(1.195,8.63),(1.42,8.63),(1.42,9.17),(1.67,9.17),(1.67,6.63),(1.195,6.63)),
        )
        createRL(
            tech, net, 'licon',
            ((1.46,8.88),(1.46,9.05),(1.63,9.05),(1.63,8.88),(1.46,8.88)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.88),(0.355,9.05),(0.525,9.05),(0.525,8.88),(0.355,8.88)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int0']
        createRL(
            tech, net, 'difftap',
            ((0.79,1.37),(0.79,3.37),(1.195,3.37),(1.195,1.37),(0.79,1.37)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.64,1.24),(0.64,4.635),(0.445,4.635),(0.445,4.965),(0.64,4.965),(0.64,8.76),(0.79,8.76),(0.79,1.24),(0.64,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((0.495,4.715),(0.495,4.885),(0.665,4.885),(0.665,4.715),(0.495,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((0.495,1.37),(0.495,8.63),(0.665,8.63),(0.665,1.37),(0.495,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((1.345,1.37),(1.345,3.37),(1.61,3.37),(1.61,1.37),(1.345,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,6.63),(0.79,8.63),(1.195,8.63),(1.195,6.63),(0.79,6.63)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,1.605),(1.4,1.775),(1.57,1.775),(1.57,1.605),(1.4,1.605)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,1.945),(1.4,2.115),(1.57,2.115),(1.57,1.945),(1.4,1.945)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,2.285),(1.4,2.455),(1.57,2.455),(1.57,2.285),(1.4,2.285)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,2.625),(1.4,2.795),(1.57,2.795),(1.57,2.625),(1.4,2.625)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,2.965),(1.4,3.135),(1.57,3.135),(1.57,2.965),(1.4,2.965)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.865),(0.845,7.035),(1.015,7.035),(1.015,6.865),(0.845,6.865)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.205),(0.845,7.375),(1.015,7.375),(1.015,7.205),(0.845,7.205)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.545),(0.845,7.715),(1.015,7.715),(1.015,7.545),(0.845,7.545)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.885),(0.845,8.055),(1.015,8.055),(1.015,7.885),(0.845,7.885)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.225),(0.845,8.395),(1.015,8.395),(1.015,8.225),(0.845,8.225)),
        )
        createRL(
            tech, net, 'li',
            ((1.4,1.37),(1.4,6.63),(0.845,6.63),(0.845,8.63),(1.57,8.63),(1.57,1.37),(1.4,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.485), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.485), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.195,1.24),(1.195,4.635),(1.0,4.635),(1.0,4.965),(1.195,4.965),(1.195,8.76),(1.345,8.76),(1.345,1.24),(1.195,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.05,4.715),(1.05,4.885),(1.22,4.885),(1.22,4.715),(1.05,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.05,1.37),(1.05,6.46),(1.22,6.46),(1.22,1.37),(1.05,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.135), u(0.17), u(1.37), u(6.46),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.135), u(0.17), u(1.37), u(6.46),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)

    cell = new_cells['nand3_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(3.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'int0': Net.create(cell, 'int0'),
            'int1': Net.create(cell, 'int1'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((1.195,1.37),(1.195,3.87),(1.345,3.87),(1.345,1.37),(1.195,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.83),(0.315,3.87),(0.79,3.87),(0.79,1.37),(0.565,1.37),(0.565,0.83),(0.315,0.83)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.75,1.37),(1.75,3.87),(1.9,3.87),(1.9,1.37),(1.75,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,0.95),(0.355,1.12),(0.525,1.12),(0.525,0.95),(0.355,0.95)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(3.0,1.2),(3.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.705),(0.19,3.995),(2.29,3.995),(2.29,1.245),(0.69,1.245),(0.69,0.705),(0.19,0.705)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,6.005),(0.19,9.295),(0.69,9.295),(0.69,8.755),(1.295,8.755),(1.295,9.295),(1.795,9.295),(1.795,8.755),(2.29,8.755),(2.29,6.005),(0.19,6.005)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.195,6.13),(1.195,8.63),(1.42,8.63),(1.42,9.17),(1.67,9.17),(1.67,8.63),(1.9,8.63),(1.9,6.13),(1.195,6.13)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,6.13),(0.315,9.17),(0.565,9.17),(0.565,8.63),(0.79,8.63),(0.79,6.13),(0.315,6.13)),
        )
        createRL(
            tech, net, 'licon',
            ((1.46,8.88),(1.46,9.05),(1.63,9.05),(1.63,8.88),(1.46,8.88)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.88),(0.355,9.05),(0.525,9.05),(0.525,8.88),(0.355,8.88)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(3.0,10.0),(3.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(3.18,10.18),(3.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int0']
        createRL(
            tech, net, 'difftap',
            ((0.79,1.37),(0.79,3.87),(1.195,3.87),(1.195,1.37),(0.79,1.37)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.64,1.24),(0.64,4.635),(0.445,4.635),(0.445,4.965),(0.64,4.965),(0.64,8.76),(0.79,8.76),(0.79,1.24),(0.64,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((0.495,4.715),(0.495,4.885),(0.665,4.885),(0.665,4.715),(0.495,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((0.495,1.37),(0.495,8.63),(0.665,8.63),(0.665,1.37),(0.495,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((0.79,6.13),(0.79,8.63),(1.195,8.63),(1.195,6.13),(0.79,6.13)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.9,6.13),(1.9,8.63),(2.165,8.63),(2.165,6.13),(1.9,6.13)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.9,1.37),(1.9,3.87),(2.165,3.87),(2.165,1.37),(1.9,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.275),(0.845,6.445),(1.015,6.445),(1.015,6.275),(0.845,6.275)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.615),(0.845,6.785),(1.015,6.785),(1.015,6.615),(0.845,6.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.955),(0.845,7.125),(1.015,7.125),(1.015,6.955),(0.845,6.955)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.295),(0.845,7.465),(1.015,7.465),(1.015,7.295),(0.845,7.295)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.635),(0.845,7.805),(1.015,7.805),(1.015,7.635),(0.845,7.635)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.975),(0.845,8.145),(1.015,8.145),(1.015,7.975),(0.845,7.975)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.315),(0.845,8.485),(1.015,8.485),(1.015,8.315),(0.845,8.315)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,1.515),(1.955,1.685),(2.125,1.685),(2.125,1.515),(1.955,1.515)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,1.855),(1.955,2.025),(2.125,2.025),(2.125,1.855),(1.955,1.855)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,2.195),(1.955,2.365),(2.125,2.365),(2.125,2.195),(1.955,2.195)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,2.535),(1.955,2.705),(2.125,2.705),(2.125,2.535),(1.955,2.535)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,2.875),(1.955,3.045),(2.125,3.045),(2.125,2.875),(1.955,2.875)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,3.215),(1.955,3.385),(2.125,3.385),(2.125,3.215),(1.955,3.215)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,3.555),(1.955,3.725),(2.125,3.725),(2.125,3.555),(1.955,3.555)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.275),(1.955,6.445),(2.125,6.445),(2.125,6.275),(1.955,6.275)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.615),(1.955,6.785),(2.125,6.785),(2.125,6.615),(1.955,6.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.955),(1.955,7.125),(2.125,7.125),(2.125,6.955),(1.955,6.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.295),(1.955,7.465),(2.125,7.465),(2.125,7.295),(1.955,7.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.635),(1.955,7.805),(2.125,7.805),(2.125,7.635),(1.955,7.635)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.975),(1.955,8.145),(2.125,8.145),(2.125,7.975),(1.955,7.975)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,8.315),(1.955,8.485),(2.125,8.485),(2.125,8.315),(1.955,8.315)),
        )
        createRL(
            tech, net, 'li',
            ((1.955,1.37),(1.955,6.13),(0.845,6.13),(0.845,8.63),(2.125,8.63),(2.125,1.37),(1.955,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.04), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.04), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int1']
        createRL(
            tech, net, 'difftap',
            ((1.345,1.37),(1.345,3.87),(1.75,3.87),(1.75,1.37),(1.345,1.37)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.195,1.24),(1.195,4.635),(1.0,4.635),(1.0,4.965),(1.195,4.965),(1.195,8.76),(1.345,8.76),(1.345,1.24),(1.195,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.05,4.715),(1.05,4.885),(1.22,4.885),(1.22,4.715),(1.05,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.05,1.37),(1.05,5.96),(1.22,5.96),(1.22,1.37),(1.05,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.135), u(0.17), u(1.37), u(5.96),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.135), u(0.17), u(1.37), u(5.96),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((1.75,1.24),(1.75,4.635),(1.555,4.635),(1.555,4.965),(1.75,4.965),(1.75,8.76),(1.9,8.76),(1.9,1.24),(1.75,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.605,4.715),(1.605,4.885),(1.775,4.885),(1.775,4.715),(1.605,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.605,1.37),(1.605,5.96),(1.775,5.96),(1.775,1.37),(1.605,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.69), u(0.17), u(1.37), u(5.96),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.69), u(0.17), u(1.37), u(5.96),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)

    cell = new_cells['nand4_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'i3': Net.create(cell, 'i3'),
            'int0': Net.create(cell, 'int0'),
            'int1': Net.create(cell, 'int1'),
            'int2': Net.create(cell, 'int2'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((1.75,1.37),(1.75,3.87),(1.9,3.87),(1.9,1.37),(1.75,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.305,1.37),(2.305,3.87),(2.455,3.87),(2.455,1.37),(2.305,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.83),(0.315,3.87),(0.79,3.87),(0.79,1.37),(0.565,1.37),(0.565,0.83),(0.315,0.83)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.195,1.37),(1.195,3.87),(1.345,3.87),(1.345,1.37),(1.195,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,0.95),(0.355,1.12),(0.525,1.12),(0.525,0.95),(0.355,0.95)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.705),(0.19,3.995),(2.845,3.995),(2.845,1.245),(0.69,1.245),(0.69,0.705),(0.19,0.705)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,6.005),(0.19,9.295),(0.69,9.295),(0.69,8.755),(1.295,8.755),(1.295,9.295),(1.795,9.295),(1.795,8.755),(2.405,8.755),(2.405,9.295),(2.905,9.295),(2.905,6.005),(0.19,6.005)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((0.315,6.13),(0.315,9.17),(0.565,9.17),(0.565,8.63),(0.79,8.63),(0.79,6.13),(0.315,6.13)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.195,6.13),(1.195,8.63),(1.42,8.63),(1.42,9.17),(1.67,9.17),(1.67,8.63),(1.9,8.63),(1.9,6.13),(1.195,6.13)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.305,6.13),(2.305,8.63),(2.53,8.63),(2.53,9.17),(2.78,9.17),(2.78,6.13),(2.305,6.13)),
        )
        createRL(
            tech, net, 'licon',
            ((1.46,8.88),(1.46,9.05),(1.63,9.05),(1.63,8.88),(1.46,8.88)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.88),(0.355,9.05),(0.525,9.05),(0.525,8.88),(0.355,8.88)),
        )
        createRL(
            tech, net, 'licon',
            ((2.57,8.88),(2.57,9.05),(2.74,9.05),(2.74,8.88),(2.57,8.88)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(4.18,10.18),(4.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int0']
        createRL(
            tech, net, 'difftap',
            ((0.79,1.37),(0.79,3.87),(1.195,3.87),(1.195,1.37),(0.79,1.37)),
        )
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.64,1.24),(0.64,4.635),(0.445,4.635),(0.445,4.965),(0.64,4.965),(0.64,8.76),(0.79,8.76),(0.79,1.24),(0.64,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((0.495,4.715),(0.495,4.885),(0.665,4.885),(0.665,4.715),(0.495,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((0.495,1.37),(0.495,8.63),(0.665,8.63),(0.665,1.37),(0.495,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((2.455,1.37),(2.455,3.87),(2.72,3.87),(2.72,1.37),(2.455,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,6.13),(0.79,8.63),(1.195,8.63),(1.195,6.13),(0.79,6.13)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.9,6.13),(1.9,8.63),(2.305,8.63),(2.305,6.13),(1.9,6.13)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.275),(0.845,6.445),(1.015,6.445),(1.015,6.275),(0.845,6.275)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.615),(0.845,6.785),(1.015,6.785),(1.015,6.615),(0.845,6.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,6.955),(0.845,7.125),(1.015,7.125),(1.015,6.955),(0.845,6.955)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.295),(0.845,7.465),(1.015,7.465),(1.015,7.295),(0.845,7.295)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.635),(0.845,7.805),(1.015,7.805),(1.015,7.635),(0.845,7.635)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,7.975),(0.845,8.145),(1.015,8.145),(1.015,7.975),(0.845,7.975)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,8.315),(0.845,8.485),(1.015,8.485),(1.015,8.315),(0.845,8.315)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,1.515),(2.51,1.685),(2.68,1.685),(2.68,1.515),(2.51,1.515)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,1.855),(2.51,2.025),(2.68,2.025),(2.68,1.855),(2.51,1.855)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,2.195),(2.51,2.365),(2.68,2.365),(2.68,2.195),(2.51,2.195)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,2.535),(2.51,2.705),(2.68,2.705),(2.68,2.535),(2.51,2.535)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,2.875),(2.51,3.045),(2.68,3.045),(2.68,2.875),(2.51,2.875)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,3.215),(2.51,3.385),(2.68,3.385),(2.68,3.215),(2.51,3.215)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,3.555),(2.51,3.725),(2.68,3.725),(2.68,3.555),(2.51,3.555)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.275),(1.955,6.445),(2.125,6.445),(2.125,6.275),(1.955,6.275)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.615),(1.955,6.785),(2.125,6.785),(2.125,6.615),(1.955,6.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.955),(1.955,7.125),(2.125,7.125),(2.125,6.955),(1.955,6.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.295),(1.955,7.465),(2.125,7.465),(2.125,7.295),(1.955,7.295)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.635),(1.955,7.805),(2.125,7.805),(2.125,7.635),(1.955,7.635)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.975),(1.955,8.145),(2.125,8.145),(2.125,7.975),(1.955,7.975)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,8.315),(1.955,8.485),(2.125,8.485),(2.125,8.315),(1.955,8.315)),
        )
        createRL(
            tech, net, 'li',
            ((2.51,1.37),(2.51,6.13),(0.845,6.13),(0.845,8.63),(2.68,8.63),(2.68,1.37),(2.51,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.595), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.595), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int1']
        createRL(
            tech, net, 'difftap',
            ((1.345,1.37),(1.345,3.87),(1.75,3.87),(1.75,1.37),(1.345,1.37)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.195,1.24),(1.195,4.635),(1.0,4.635),(1.0,4.965),(1.195,4.965),(1.195,8.76),(1.345,8.76),(1.345,1.24),(1.195,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.05,4.715),(1.05,4.885),(1.22,4.885),(1.22,4.715),(1.05,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.05,1.37),(1.05,5.96),(1.22,5.96),(1.22,1.37),(1.05,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.135), u(0.17), u(1.37), u(5.96),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.135), u(0.17), u(1.37), u(5.96),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int2']
        createRL(
            tech, net, 'difftap',
            ((1.9,1.37),(1.9,3.87),(2.305,3.87),(2.305,1.37),(1.9,1.37)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((1.75,1.24),(1.75,4.635),(1.555,4.635),(1.555,4.965),(1.75,4.965),(1.75,8.76),(1.9,8.76),(1.9,1.24),(1.75,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.605,4.715),(1.605,4.885),(1.775,4.885),(1.775,4.715),(1.605,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.605,1.37),(1.605,5.96),(1.775,5.96),(1.775,1.37),(1.605,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.69), u(0.17), u(1.37), u(5.96),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.69), u(0.17), u(1.37), u(5.96),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i3']
        createRL(
            tech, net, 'poly',
            ((2.305,1.24),(2.305,4.635),(2.11,4.635),(2.11,4.965),(2.305,4.965),(2.305,8.76),(2.455,8.76),(2.455,1.24),(2.305,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((2.16,4.715),(2.16,4.885),(2.33,4.885),(2.33,4.715),(2.16,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((2.16,1.37),(2.16,5.96),(2.33,5.96),(2.33,1.37),(2.16,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.245), u(0.17), u(1.37), u(5.96),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.245), u(0.17), u(1.37), u(5.96),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)
        nets['i3'].setDirection(Net.Direction.IN)

    cell = new_cells['nor2_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(2.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'int0': Net.create(cell, 'int0'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((1.42,0.83),(1.42,1.37),(1.195,1.37),(1.195,2.37),(1.67,2.37),(1.67,0.83),(1.42,0.83)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.83),(0.315,2.37),(0.79,2.37),(0.79,1.37),(0.565,1.37),(0.565,0.83),(0.315,0.83)),
        )
        createRL(
            tech, net, 'licon',
            ((1.46,0.95),(1.46,1.12),(1.63,1.12),(1.63,0.95),(1.46,0.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,0.95),(0.355,1.12),(0.525,1.12),(0.525,0.95),(0.355,0.95)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(2.0,1.2),(2.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.705),(0.19,2.495),(1.795,2.495),(1.795,0.705),(1.295,0.705),(1.295,1.245),(0.69,1.245),(0.69,0.705),(0.19,0.705)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.505),(0.19,9.295),(0.69,9.295),(0.69,8.755),(1.735,8.755),(1.735,5.505),(0.19,5.505)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((0.315,5.63),(0.315,9.17),(0.565,9.17),(0.565,8.63),(0.79,8.63),(0.79,5.63),(0.315,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.195,5.63),(1.195,8.63),(1.345,8.63),(1.345,5.63),(1.195,5.63)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.88),(0.355,9.05),(0.525,9.05),(0.525,8.88),(0.355,8.88)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(2.0,10.0),(2.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(2.18,10.18),(2.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.0), u(2.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((1.345,5.63),(1.345,8.63),(1.61,8.63),(1.61,5.63),(1.345,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,1.37),(0.79,2.37),(1.195,2.37),(1.195,1.37),(0.79,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.615),(0.845,1.785),(1.015,1.785),(1.015,1.615),(0.845,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.955),(0.845,2.125),(1.015,2.125),(1.015,1.955),(0.845,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,5.855),(1.4,6.025),(1.57,6.025),(1.57,5.855),(1.4,5.855)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,6.195),(1.4,6.365),(1.57,6.365),(1.57,6.195),(1.4,6.195)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,6.535),(1.4,6.705),(1.57,6.705),(1.57,6.535),(1.4,6.535)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,6.875),(1.4,7.045),(1.57,7.045),(1.57,6.875),(1.4,6.875)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,7.215),(1.4,7.385),(1.57,7.385),(1.57,7.215),(1.4,7.215)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,7.555),(1.4,7.725),(1.57,7.725),(1.57,7.555),(1.4,7.555)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,7.895),(1.4,8.065),(1.57,8.065),(1.57,7.895),(1.4,7.895)),
        )
        createRL(
            tech, net, 'licon',
            ((1.4,8.235),(1.4,8.405),(1.57,8.405),(1.57,8.235),(1.4,8.235)),
        )
        createRL(
            tech, net, 'li',
            ((0.845,1.37),(0.845,2.37),(1.4,2.37),(1.4,8.63),(1.57,8.63),(1.57,1.37),(0.845,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.485), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.485), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.64,1.24),(0.64,4.635),(0.445,4.635),(0.445,4.965),(0.64,4.965),(0.64,8.76),(0.79,8.76),(0.79,1.24),(0.64,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((0.495,4.715),(0.495,4.885),(0.665,4.885),(0.665,4.715),(0.495,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((0.495,1.37),(0.495,8.63),(0.665,8.63),(0.665,1.37),(0.495,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int0']
        createRL(
            tech, net, 'difftap',
            ((0.79,5.63),(0.79,8.63),(1.195,8.63),(1.195,5.63),(0.79,5.63)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.195,1.24),(1.195,4.635),(1.0,4.635),(1.0,4.965),(1.195,4.965),(1.195,8.76),(1.345,8.76),(1.345,1.24),(1.195,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.05,4.715),(1.05,4.885),(1.22,4.885),(1.22,4.715),(1.05,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.05,2.54),(1.05,8.63),(1.22,8.63),(1.22,2.54),(1.05,2.54)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.135), u(0.17), u(2.54), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.135), u(0.17), u(2.54), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)

    cell = new_cells['nor3_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(3.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'int0': Net.create(cell, 'int0'),
            'int1': Net.create(cell, 'int1'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((1.42,0.83),(1.42,1.37),(1.195,1.37),(1.195,2.37),(1.9,2.37),(1.9,1.37),(1.67,1.37),(1.67,0.83),(1.42,0.83)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,0.83),(0.315,2.37),(0.79,2.37),(0.79,1.37),(0.565,1.37),(0.565,0.83),(0.315,0.83)),
        )
        createRL(
            tech, net, 'licon',
            ((1.46,0.95),(1.46,1.12),(1.63,1.12),(1.63,0.95),(1.46,0.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,0.95),(0.355,1.12),(0.525,1.12),(0.525,0.95),(0.355,0.95)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(3.0,1.2),(3.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.705),(0.19,2.495),(2.29,2.495),(2.29,1.245),(1.795,1.245),(1.795,0.705),(1.295,0.705),(1.295,1.245),(0.69,1.245),(0.69,0.705),(0.19,0.705)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.505),(0.19,9.295),(0.69,9.295),(0.69,8.755),(2.29,8.755),(2.29,5.505),(0.19,5.505)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.75,5.63),(1.75,8.63),(1.9,8.63),(1.9,5.63),(1.75,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,5.63),(0.315,9.17),(0.565,9.17),(0.565,8.63),(0.79,8.63),(0.79,5.63),(0.315,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.195,5.63),(1.195,8.63),(1.345,8.63),(1.345,5.63),(1.195,5.63)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.88),(0.355,9.05),(0.525,9.05),(0.525,8.88),(0.355,8.88)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(3.0,10.0),(3.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(3.18,10.18),(3.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.5), u(3.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((0.79,1.37),(0.79,2.37),(1.195,2.37),(1.195,1.37),(0.79,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.9,1.37),(1.9,2.37),(2.165,2.37),(2.165,1.37),(1.9,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.9,5.63),(1.9,8.63),(2.165,8.63),(2.165,5.63),(1.9,5.63)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,1.615),(1.955,1.785),(2.125,1.785),(2.125,1.615),(1.955,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,1.955),(1.955,2.125),(2.125,2.125),(2.125,1.955),(1.955,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.615),(0.845,1.785),(1.015,1.785),(1.015,1.615),(0.845,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.955),(0.845,2.125),(1.015,2.125),(1.015,1.955),(0.845,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,5.855),(1.955,6.025),(2.125,6.025),(2.125,5.855),(1.955,5.855)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.195),(1.955,6.365),(2.125,6.365),(2.125,6.195),(1.955,6.195)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.535),(1.955,6.705),(2.125,6.705),(2.125,6.535),(1.955,6.535)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,6.875),(1.955,7.045),(2.125,7.045),(2.125,6.875),(1.955,6.875)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.215),(1.955,7.385),(2.125,7.385),(2.125,7.215),(1.955,7.215)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.555),(1.955,7.725),(2.125,7.725),(2.125,7.555),(1.955,7.555)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,7.895),(1.955,8.065),(2.125,8.065),(2.125,7.895),(1.955,7.895)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,8.235),(1.955,8.405),(2.125,8.405),(2.125,8.235),(1.955,8.235)),
        )
        createRL(
            tech, net, 'li',
            ((0.845,1.37),(0.845,2.37),(1.955,2.37),(1.955,8.63),(2.125,8.63),(2.125,1.37),(0.845,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.04), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.04), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.64,1.24),(0.64,4.635),(0.445,4.635),(0.445,4.965),(0.64,4.965),(0.64,8.76),(0.79,8.76),(0.79,1.24),(0.64,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((0.495,4.715),(0.495,4.885),(0.665,4.885),(0.665,4.715),(0.495,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((0.495,1.37),(0.495,8.63),(0.665,8.63),(0.665,1.37),(0.495,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int0']
        createRL(
            tech, net, 'difftap',
            ((0.79,5.63),(0.79,8.63),(1.195,8.63),(1.195,5.63),(0.79,5.63)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.195,1.24),(1.195,4.635),(1.0,4.635),(1.0,4.965),(1.195,4.965),(1.195,8.76),(1.345,8.76),(1.345,1.24),(1.195,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.05,4.715),(1.05,4.885),(1.22,4.885),(1.22,4.715),(1.05,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.05,2.54),(1.05,8.63),(1.22,8.63),(1.22,2.54),(1.05,2.54)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.135), u(0.17), u(2.54), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.135), u(0.17), u(2.54), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int1']
        createRL(
            tech, net, 'difftap',
            ((1.345,5.63),(1.345,8.63),(1.75,8.63),(1.75,5.63),(1.345,5.63)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((1.75,1.24),(1.75,4.635),(1.555,4.635),(1.555,4.965),(1.75,4.965),(1.75,8.76),(1.9,8.76),(1.9,1.24),(1.75,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.605,4.715),(1.605,4.885),(1.775,4.885),(1.775,4.715),(1.605,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.605,2.54),(1.605,8.63),(1.775,8.63),(1.775,2.54),(1.605,2.54)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.69), u(0.17), u(2.54), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.69), u(0.17), u(2.54), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)

    cell = new_cells['nor4_x0']
    with UpdateSession():
        cell.setAbutmentBox(Box(
            u(0.0), u(0.0), u(4.0), u(10.0),
        ))
        nets = {
            '*': Net.create(cell, '*'),
            'i0': Net.create(cell, 'i0'),
            'i1': Net.create(cell, 'i1'),
            'i2': Net.create(cell, 'i2'),
            'i3': Net.create(cell, 'i3'),
            'int0': Net.create(cell, 'int0'),
            'int1': Net.create(cell, 'int1'),
            'int2': Net.create(cell, 'int2'),
            'nq': Net.create(cell, 'nq'),
            'vdd': Net.create(cell, 'vdd'),
            'vss': Net.create(cell, 'vss'),
        }
        net = nets['vss']
        createRL(
            tech, net, 'difftap',
            ((0.315,0.83),(0.315,2.37),(0.79,2.37),(0.79,1.37),(0.565,1.37),(0.565,0.83),(0.315,0.83)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.53,0.83),(2.53,1.37),(2.305,1.37),(2.305,2.37),(2.78,2.37),(2.78,0.83),(2.53,0.83)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.42,0.83),(1.42,1.37),(1.195,1.37),(1.195,2.37),(1.9,2.37),(1.9,1.37),(1.67,1.37),(1.67,0.83),(1.42,0.83)),
        )
        createRL(
            tech, net, 'licon',
            ((1.46,0.95),(1.46,1.12),(1.63,1.12),(1.63,0.95),(1.46,0.95)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,0.95),(0.355,1.12),(0.525,1.12),(0.525,0.95),(0.355,0.95)),
        )
        createRL(
            tech, net, 'licon',
            ((2.57,0.95),(2.57,1.12),(2.74,1.12),(2.74,0.95),(2.57,0.95)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,0.0),(0.0,1.2),(4.0,1.2),(4.0,0.0),(0.0,0.0)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(0.0), u(1.2),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['*']
        createRL(
            tech, net, 'nsdm',
            ((0.19,0.705),(0.19,2.495),(2.905,2.495),(2.905,0.705),(2.405,0.705),(2.405,1.245),(1.795,1.245),(1.795,0.705),(1.295,0.705),(1.295,1.245),(0.69,1.245),(0.69,0.705),(0.19,0.705)),
        )
        createRL(
            tech, net, 'psdm',
            ((0.19,5.505),(0.19,9.295),(0.69,9.295),(0.69,8.755),(2.845,8.755),(2.845,5.505),(0.19,5.505)),
        )
        net = nets['vdd']
        createRL(
            tech, net, 'difftap',
            ((1.75,5.63),(1.75,8.63),(1.9,8.63),(1.9,5.63),(1.75,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.305,5.63),(2.305,8.63),(2.455,8.63),(2.455,5.63),(2.305,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.315,5.63),(0.315,9.17),(0.565,9.17),(0.565,8.63),(0.79,8.63),(0.79,5.63),(0.315,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((1.195,5.63),(1.195,8.63),(1.345,8.63),(1.345,5.63),(1.195,5.63)),
        )
        createRL(
            tech, net, 'licon',
            ((0.355,8.88),(0.355,9.05),(0.525,9.05),(0.525,8.88),(0.355,8.88)),
        )
        createRL(
            tech, net, 'li',
            ((0.0,8.8),(0.0,10.0),(4.0,10.0),(4.0,8.8),(0.0,8.8)),
        )
        createRL(
            tech, net, 'nwm',
            ((-0.18,4.8),(-0.18,10.18),(4.18,10.18),(4.18,4.8),(-0.18,4.8)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.0), u(4.0), u(8.8), u(10.0),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['nq']
        createRL(
            tech, net, 'difftap',
            ((1.9,1.37),(1.9,2.37),(2.305,2.37),(2.305,1.37),(1.9,1.37)),
        )
        createRL(
            tech, net, 'difftap',
            ((2.455,5.63),(2.455,8.63),(2.72,8.63),(2.72,5.63),(2.455,5.63)),
        )
        createRL(
            tech, net, 'difftap',
            ((0.79,1.37),(0.79,2.37),(1.195,2.37),(1.195,1.37),(0.79,1.37)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,1.615),(1.955,1.785),(2.125,1.785),(2.125,1.615),(1.955,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((1.955,1.955),(1.955,2.125),(2.125,2.125),(2.125,1.955),(1.955,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.615),(0.845,1.785),(1.015,1.785),(1.015,1.615),(0.845,1.615)),
        )
        createRL(
            tech, net, 'licon',
            ((0.845,1.955),(0.845,2.125),(1.015,2.125),(1.015,1.955),(0.845,1.955)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,5.855),(2.51,6.025),(2.68,6.025),(2.68,5.855),(2.51,5.855)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,6.195),(2.51,6.365),(2.68,6.365),(2.68,6.195),(2.51,6.195)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,6.535),(2.51,6.705),(2.68,6.705),(2.68,6.535),(2.51,6.535)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,6.875),(2.51,7.045),(2.68,7.045),(2.68,6.875),(2.51,6.875)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,7.215),(2.51,7.385),(2.68,7.385),(2.68,7.215),(2.51,7.215)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,7.555),(2.51,7.725),(2.68,7.725),(2.68,7.555),(2.51,7.555)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,7.895),(2.51,8.065),(2.68,8.065),(2.68,7.895),(2.51,7.895)),
        )
        createRL(
            tech, net, 'licon',
            ((2.51,8.235),(2.51,8.405),(2.68,8.405),(2.68,8.235),(2.51,8.235)),
        )
        createRL(
            tech, net, 'li',
            ((0.845,1.37),(0.845,2.37),(2.51,2.37),(2.51,8.63),(2.68,8.63),(2.68,1.37),(0.845,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.595), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.595), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['i0']
        createRL(
            tech, net, 'poly',
            ((0.64,1.24),(0.64,4.635),(0.445,4.635),(0.445,4.965),(0.64,4.965),(0.64,8.76),(0.79,8.76),(0.79,1.24),(0.64,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((0.495,4.715),(0.495,4.885),(0.665,4.885),(0.665,4.715),(0.495,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((0.495,1.37),(0.495,8.63),(0.665,8.63),(0.665,1.37),(0.495,1.37)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(0.58), u(0.17), u(1.37), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int0']
        createRL(
            tech, net, 'difftap',
            ((0.79,5.63),(0.79,8.63),(1.195,8.63),(1.195,5.63),(0.79,5.63)),
        )
        net = nets['i1']
        createRL(
            tech, net, 'poly',
            ((1.195,1.24),(1.195,4.635),(1.0,4.635),(1.0,4.965),(1.195,4.965),(1.195,8.76),(1.345,8.76),(1.345,1.24),(1.195,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.05,4.715),(1.05,4.885),(1.22,4.885),(1.22,4.715),(1.05,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.05,2.54),(1.05,8.63),(1.22,8.63),(1.22,2.54),(1.05,2.54)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.135), u(0.17), u(2.54), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.135), u(0.17), u(2.54), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int1']
        createRL(
            tech, net, 'difftap',
            ((1.345,5.63),(1.345,8.63),(1.75,8.63),(1.75,5.63),(1.345,5.63)),
        )
        net = nets['i2']
        createRL(
            tech, net, 'poly',
            ((1.75,1.24),(1.75,4.635),(1.555,4.635),(1.555,4.965),(1.75,4.965),(1.75,8.76),(1.9,8.76),(1.9,1.24),(1.75,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((1.605,4.715),(1.605,4.885),(1.775,4.885),(1.775,4.715),(1.605,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((1.605,2.54),(1.605,8.63),(1.775,8.63),(1.775,2.54),(1.605,2.54)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(1.69), u(0.17), u(2.54), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(1.69), u(0.17), u(2.54), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        net = nets['int2']
        createRL(
            tech, net, 'difftap',
            ((1.9,5.63),(1.9,8.63),(2.305,8.63),(2.305,5.63),(1.9,5.63)),
        )
        net = nets['i3']
        createRL(
            tech, net, 'poly',
            ((2.305,1.24),(2.305,4.635),(2.11,4.635),(2.11,4.965),(2.305,4.965),(2.305,8.76),(2.455,8.76),(2.455,1.24),(2.305,1.24)),
        )
        createRL(
            tech, net, 'licon',
            ((2.16,4.715),(2.16,4.885),(2.33,4.885),(2.33,4.715),(2.16,4.715)),
        )
        createRL(
            tech, net, 'li',
            ((2.16,2.54),(2.16,8.63),(2.33,8.63),(2.33,2.54),(2.16,2.54)),
        )
        Vertical.create(
            net, tech.getLayer('li.pin'),
            u(2.245), u(0.17), u(2.54), u(8.63),
        )
        pin = Vertical.create(
            net, tech.getLayer('li'),
            u(2.245), u(0.17), u(2.54), u(8.63),
        )
        net.setExternal(True)
        NetExternalComponents.setExternal(pin)
        nets['vdd'].setGlobal(True)
        nets['vss'].setGlobal(True)
        nets['vdd'].setType(Net.Type.POWER)
        nets['vdd'].setDirection(Net.Direction.IN)
        nets['vss'].setType(Net.Type.GROUND)
        nets['vss'].setDirection(Net.Direction.IN)
        nets['nq'].setDirection(Net.Direction.OUT)
        nets['i0'].setDirection(Net.Direction.IN)
        nets['i1'].setDirection(Net.Direction.IN)
        nets['i2'].setDirection(Net.Direction.IN)
        nets['i3'].setDirection(Net.Direction.IN)

    af.wrapLibrary(lib, 0)

    return lib

def setup():
    lib = _load()
    _routing()
    try:
        from .StdCellLib_fix import fix
    except:
        pass
    else:
        fix(lib)

    return lib
